Package: interplot
Title: Plot the Effects of Variables in Interaction Terms
Version: 1.2.0
Authors@R: c(
    person("Frederick", "Solt", email = "frederick-solt@uiowa.edu", role = c("aut")),
    person("Yue", "Hu", email = "yuehu@tsinghua.edu.cn", role = c("aut", "cre")),
    person("Brenton", "Kenkel", email = "brenton.kenkel@gmail.com", role = "ctb"))
Maintainer: Yue Hu <yuehu@tsinghua.edu.cn>
Description: Plots the conditional coefficients ("marginal effects") of
    variables included in multiplicative interaction terms.
BugReports: https://github.com/sammo3182/interplot/issues
Encoding: UTF-8
Depends: R (>= 4.1.0), ggplot2
Imports: stats, abind, arm, dplyr, purrr, lme4, interactionTest
License: MIT + file LICENSE
Suggests: knitr, rmarkdown, mitools, gridExtra, merTools, brms,
        splines, testthat (>= 3.0.0)
VignetteBuilder: knitr
RoxygenNote: 8.0.0
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-06-24 09:47:09 UTC; yuehu
Author: Frederick Solt [aut],
  Yue Hu [aut, cre],
  Brenton Kenkel [ctb]
Repository: CRAN
Date/Publication: 2026-06-24 10:10:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(interplot,default)
S3method(interplot,glmerMod)
S3method(interplot,glmmi)
S3method(interplot,gmlmmi)
S3method(interplot,lmerMod)
S3method(interplot,lmmi)
S3method(interplot,mlmmi)
S3method(interplot,plot)
export(interplot)
import(dplyr)
import(ggplot2)
importFrom(abind,abind)
importFrom(arm,sim)
importFrom(dplyr,mutate)
importFrom(graphics,hist)
importFrom(interactionTest,fdrInteraction)
importFrom(purrr,map)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,plogis)
importFrom(stats,qnorm)
importFrom(stats,quantile)

pkgdown <img src="https://user-images.githubusercontent.com/58319029/112086636-891c0600-8bc7-11eb-8ab3-9737dd1483d1.png" width = "134.435" height = "155.25" align="right" />

[![CRAN version](http://www.r-pkg.org/badges/version/interplot)](https://cran.r-project.org/package=interplot) ![](http://cranlogs.r-pkg.org/badges/grand-total/interplot)

------------------------------------------------------------------------
interplot
=========

`interplot` is a tool for plotting the conditional coefficients ("marginal effects") of variables included in multiplicative interaction terms. The function plots the changes in the coefficient of one variable in a two-way interaction term conditional on the value of the other included variable. The plot also includes simulated 95% confidential intervals of these coefficients.

To install:

* the latest released version: `install.packages("interplot")`.
* the latest developing version: `remotes::install_github("sammo3182/interplot")`.


More details are available at:

http://cran.r-project.org/web/packages/interplot/vignettes/interplot-vignette.html


Please note that this project is released with a [Contributor Code of Conduct](https://github.com/sammo3182/interplot/blob/master/CONDUCT.md). By participating in this project you agree to abide by its terms.

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  message = FALSE,
  warning = FALSE,
  dpi = 300
)

## -----------------------------------------------------------------------------
data(mtcars)  #load the data

## -----------------------------------------------------------------------------
m_cyl <- lm(mpg ~ wt * cyl, data = mtcars)
summary(m_cyl)

## ----out.width="80%"----------------------------------------------------------
library(interplot)

interplot(m = m_cyl, var1 = "cyl", var2 = "wt")

## ----out.width="80%"----------------------------------------------------------
interplot(m = m_cyl, var1 = "wt", var2 = "cyl")

## ----out.width="80%"----------------------------------------------------------
interplot(m = m_cyl, var1 = "wt", var2 = "cyl", ci = .9, point = T)

## ----out.width="80%"----------------------------------------------------------
interplot(m = m_cyl, var1 = "cyl", var2 = "wt", point = T) +
  # changing the angle of x labels for a clearer vision
  theme(axis.text.x  = element_text(angle=90))

## ----out.width="80%"----------------------------------------------------------
interplot(m = m_cyl, var1 = "cyl", var2 = "wt") + 
  # Add labels for X and Y axes
    xlab("Automobile Weight (thousands lbs)") +
    ylab("Estimated Coefficient for\nNumber of Cylinders") +
  # Change the background
    theme_bw() +
  # Add the title
    ggtitle("Estimated Coefficient of Engine Cylinders \non Mileage by Automobile Weight") +
    theme(plot.title = element_text(face="bold")) +
  # Add a horizontal line at y = 0
    geom_hline(yintercept = 0, linetype = "dashed")

## ----out.width="80%"----------------------------------------------------------
interplot(m = m_cyl, var1 = "wt", var2 = "cyl", ercolor = "blue", esize = 1.5) +
  geom_point(size = 2, color = "red")

## ----out.width="80%"----------------------------------------------------------
m_wt <- lm(mpg ~ wt + I(wt^2), data = mtcars)

interplot(m = m_wt, var1 = "wt", var2 = "wt")

## ----out.width="80%"----------------------------------------------------------
mtcars$gear <- factor(mtcars$gear)
m_gear <- lm(mpg ~ gear * wt, data = mtcars)

interplot(m = m_gear, var1 = "wt", var2 = "gear")

## ----out.width="80%"----------------------------------------------------------
interplot(m = m_gear, var1 = "wt", var2 = "gear", facet_labs = c("4-speed", "5-speed"))

## ----out.width="80%"----------------------------------------------------------
interplot(m = m_cyl, var1 = "cyl", var2 = "wt", hist = TRUE) +
    geom_hline(yintercept = 0, linetype = "dashed")

## ----out.width="80%"----------------------------------------------------------
interplot(m = m_cyl, var1 = "cyl", var2 = "wt", hist = TRUE) +
  aes(color = "pink") + theme(legend.position="none") +  # geom_line(color = "pink") + 
  geom_hline(yintercept = 0, linetype = "dashed")

## ----out.width="80%"----------------------------------------------------------
set.seed(313)

interplot(m = m_cyl, var1 = "cyl", var2 = "wt", stats_cp = "ci")
interplot(m = m_cyl, var1 = "cyl", var2 = "wt", stats_cp = "ks")

## ----out.width="80%"----------------------------------------------------------
set.seed(313)

interplot(m = m_cyl, var1 = "cyl", var2 = "wt", stats_cp = "ci", txt_caption = "\n Source: Motor Trend Car Road Tests 1973.")

## ----out.width="80%"----------------------------------------------------------
stdCI_plot <- interplot(m = m_wt, var1 = "wt", var2 = "wt", adjCI = FALSE) +
ggtitle("Marginal Effects with Standard CIs")
adjCI_plot <- interplot(m = m_wt, var1 = "wt", var2 = "wt", adjCI = TRUE) +
ggtitle("Marginal Effects with Adjusted CIs")

library(gridExtra)
grid.arrange(stdCI_plot, adjCI_plot, ncol = 2)

## ----out.width="80%"----------------------------------------------------------
pew1.w <- read.csv("pew1_w.csv")

m <- glm(formula=meritocracy~ginicnty+income_i+ginicnty:income_i+income_cnty+black_cnty+
                perc_bush04+pop_cnty+educ_i+age_i+gender_i+unemp_i+union_i+partyid_i+
                ideo_i+attend_i+survid2006+survid2007+survid2009,
              data=pew1.w,family=binomial(link="logit"))
              
plot_avg <- interplot(m, var1 = "ginicnty",var2 = "income_i", predPro = FALSE) + 
ggtitle("Average Conditional Effects")

plot_3val <- interplot(m, var1 = "ginicnty",var2 = "income_i", predPro = TRUE, var2_vals = c(min(pew1.w$income_i), max(pew1.w$income_i))) +
ggtitle("Conditional Predicted Probabilities for \nCitizens with Low and High Incomes") +
scale_colour_discrete(guide = guide_legend(title = "Income"), labels = c("Low", "High")) + 
scale_fill_discrete(guide = guide_legend(title = "Income"), labels = c("Low", "High")) +
theme(legend.position = c(0, .8), legend.justification = c(0, .5))

grid.arrange(plot_avg, plot_3val, ncol = 2)

## ----out.width="80%"----------------------------------------------------------
# Create a fake dataset of conditional effects
fake <- rnorm(100, 0, 1)
coef1 <- fake * sample(.5:2.5, 100, replace = T)
lb <- coef1 - .5
ub <- coef1 + .5

df_fake <- data.frame(cbind(fake, coef1, lb, ub))

# Use interplot directly with the dataset
interplot(df_fake)

## ----out.width="80%"----------------------------------------------------------
var2_fake <- fake
# Set `hist` to TRUE is required to superimpose a histogram.
interplot(df_fake, hist = TRUE, var2_dt = var2_fake)
## The ribbon and histogram do not fit. This is just an illustration

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  message = FALSE,
  warning = FALSE,
  fig.width = 6,
  fig.height = 4,
  out.width = "100%",
  dpi = 120
)

## -----------------------------------------------------------------------------
data(mtcars)  #load the data

## -----------------------------------------------------------------------------
m_cyl <- lm(mpg ~ wt * cyl, data = mtcars)
summary(m_cyl)

## ----out.width="80%"----------------------------------------------------------
library(interplot)

interplot(m = m_cyl, var1 = "cyl", var2 = "wt")

## ----out.width="80%"----------------------------------------------------------
interplot(m = m_cyl, var1 = "wt", var2 = "cyl")

## ----out.width="80%"----------------------------------------------------------
interplot(m = m_cyl, var1 = "wt", var2 = "cyl", ci = .9, point = T)

## ----out.width="80%"----------------------------------------------------------
interplot(m = m_cyl, var1 = "cyl", var2 = "wt", point = T) +
  # changing the angle of x labels for a clearer vision
  theme(axis.text.x  = element_text(angle=90))

## ----out.width="80%"----------------------------------------------------------
interplot(m = m_cyl, var1 = "cyl", var2 = "wt") + 
  # Add labels for X and Y axes
    xlab("Automobile Weight (thousands lbs)") +
    ylab("Estimated Coefficient for\nNumber of Cylinders") +
  # Change the background
    theme_bw() +
  # Add the title
    ggtitle("Estimated Coefficient of Engine Cylinders \non Mileage by Automobile Weight") +
    theme(plot.title = element_text(face="bold")) +
  # Add a horizontal line at y = 0
    geom_hline(yintercept = 0, linetype = "dashed")

## ----out.width="80%"----------------------------------------------------------
interplot(m = m_cyl, var1 = "wt", var2 = "cyl", ercolor = "blue", esize = 1.5) +
  geom_point(size = 2, color = "red")

## ----out.width="80%"----------------------------------------------------------
m_wt <- lm(mpg ~ wt + I(wt^2), data = mtcars)

interplot(m = m_wt, var1 = "wt", var2 = "wt")

## ----out.width="80%"----------------------------------------------------------
mtcars$gear <- factor(mtcars$gear)
m_gear <- lm(mpg ~ gear * wt, data = mtcars)

interplot(m = m_gear, var1 = "wt", var2 = "gear")

## ----out.width="80%"----------------------------------------------------------
interplot(m = m_gear, var1 = "wt", var2 = "gear", facet_labs = c("4-speed", "5-speed"))

## ----out.width="80%"----------------------------------------------------------
interplot(m = m_cyl, var1 = "cyl", var2 = "wt", hist = TRUE) +
    geom_hline(yintercept = 0, linetype = "dashed")

## ----out.width="80%"----------------------------------------------------------
interplot(m = m_cyl, var1 = "cyl", var2 = "wt", hist = TRUE) +
  aes(color = "pink") + theme(legend.position="none") +  # geom_line(color = "pink") + 
  geom_hline(yintercept = 0, linetype = "dashed")

## -----------------------------------------------------------------------------
set.seed(313)

interplot(m = m_cyl, var1 = "cyl", var2 = "wt", stats_cp = "ci")

## -----------------------------------------------------------------------------
set.seed(313)

interplot(m = m_cyl, var1 = "cyl", var2 = "wt", stats_cp = "ci", 
          txt_caption = "\n Source: Motor Trend Car Road Tests 1973.")

## ----fig.width=8, fig.height=3.5----------------------------------------------
stdCI_plot <- interplot(m = m_wt, var1 = "wt", var2 = "wt", adjCI = FALSE) +
ggtitle("Marginal Effects with Standard CIs")
adjCI_plot <- interplot(m = m_wt, var1 = "wt", var2 = "wt", adjCI = TRUE) +
ggtitle("Marginal Effects with Adjusted CIs")

library(gridExtra)
grid.arrange(stdCI_plot, adjCI_plot, ncol = 2)

## ----out.width="80%"----------------------------------------------------------
pew1.w <- read.csv("pew1_w.csv")

m <- glm(formula=meritocracy~ginicnty+income_i+ginicnty:income_i+income_cnty+black_cnty+
                perc_bush04+pop_cnty+educ_i+age_i+gender_i+unemp_i+union_i+partyid_i+
                ideo_i+attend_i+survid2006+survid2007+survid2009,
              data=pew1.w,family=binomial(link="logit"))
              
plot_avg <- interplot(m, var1 = "ginicnty",var2 = "income_i", predPro = FALSE) + 
ggtitle("Average Conditional Effects")

plot_3val <- interplot(m, var1 = "ginicnty",var2 = "income_i", predPro = TRUE, var2_vals = c(min(pew1.w$income_i), max(pew1.w$income_i))) +
ggtitle("Conditional Predicted Prob for \nLow and High Income Citizens") +
scale_colour_discrete(guide = guide_legend(title = "Income"), labels = c("Low", "High")) + 
scale_fill_discrete(guide = guide_legend(title = "Income"), labels = c("Low", "High")) +
theme(legend.position = c(0, .8), legend.justification = c(0, .5))

grid.arrange(plot_avg, plot_3val, ncol = 2)

## ----fig.width=7.5, fig.height=3----------------------------------------------
m_3way <- lm(mpg ~ wt * hp * cyl, data = mtcars)

# One small-multiple panel per value of cyl (the default)
interplot(m_3way, var1 = "wt", var2 = "hp", var3 = "cyl")

## ----message=FALSE------------------------------------------------------------
interplot(m_3way, var1 = "wt", var2 = "hp", var3 = "cyl", facet = FALSE)

## ----message=FALSE------------------------------------------------------------
# Effect of wt allowed to vary quadratically with cyl
m_nl <- lm(mpg ~ wt * (cyl + I(cyl^2)), data = mtcars)
interplot(m_nl, var1 = "wt", var2 = "cyl")

## ----message=FALSE------------------------------------------------------------
library(splines)
m_spline <- lm(mpg ~ wt * ns(hp, 3), data = mtcars)
interplot(m_spline, var1 = "wt", var2 = "hp")

## ----message=FALSE------------------------------------------------------------
m_lin <- lm(mpg ~ wt * cyl, data = mtcars)
interplot(m_lin, var1 = "wt", var2 = "cyl") +
  bin_layer(m_lin, var1 = "wt", var2 = "cyl")

## -----------------------------------------------------------------------------
m_cyl <- lm(mpg ~ wt * cyl, data = mtcars)
jn <- jn_interval(m_cyl, var1 = "cyl", var2 = "wt")
print(jn)

## ----message=FALSE------------------------------------------------------------
interplot(m_cyl, var1 = "cyl", var2 = "wt") +
  jn_layer(jn)

## ----out.width="80%"----------------------------------------------------------
# Create a fake dataset of conditional effects
fake <- rnorm(100, 0, 1)
coef1 <- fake * sample(.5:2.5, 100, replace = T)
lb <- coef1 - .5
ub <- coef1 + .5

df_fake <- data.frame(cbind(fake, coef1, lb, ub))

# Use interplot directly with the dataset
interplot(df_fake)

## ----out.width="80%"----------------------------------------------------------
var2_fake <- fake
# Set `hist` to TRUE is required to superimpose a histogram.
interplot(df_fake, hist = TRUE, var2_dt = var2_fake)
## The ribbon and histogram do not fit. This is just an illustration

if(getRversion() >= "2.15.1") utils::globalVariables(c("fake", "coef1", "value", "lb", "ub"))

#' Plot Conditional Coefficients in Models with Interaction Terms
#'
#' Graph based on the data frame of statistics about the conditional effect of an interaction.
#' 
#' @param m A model object including an interaction term, or, alternately, a data frame recording conditional coefficients. This data frame should includes four columns:
#' \itemize{
#'    \item fake: The sequence of \code{var1} (the item whose effect will be conditioned on in the interaction);
#'    \item coef1: The point estimates of the coefficient of \code{var1} at each break point.
#'    \item ub: The upper bound of the simulated 95\% CI.
#'    \item lb: The lower bound of the simulated 95\% CI.
#' }
#' @param var1 The name (as a string) of the variable of interest in the interaction term; its conditional coefficient estimates will be plotted.
#' @param var2 The name (as a string) of the other variable in the interaction term.
#' @param plot A logical value indicating whether the output is a plot or a dataframe including the conditional coefficient estimates of var1, their upper and lower bounds, and the corresponding values of var2.
#' @param steps Desired length of the sequence. A non-negative number, which for seq and seq.int will be rounded up if fractional. The default is 100 or the unique categories in the \code{var2} (when it is less than 100. Also see \code{\link{unique}}).
#' @param ci is a numeric value inherited from the data wrangling functions in this package. Adding it here is just for the method consistency.
#' @param adjCI Succeeded from the data management functions in `interplot` package. 
#' @param hist A logical value indicating if there is a histogram of `var2` added at the bottom of the conditional effect plot.
#' @param var2_dt A numerical value indicating the frequency distribution of `var2`. It is only used when `hist == TRUE`. When the object is a model, the default is the distribution of `var2` of the model. 
#' @param predPro A logical value with default of `FALSE`. When the `m` is an output of a general linear model (class `glm` or `glmerMod`) and the argument is set to `TRUE`, the function will plot predicted probabilities at the values given by `var2_vals`. 
#' @param var2_vals A numerical value indicating the values the predicted probabilities are estimated, when `predPro` is `TRUE`. 
#' @param point A logical value determining the format of plot. By default, the function produces a line plot when var2 takes on ten or more distinct values and a point (dot-and-whisker) plot otherwise; option TRUE forces a point plot.
#' @param sims Number of independent simulation draws used to calculate upper and lower bounds of coefficient estimates: lower values run faster; higher values produce smoother curves.
#' @param xmin A numerical value indicating the minimum value shown of x shown in the graph. Rarely used.
#' @param xmax A numerical value indicating the maximum value shown of x shown in the graph. Rarely used.
#' @param ercolor A character value indicating the outline color of the whisker or ribbon.
#' @param esize A numerical value indicating the size of the whisker or ribbon.
#' @param ralpha A numerical value indicating the transparency of the ribbon.
#' @param rfill A character value indicating the filling color of the ribbon.
#' @param ... Other ggplot aesthetics arguments for points in the dot-whisker plot or lines in the line-ribbon plots. Not currently used.
#' @param stats_cp A character value indicating what statistics to present as the plot note. Three options are available: "none", "ci", and "ks". The default is "none". See the Details for more information.
#' @param txt_caption A character string to add a note for the plot, a value will sending to \code{ggplot2::labs(caption = txt_caption))}.
#' @param ci_diff A numerical vector with a pair of values indicating the confidence intervals of the difference between \code{var1} and \code{var2}.
#' @param ks_diff A \code{ks.test} object of the effect of \code{var1} conditioned on \code{var2}.
#' @param overlay A logical value. When \code{TRUE}, the conditional-effect curves are colored by the grouping column \code{value} in a single panel (used by three-way interaction overlays). The default is \code{FALSE}.
#' 
#' @details \code{interplot.plot} is a S3 method from the \code{interplot}. It generates plots of conditional coefficients.
#' 
#' Because the output function is based on \code{\link[ggplot2]{ggplot}}, any additional arguments and layers supported by \code{ggplot2} can be added with the \code{+}. 
#' 
#' \code{interplot} visualizes the conditional effect based on simulated marginal effects. The simulation provides a probabilistic distribution of moderation effect of the conditioning variable (\code{var2}) at every preset values (including the minimum and maximum values) of the conditioned variable (\code{var1}), denoted as Emin and Emax. This output allows the function to further examine the conditional effect statistically in two ways. One is to examine if the distribution of \eqn{Emax - Emin} covers zero. The other is to directly compare Emin and Emax through statistical tools for distributional comparisons. Users can choose either method by setting the argument \code{stats_cp} to "ci" or "ks".
#' \itemize{
#'   \item "ci" provides the confidence interval of the difference of \eqn{Emax - Emin}. An interval including 0 suggests no statistical difference before and after the conditional effect is applied, and vise versa.
#'   \item "ks" presents the result of a two-sample Kolmogorov-Smirnov test of the simulated distributions of Emin and Emax. The output includes a D statistics and a p-value of the null hypothesis that the two distributions come from the same distribution at the 0.05 level.
#' }
#' 
#' See an illustration in the package vignette.
#' 
#' @return The function returns a \code{ggplot} object.
#' 
#' @import  ggplot2
#' @importFrom graphics hist
#' @importFrom dplyr mutate
#' 
#' 
#' @export

## S3 method for class 'data.frame'
interplot.plot <- function(m,
           var1 = NULL,
           var2 = NULL,
           plot = TRUE,
           steps = NULL,
           ci = .95,
           adjCI = FALSE,
           hist = FALSE,
           var2_dt = NULL,
           predPro = FALSE,
           var2_vals = NULL,
           point = FALSE,
           sims = 5000,
           xmin = NA,
           xmax = NA,
           ercolor = NA,
           esize = 0.5,
           ralpha = 0.5,
           rfill = "grey70",
           stats_cp = "none",
           txt_caption = NULL,
           ci_diff = NULL,
           ks_diff = NULL,
           overlay = FALSE,
           ...) {

  # `overlay` (three-way, facet = FALSE) reuses the colour = value / fill = value
  # aesthetics built for predPro, but keeps the conditional-effect y-axis label.
  grouped <- predPro == TRUE || overlay == TRUE

  required_cols <- c("fake", "coef1", "ub", "lb")
  missing_cols <- setdiff(required_cols, names(m))
  if (length(missing_cols) > 0)
    stop("Input data.frame must include columns: ",
         paste(required_cols, collapse = ", "),
         ". Missing: ", paste(missing_cols, collapse = ", "), ".")

  if (is.null(steps)) steps <- nrow(m)

  default_ylab <- if (predPro) {
    "Predicted Probability (%)"
  } else if (!is.null(var1)) {
    paste("Conditional Effect of", var1)
  } else {
    "Conditional Coefficient"
  }
  # In predPro mode the x-axis sweeps var1 (coloured by var2); otherwise it is var2.
  default_xlab <- if (predPro && !is.null(var1)) {
    var1
  } else if (!is.null(var2)) {
    var2
  } else {
    "Conditioning Variable"
  }

  levels <- sort(unique(m$fake))
  ymin <- ymax <- vector()
  xdiff <- vector()
  test_cp <- vector()
  
  if (grouped) {
    if (is.null(m$value))
      stop("The input data.frame does not include required information.")
  }
  
  if (hist == FALSE) {
    if (steps < 5 | point == TRUE) {
      if (is.na(ercolor)) ercolor <- "black"  # ensure whisker can be drawn
      if (grouped) {
        coef.plot <- ggplot(m, aes(
            x = fake,
            y = coef1,
            group = value,
            colour = value
          )) + 
          geom_point(position = position_dodge(width = .3),...) + 
          geom_errorbar(
            aes(
              ymin = lb,
              ymax = ub,
              colour = value
            ),
            width = 0,
            position = position_dodge(width = .3),
            linewidth = esize
          ) + 
          scale_x_continuous(breaks = levels) + ylab(default_ylab) + xlab(default_xlab)
      } else{
        coef.plot <-
          ggplot(m, aes(x = fake, 
                               y = coef1)) + geom_point(...) +
          geom_errorbar(
            aes(ymin = lb, 
                       ymax = ub),
            width = 0,
            color = ercolor,
            linewidth = esize
          ) + 
          scale_x_continuous(breaks = levels) + 
          ylab(default_ylab) + xlab(default_xlab)
      }
    } else {
      if (grouped) {
        coef.plot <-
          ggplot(m, aes(
            x = fake,
            y = coef1,
            colour = value
          )) + 
          geom_line(...) + 
          geom_ribbon(aes(
            ymin = lb,
            ymax = ub,
            fill = value
          ), alpha = ralpha) + 
          ylab(default_ylab) + xlab(default_xlab)
      } else {
        coef.plot <-
          ggplot(m, aes(x = fake, 
                        y = coef1)) + 
          geom_line(...) + 
          geom_ribbon(
            aes(ymin = lb, 
                ymax = ub),
            alpha = ralpha,
            color = ercolor,
            fill = rfill
          ) + 
          ylab(default_ylab) + xlab(default_xlab)
      }
    }
    
    
    if (stats_cp == "ci") {
      test_cp <-
        if (is.list(ci_diff)) {
          test_cp <- "" # not paste ci_diff here but in `facet_grid` in `interplot_default`
        } else {
          paste0("CI(Max - Min): [",
                 round(ci_diff[1], digits = 3),
                 ", ",
                 round(ci_diff[2], digits = 3),
                 "]")
        }
    } else if (stats_cp == "ks") {
      test_cp <-
        paste0("(p-ks = ", format(round(ks_diff$p.value, digits = 3), nsmall = 3), ")")
    }
    
    coef.plot <- coef.plot +
      labs(caption = paste0(test_cp, txt_caption))
    
    return(coef.plot)
  } else {
    if (point == TRUE) {
      if (is.na(ercolor))
        ercolor <- "black"  # ensure whisker can be drawn
      
      yrange <- c(m$ub, m$lb, var2_dt)
      maxdiff <- (max(yrange) - min(yrange))
      
      break_var2 <- steps + 1
      if (break_var2 >= 100)
        break_var2 <- 100
      hist.out <-
        hist(var2_dt,
             breaks = seq(min(var2_dt), max(var2_dt), l = break_var2),
             plot = FALSE)
      
      n.hist <- length(hist.out$mids)
      
      if (steps < 10) {
        dist <- (hist.out$mids[2] - hist.out$mids[1]) / 3
      } else {
        dist <- hist.out$mids[2] - hist.out$mids[1]
      }
      hist.max <- max(hist.out$counts)
      
      if (steps < 10) {
        histX <- data.frame(
          ymin = rep(min(yrange) - maxdiff / 5, n.hist),
          ymax = hist.out$counts / hist.max * maxdiff /
            5 + min(yrange) - maxdiff / 5,
          xmin = sort(unique(var2_dt)) - dist / 2,
          xmax = sort(unique(var2_dt)) + dist / 2
        )
      } else {
        histX <- data.frame(
          ymin = rep(min(yrange) - maxdiff / 5, n.hist),
          ymax = hist.out$counts / hist.max * maxdiff /
            5 + min(yrange) - maxdiff / 5,
          xmin = hist.out$mids - dist / 2,
          xmax = hist.out$mids + dist / 2
        )
      }
      #when up to 10, the sort(unique(var2_dt)) - dist/2 leads to problemtic histogram
      
      
      if (steps < 10) {
        histX_sub <- histX
      } else {
        histX_sub <-
          mutate(histX, xdiff = xmax - xmin, xmax = xmax - xdiff / 2)
      }
      
      coef.plot <- ggplot()
      coef.plot <-
        coef.plot + geom_rect(
          data = histX,
          aes(
            xmin = xmin,
            xmax = xmax,
            ymin = ymin,
            ymax = ymax
          ),
          colour = "gray50",
          alpha = 0,
          linewidth = 0.5
        )
      
      if (grouped) {
        coef.plot <-
          coef.plot + geom_errorbar(
            data = m,
            aes(
              x = fake,
              ymin = lb,
              ymax = ub,
              colour = value
            ),
            width = 0,
            linewidth = esize
          ) + scale_x_continuous(breaks = levels) + 
          ylab(default_ylab) + xlab(default_xlab) + 
          geom_point(data = m,
                     aes(x = fake,
                         y = coef1,
                         colour = value))
      } else{
        coef.plot <-
          coef.plot + geom_errorbar(
            data = m,
            aes(
              x = fake,
              ymin = lb,
              ymax = ub
            ),
            width = 0,
            color = ercolor,
            linewidth = esize
          ) + 
          scale_x_continuous(breaks = levels) + 
          ylab(default_ylab) + xlab(default_xlab) + 
          geom_point(data = m, 
                     aes(x = fake, 
                         y = coef1))
      }
    } else {
      yrange <- c(m$ub, m$lb)
      
      maxdiff <- (max(yrange) - min(yrange))
      
      break_var2 <- length(unique(var2_dt))
      if (break_var2 >= 100)
        break_var2 <- 100
      hist.out <-
        hist(var2_dt, breaks = break_var2, plot = FALSE)
      
      n.hist <- length(hist.out$mids)
      dist <- hist.out$mids[2] - hist.out$mids[1]
      hist.max <- max(hist.out$counts)
      
      histX <-
        data.frame(
          ymin = rep(min(yrange) - maxdiff / 5, n.hist),
          ymax = hist.out$counts / hist.max * maxdiff / 5 + min(yrange) - maxdiff /
            5,
          xmin = hist.out$mids - dist / 2,
          xmax = hist.out$mids + dist / 2
        )
      
      coef.plot <- ggplot()
      coef.plot <-
        coef.plot + geom_rect(
          data = histX,
          aes(
            xmin = xmin,
            xmax = xmax,
            ymin = ymin,
            ymax = ymax
          ),
          colour = "gray50",
          alpha = 0,
          linewidth = 0.5
        )
      
      if (grouped) {
        coef.plot <-
          coef.plot + 
          geom_line(data = m,
                    aes(x = fake,
                        y = coef1,
                        colour = value)) +
          geom_ribbon(data = m,
                      aes(
                        x = fake,
                        ymin = lb,
                        ymax = ub,
                        fill = value
                      ), 
            alpha = ralpha
          ) + ylab(default_ylab) + xlab(default_xlab)
      } else{
        coef.plot <-
          coef.plot + 
          geom_line(data = m,
                    aes(x = fake,
                        y = coef1)) + 
          geom_ribbon(
            data = m,
            aes(
              x = fake,
              ymin = lb,
              ymax = ub
            ),
            alpha = ralpha,
            color = ercolor,
            fill = rfill
          ) + 
          ylab(default_ylab) + xlab(default_xlab)
      }
    }
    
    if (stats_cp == "ci") {
      if (is.list(ci_diff)) {
        test_cp <- "" # not paste ci_diff here but in `facet_grid` in `interplot_default`
      } else {
        test_cp <- paste0("CI(Max - Min): [",
               round(ci_diff[1], digits = 3),
               ", ",
               round(ci_diff[2], digits = 3),
               "]")
      }
    } else if (stats_cp == "ks") {
      test_cp <-
        paste0("D(KS): ",
               ks_diff$statistic,
               "(p-value = ",
               format(round(ks_diff$p.value, digits = 3), nsmall = 3),
               ")")
    }
    
    coef.plot <- coef.plot +
      labs(caption = paste0(test_cp, txt_caption))
    
    return(coef.plot)
  }
  
} 

# Internal vectorized helpers for interplot (not exported)

#' Vectorized computation of conditional coefficients from simulation draws
#' @noRd
#' @keywords internal
sim_coef_vec <- function(sim_v1, sim_v12, fake, ci, multiplier = 1) {
  sim_matrix <- sim_v1 + multiplier * (sim_v12 %o% fake)
  probs <- c((1 - ci) / 2, 1 - (1 - ci) / 2)

  data.frame(
    fake = fake,
    coef1 = colMeans(sim_matrix),
    ub = apply(sim_matrix, 2, quantile, probs = probs[2]),
    lb = apply(sim_matrix, 2, quantile, probs = probs[1])
  )
}

#' General-gradient conditional effect from simulation draws
#'
#' Generalizes \code{sim_coef_vec} from the hardcoded linear basis
#' \code{[1, multiplier * z]} to an arbitrary gradient. \code{sim_mat} is the
#' (draws x terms) matrix of simulated coefficients for the terms that make up
#' the marginal effect of var1; \code{G} is the (n_fake x terms) basis matrix
#' giving each term's contribution to that effect across the var2 sequence.
#' The conditional effect for every draw is \code{sim_mat \%*\% t(G)}.
#' @noRd
#' @keywords internal
sim_coef_grad <- function(sim_mat, G, fake, ci) {
  sim_eff <- sim_mat %*% t(G) # draws x n_fake
  probs <- c((1 - ci) / 2, 1 - (1 - ci) / 2)

  data.frame(
    fake = fake,
    coef1 = colMeans(sim_eff),
    ub = apply(sim_eff, 2, quantile, probs = probs[2]),
    lb = apply(sim_eff, 2, quantile, probs = probs[1])
  )
}

#' ci_diff / ks_diff for the general-gradient effect (min vs max moderator)
#' @noRd
#' @keywords internal
sim_diff_stats_grad <- function(sim_mat, G, ci) {
  min_sim <- as.vector(sim_mat %*% G[1, ])
  max_sim <- as.vector(sim_mat %*% G[nrow(G), ])
  diff_sim <- max_sim - min_sim

  list(
    ci_diff = unname(stats::quantile(diff_sim, c((1 - ci) / 2, 1 - (1 - ci) / 2))),
    ks_diff = suppressWarnings(stats::ks.test(min_sim, max_sim))
  )
}

#' Simulate coefficient draws from N(mu, V) via Cholesky
#'
#' A drop-in alternative to \code{arm::sim} for the nonlinear path. \code{arm::sim}
#' silently drops spline/basis (matrix-valued) coefficient columns; sampling
#' directly from the coefficient vector \code{mu} and covariance \code{V} keeps
#' every term. \code{mu} is \code{coef(m)} for (g)lm and \code{fixef(m)} for
#' mixed models; \code{V} is the corresponding \code{vcov(m)}.
#' @noRd
#' @keywords internal
sim_betas_mvn <- function(mu, V, sims) {
  keep <- !is.na(mu) # drop aliased (rank-deficient) coefficients
  mu <- mu[keep]
  V <- as.matrix(V)[keep, keep, drop = FALSE]

  L <- chol(V) # V = t(L) %*% L
  Z <- matrix(stats::rnorm(sims * length(mu)), nrow = sims)
  draws <- Z %*% L + matrix(mu, nrow = sims, ncol = length(mu), byrow = TRUE)
  colnames(draws) <- names(mu)
  draws
}

#' Resolve the original model data (for rebuilding spline/polynomial bases)
#' @noRd
#' @keywords internal
me_model_data <- function(m, fallback = NULL) {
  data_full <- tryCatch(eval(stats::getCall(m)$data, environment(stats::formula(m))),
                        error = \(e) NULL)
  if (is.null(data_full)) data_full <- fallback
  data_full
}

#' Detect a nonlinear-in-moderator conditional effect
#'
#' Returns TRUE when the marginal effect of \code{var1} is a nonlinear function
#' of \code{var2}, i.e. \code{var1} interacts with a transformation of
#' \code{var2} (\code{I(var2^2)}, \code{poly(var2)}, \code{ns(var2)},
#' \code{bs(var2)}, ...) or with more than the single plain \code{var1:var2}
#' term. A plain two-way interaction returns FALSE so the fast legacy path runs.
#' \code{tt} is a \code{terms} object (use the fixed-effects-only terms for
#' mixed models).
#' @noRd
#' @keywords internal
detect_nonlinear <- function(tt, var1, var2) {
  factors <- attr(tt, "factors")
  if (is.null(factors) || !(var1 %in% rownames(factors))) return(FALSE)

  term_labels <- colnames(factors)
  v1_terms <- term_labels[factors[var1, ] > 0]

  involves_v2 <- vapply(v1_terms, \(tl) {
    vars <- tryCatch(all.vars(parse(text = tl)[[1]]), error = \(e) character(0))
    var2 %in% vars
  }, logical(1))

  v1v2_terms <- v1_terms[involves_v2]
  if (length(v1v2_terms) == 0) return(FALSE)

  plain <- v1v2_terms %in% c(paste0(var1, ":", var2), paste0(var2, ":", var1))
  any(!plain)
}

#' Build the (n_fake x terms) gradient basis for the var1 marginal effect
#'
#' Sets var1 = 1 and all other covariates to their null value (0 for numeric,
#' reference level for factors), sweeps var2 over \code{fake}, and reads the
#' design-matrix columns for the terms containing var1. Because the model is
#' linear in its coefficients, those columns are exactly the basis of the var1
#' marginal effect; \code{model.matrix} regenerates spline/polynomial bases at
#' the original knots via the terms' \code{predvars}. \code{tt} is the
#' response-deleted (fixed-effects) terms object; \code{data_full} is the
#' original model data.
#' @noRd
#' @keywords internal
build_me_design <- function(tt, data_full, var1, var2, fake) {
  needed <- all.vars(tt)
  miss <- setdiff(needed, names(data_full))
  if (length(miss))
    stop("Cannot reconstruct the design for nonlinear effects: variable(s) '",
         paste(miss, collapse = "', '"),
         "' not found in the model data. Refit the model with the data available.")

  ref <- lapply(needed, \(v) {
    x <- data_full[[v]]
    if (is.factor(x)) factor(levels(x)[1], levels = levels(x))
    else if (is.numeric(x)) 0
    else x[1]
  })
  names(ref) <- needed

  newdata <- as.data.frame(ref, stringsAsFactors = FALSE)[rep(1, length(fake)), , drop = FALSE]
  newdata[[var1]] <- 1
  newdata[[var2]] <- fake

  X <- stats::model.matrix(tt, newdata)
  asn <- attr(X, "assign")
  factors <- attr(tt, "factors")
  if (!(var1 %in% rownames(factors)))
    stop("var1 '", var1, "' is not a base term in the model.")

  v1_terms <- which(factors[var1, ] > 0)
  v1_cols <- which(asn %in% v1_terms)

  list(G = X[, v1_cols, drop = FALSE], term_names = colnames(X)[v1_cols])
}

#' Three-way / nonlinear handler for multiply-imputed models
#'
#' Shared by the \code{lmmi}/\code{glmmi}/\code{mlmmi}/\code{gmlmmi} methods.
#' Returns \code{NULL} when neither a three-way (\code{var3}) nor a nonlinear
#' moderator is requested (so the caller continues with its 2-way logic), or a
#' one-element list \code{list(value = ...)} wrapping the finished plot/data.
#' \code{draws_coef} are the imputation-pooled coefficient draws (for three-way);
#' nonlinear draws are rebuilt per imputation via \code{coef_fun}/\code{vcov_fun}
#' (because \code{arm::sim} drops spline columns) and pooled by stacking.
#' @noRd
#' @keywords internal
interplot_mi_extra <- function(m, m.list, draws_coef, frame, tt_fixed,
                               coef_fun, vcov_fun, df_resid,
                               var1, var2, var3, var3_vals, facet,
                               factor_v1, factor_v2, predPro,
                               plot, steps, ci, adjCI, hist, var2_dt, point,
                               sims, xmin, xmax, ercolor, esize, ralpha, rfill,
                               stats_cp, txt_caption, orig_var1, orig_var2, ...) {

  is_3way <- !is.null(var3)
  is_nonlinear <- !is_3way && !factor_v1 && !factor_v2 && predPro == FALSE &&
    var1 != var2 && detect_nonlinear(tt_fixed, var1, var2)
  if (!is_3way && !is_nonlinear) return(NULL)

  if (is_3way) {
    if (isTRUE(predPro))
      stop("Predicted probabilities are not supported for three-way interactions.")
    if (var1 == var2 || var1 == var3 || var2 == var3)
      stop("var1, var2, and var3 must be three distinct variables.")
    ls <- extract_coef_3way(
      draws = draws_coef, frame = frame, var1 = var1, var2 = var2, var3 = var3,
      var3_vals = var3_vals, ci = ci, adjCI = adjCI, df_resid = df_resid,
      steps = steps, xmin = xmin, xmax = xmax
    )
  } else {
    # Pool nonlinear draws across imputations by stacking N(coef, vcov) samples.
    draws <- do.call(rbind, map(m.list, \(i) sim_betas_mvn(coef_fun(i), as.matrix(vcov_fun(i)), sims)))
    ls <- extract_coef_nonlinear(
      tt = stats::delete.response(tt_fixed),
      data_full = me_model_data(m, fallback = frame),
      draws = draws, var1 = var1, var2 = var2, ci = ci, adjCI = adjCI,
      df_resid = df_resid, steps = steps, xmin = xmin, xmax = xmax
    )
  }

  coef_df <- ls[[1]]
  ci_diff <- ls[[2]]
  steps   <- ls[[3]]
  ks_diff <- ls[[4]]

  if (plot == FALSE) {
    names(coef_df)[1:4] <- c(var1, "coef", "ub", "lb")
    return(list(value = list(df_coef = coef_df, stats_ci = ci_diff)))
  }

  if (hist == TRUE && all(is.na(var2_dt))) var2_dt <- frame[[var2]]

  if (is_3way && stats_cp == "ci") {
    ci_note <- map2(ci_diff, levels(coef_df$value), \(aCI, aLevel) {
      paste0(aLevel, " CI(Max - Min): [",
             round(aCI[1], digits = 3), ", ",
             round(aCI[2], digits = 3), "]")
    }) |> list_c() |> paste(collapse = "\n")
    txt_caption <- paste0(ci_note, txt_caption)
  }

  overlay <- is_3way && !facet
  aPlot <- interplot.plot(
    m = coef_df, var1 = orig_var1, var2 = orig_var2, hist = hist, steps = steps,
    var2_dt = var2_dt, point = point, ercolor = ercolor, esize = esize,
    ralpha = ralpha, rfill = rfill, ci_diff = ci_diff, ks_diff = ks_diff,
    stats_cp = stats_cp, txt_caption = txt_caption, overlay = overlay, ...
  )
  if (is_3way && facet) {
    aPlot <- aPlot + ggplot2::facet_wrap(~ value)
  } else if (is_3way && !facet) {
    aPlot <- aPlot + ggplot2::labs(colour = var3, fill = var3)
  }

  list(value = aPlot)
}

#' Resolve an interaction coefficient name among the orderings R may emit
#'
#' Given the term components (e.g. c("wt", "cyl") or c("wt", "cyl", "hp")),
#' returns the first ":"-joined permutation that matches a coefficient name,
#' or NA if none match. R orders interaction terms by formula position, which
#' need not match the order var1/var2/var3 are passed to interplot.
#' @noRd
#' @keywords internal
resolve_term <- function(parts, coef_names) {
  perms <- if (length(parts) == 2) {
    list(parts, rev(parts))
  } else if (length(parts) == 3) {
    list(
      parts[c(1, 2, 3)], parts[c(1, 3, 2)], parts[c(2, 1, 3)],
      parts[c(2, 3, 1)], parts[c(3, 1, 2)], parts[c(3, 2, 1)]
    )
  } else {
    list(parts)
  }
  cand <- vapply(perms, \(p) paste(p, collapse = ":"), character(1))
  hit <- cand[cand %in% coef_names]
  if (length(hit)) hit[1] else NA_character_
}

#' Compute ci_diff and ks_diff statistics from simulation draws
#' @noRd
#' @keywords internal
sim_diff_stats <- function(sim_v1, sim_v12, xmin, xmax, ci, multiplier = 1) {
  min_sim <- sim_v1 + multiplier * xmin * sim_v12
  max_sim <- sim_v1 + multiplier * xmax * sim_v12
  diff_sim <- max_sim - min_sim

  list(
    ci_diff = unname(stats::quantile(diff_sim, c((1 - ci) / 2, 1 - (1 - ci) / 2))),
    ks_diff = stats::ks.test(min_sim, max_sim)
  )
}

# Feature B1: nonlinear-in-moderator conditional effects (gradient engine).

test_that("the gradient engine reproduces the linear engine exactly", {
  set.seed(1)
  sv1 <- rnorm(500, 2, 0.5)
  sv12 <- rnorm(500, -0.3, 0.1)
  fake <- seq(1, 5, length.out = 30)

  a <- sim_coef_vec(sv1, sv12, fake, 0.95)
  b <- sim_coef_grad(cbind(sv1, sv12), cbind(1, fake), fake, 0.95)
  expect_equal(a$coef1, b$coef1, tolerance = 1e-12)
  expect_equal(a$ub, b$ub, tolerance = 1e-12)
  expect_equal(a$lb, b$lb, tolerance = 1e-12)
})

test_that("the gradient engine reproduces the quadratic (multiplier = 2) engine", {
  set.seed(2)
  sv1 <- rnorm(500, 1, 0.4)
  sv12 <- rnorm(500, 0.2, 0.05)
  fake <- seq(0, 4, length.out = 25)

  a <- sim_coef_vec(sv1, sv12, fake, 0.95, multiplier = 2)
  b <- sim_coef_grad(cbind(sv1, sv12), cbind(1, 2 * fake), fake, 0.95)
  expect_equal(a$coef1, b$coef1, tolerance = 1e-12)
})

test_that("detect_nonlinear is FALSE for a plain interaction, TRUE otherwise", {
  expect_false(detect_nonlinear(terms(lm(mpg ~ wt * cyl, mtcars)), "wt", "cyl"))
  expect_true(detect_nonlinear(terms(lm(mpg ~ wt * (cyl + I(cyl^2)), mtcars)), "wt", "cyl"))
  expect_true(detect_nonlinear(terms(lm(mpg ~ wt * splines::ns(hp, 3), mtcars)), "wt", "hp"))
})

test_that("polynomial-in-moderator effect matches the analytic gradient", {
  m <- lm(mpg ~ wt * (cyl + I(cyl^2)), data = mtcars)
  set.seed(3)
  d <- interplot(m, "wt", "cyl", plot = FALSE)$df_coef
  cf <- coef(m)
  z <- d$wt # column 1 holds the cyl sequence
  expected <- cf["wt"] + cf["wt:cyl"] * z + cf["wt:I(cyl^2)"] * z^2
  expect_lt(max(abs(d$coef - unname(expected))), 0.2)
})

test_that("spline-in-moderator effect matches a finite difference of predict()", {
  m <- lm(mpg ~ wt * splines::ns(hp, 3), data = mtcars)
  set.seed(4)
  d <- interplot(m, "wt", "hp", plot = FALSE)$df_coef
  z <- d$wt
  h <- 1e-4
  wt0 <- mean(mtcars$wt)
  fd <- vapply(z, function(zz) {
    (predict(m, data.frame(wt = wt0 + h, hp = zz)) -
       predict(m, data.frame(wt = wt0, hp = zz))) / h
  }, numeric(1))
  expect_lt(max(abs(d$coef - fd)), 0.35)
})

test_that("a nonlinear model still returns a ggplot", {
  m <- lm(mpg ~ wt * (cyl + I(cyl^2)), data = mtcars)
  set.seed(5)
  expect_s3_class(interplot(m, "wt", "cyl"), "ggplot")
})

if(getRversion() >= "2.15.1") utils::globalVariables(c(".", "X.weights.", "value", ":="))

#' Plot Conditional Coefficients in Mixed-Effects Models with Interaction Terms 
#' 
#' \code{interplot.mlm} is a method to calculate conditional coefficient estimates from the results of multilevel (mixed-effects) regression models with interaction terms. 
#' 
#' @param m A model object including an interaction term, or, alternately, a data frame recording conditional coefficients.
#' @param var1 The name (as a string) of the variable of interest in the interaction term; its conditional coefficient estimates will be plotted.
#' @param var2 The name (as a string) of the other variable in the interaction term.
#' @param plot A logical value indicating whether the output is a plot or a dataframe including the conditional coefficient estimates of var1, their upper and lower bounds, and the corresponding values of var2.
#' @param steps Desired length of the sequence. A non-negative number, which for seq and seq.int will be rounded up if fractional. The default is 100 or the unique categories in the \code{var2} (when it is less than 100. Also see \code{\link{unique}}).
#' @param ci A numeric value defining the confidence intervals. The default value is 95\% (0.95).
#' @param adjCI Not working for `lmer` outputs yet.
#' @param hist A logical value indicating if there is a histogram of `var2` added at the bottom of the conditional effect plot.
#' @param var2_dt A numerical value indicating the frequency distribution of `var2`. It is only used when `hist == TRUE`. When the object is a model, the default is the distribution of `var2` of the model. 
#' @param predPro A logical value with default of `FALSE`. When the `m` is an object of class `glmerMod` and the argument is set to `TRUE`, the function will plot predicted probabilities at the values given by `var2_vals`. 
#' @param var2_vals A numerical value indicating the values the predicted probabilities are estimated, when `predPro` is `TRUE`. 
#' @param point A logical value determining the format of plot. By default, the function produces a line plot when var2 takes on ten or more distinct values and a point (dot-and-whisker) plot otherwise; option TRUE forces a point plot.
#' @param sims Number of independent simulation draws used to calculate upper and lower bounds of coefficient estimates: lower values run faster; higher values produce smoother curves.
#' @param xmin A numerical value indicating the minimum value shown of x shown in the graph. Rarely used.
#' @param xmax A numerical value indicating the maximum value shown of x shown in the graph. Rarely used.
#' @param ercolor A character value indicating the outline color of the whisker or ribbon.
#' @param esize A numerical value indicating the size of the whisker or ribbon.
#' @param ralpha A numerical value indicating the transparency of the ribbon.
#' @param rfill A character value indicating the filling color of the ribbon.
#' @param stats_cp A character value indicating what statistics to present as the plot note. Three options are available: "none", "ci", and "ks". The default is "none". See the Details for more information.
#' @param txt_caption A character string to add a note for the plot, a value will sending to \code{ggplot2::labs(caption = txt_caption))}.
#' @param facet_labs An optional character vector of facet labels to be used when plotting an interaction with a factor variable.
#' @param var3 An optional name (as a string) of a third variable for a three-way interaction \code{var1 * var2 * var3}. When supplied, the conditional effect of \code{var1} across \code{var2} is shown at several values/levels of \code{var3}. The default \code{NULL} gives the standard two-way behavior. Requires continuous \code{var1} and \code{var2}.
#' @param var3_vals An optional numeric vector giving the values of a continuous \code{var3} to condition on. The default is the mean and the mean plus or minus one standard deviation. Ignored when \code{var3} is a factor.
#' @param facet A logical value, used only with \code{var3}. \code{TRUE} (default) draws one panel per value/level of \code{var3}; \code{FALSE} overlays the curves colored by \code{var3}.
#' @param ... Other ggplot aesthetics arguments for points in the dot-whisker plot or lines in the line-ribbon plots. Not currently used.
#' 
#' @details \code{interplot.mlm} is a S3 method from the \code{interplot}. It works on mixed-effects objects with class \code{lmerMod} and \code{glmerMod}.
#' 
#' Because the output function is based on \code{\link[ggplot2]{ggplot}}, any additional arguments and layers supported by \code{ggplot2} can be added with the \code{+}. 
#' 
#' \code{interplot} visualizes the conditional effect based on simulated marginal effects. The simulation provides a probabilistic distribution of moderation effect of the conditioning variable (\code{var2}) at every preset values (including the minimum and maximum values) of the conditioned variable (\code{var1}), denoted as Emin and Emax. This output allows the function to further examine the conditional effect statistically in two ways. One is to examine if the distribution of \eqn{Emax - Emin} covers zero. The other is to directly compare Emin and Emax through statistical tools for distributional comparisons. Users can choose either method by setting the argument \code{stats_cp} to "ci" or "ks".
#' \itemize{
#'   \item "ci" provides the confidence interval of the difference of \eqn{Emax - Emin}. An interval including 0 suggests no statistical difference before and after the conditional effect is applied, and vise versa.
#'   \item "ks" presents the result of a two-sample Kolmogorov-Smirnov test of the simulated distributions of Emin and Emax. The output includes a D statistics and a p-value of the null hypothesis that the two distributions come from the same distribution at the 0.05 level.
#' }
#' 
#' See an illustration in the package vignette.
#' 
#' @return The function returns a \code{ggplot} object.
#' 
#' @importFrom arm sim
#' @importFrom stats quantile median plogis model.matrix model.frame setNames ks.test
#' @importFrom purrr map map2 list_c list_rbind
#' @importFrom lme4 fixef ranef getME
#' @import ggplot2
#' @import dplyr
#' 
#' 
#' @export


# Coding function for non-mi mlm objects
interplot.lmerMod <- function(m,
           var1,
           var2,
           plot = TRUE,
           steps = NULL,
           ci = .95,
           adjCI = FALSE,
           hist = FALSE,
           var2_dt = NA,
           predPro = FALSE,
           var2_vals = NULL,
           point = FALSE,
           sims = 5000,
           xmin = NA,
           xmax = NA,
           ercolor = NA,
           esize = 0.5,
           ralpha = 0.5,
           rfill = "grey70",
           stats_cp = "none",
           txt_caption = NULL,
           facet_labs = NULL,
           var3 = NULL,
           var3_vals = NULL,
           facet = TRUE,
           ...)
  {

    # Coefficient data frame ####
    ## Factorial base terms
    factor_v1 <- is.factor(m@frame[[var1]])
    factor_v2 <- is.factor(m@frame[[var2]])

    if (factor_v1 & factor_v2)
      stop("The function does not support interactions between two factors.")

    if ((factor_v1 | factor_v2) & predPro == TRUE)
      stop("The current version does not support estimating predicted probabilities for factor base terms.")

    is_3way <- !is.null(var3)
    tt_fixed <- stats::terms(m, fixed.only = TRUE)
    is_nonlinear <- !is_3way && !factor_v1 && !factor_v2 && predPro == FALSE &&
      var1 != var2 && detect_nonlinear(tt_fixed, var1, var2)

    # The nonlinear path draws its own coefficients (arm::sim drops spline
    # columns); every other path uses arm::sim.
    m.sims <- if (is_nonlinear) NULL else arm::sim(m, sims)

    if (is_3way) {
      if (predPro == TRUE)
        stop("Predicted probabilities are not supported for three-way interactions.")
      if (var1 == var2 || var1 == var3 || var2 == var3)
        stop("var1, var2, and var3 must be three distinct variables.")
      fixef_draws <- m.sims@fixef
      colnames(fixef_draws) <- unlist(dimnames(m@pp$X)[2])
      ls_results <- extract_coef_3way(
        draws = fixef_draws,
        frame = m@frame,
        var1 = var1,
        var2 = var2,
        var3 = var3,
        var3_vals = var3_vals,
        ci = ci,
        adjCI = adjCI,
        df_resid = NULL,
        steps = steps,
        xmin = xmin,
        xmax = xmax
      )
    } else if (is_nonlinear) {
      ls_results <- extract_coef_nonlinear(
        tt = stats::delete.response(tt_fixed),
        data_full = me_model_data(m, fallback = m@frame),
        draws = sim_betas_mvn(lme4::fixef(m), as.matrix(stats::vcov(m)), sims),
        var1 = var1,
        var2 = var2,
        ci = ci,
        adjCI = adjCI,
        df_resid = NULL,
        steps = steps,
        xmin = xmin,
        xmax = xmax
      )
    } else if (factor_v1 | factor_v2) {
      ls_results <- extract_coef_facM(
        factor_v1 = factor_v1,
        factor_v2 = factor_v2,
        m = m,
        m.sims = m.sims,
        var1 = var1,
        var2 = var2, 
        plot = plot,
        steps = steps,
        ci = ci,
        adjCI = adjCI,
        predPro = predPro,
        var2_vals = var2_vals,
        point = point,
        xmin = xmin,
        xmax = xmax
      )
    } else {
      ls_results <- extract_coef_numM(
        m = m,
        m.sims = m.sims,
        var1 = var1,
        var2 = var2, 
        plot = plot,
        steps = steps,
        ci = ci,
        sims = sims,
        adjCI = adjCI,
        predPro = predPro,
        var2_vals = var2_vals,
        point = point,
        xmin = xmin,
        xmax = xmax
      )
    }
    
    coef_df <- ls_results[[1]]
    ci_diff <- ls_results[[2]]
    steps <- ls_results[[3]]
    ks_diff <- ls_results[[4]]
    
    # Plotting ####
    
    if (hist == TRUE & all(is.na(var2_dt))) { # when var2_dt has values, is.na returns multiple values
      var2_dt <- m@frame[[var2]]
    } 
    
    # Replace the labels
    if (is_3way) {
      # The "value" column already carries ordered var3 labels; route any
      # CI(Max - Min) note to the caption so it survives faceting/overlay.
      if (stats_cp == "ci") {
        ci_note <- map2(ci_diff, levels(coef_df$value), \(aCI, aLevel) {
          paste0(aLevel, " CI(Max - Min): [",
                 round(aCI[1], digits = 3), ", ",
                 round(aCI[2], digits = 3), "]")
        }) |>
          list_c() |>
          paste(collapse = "\n")
        txt_caption <- paste0(ci_note, txt_caption)
      }
    } else if (factor_v1 | factor_v2) {
      if (is.null(facet_labs)) facet_labs <- unique(coef_df$value)

      # The CI(Max - Min) note goes to the plot caption (always visible) rather
      # than the facet strip, so it survives `strip.text = element_blank()`.
      if (stats_cp == "ci") {
        ci_note <- map2(ci_diff, facet_labs, \(aCI, aLevel) {
          paste0(if (length(facet_labs) > 1) paste0(aLevel, " ") else "",
                 "CI(Max - Min): [",
                 round(aCI[1], digits = 3),
                 ", ",
                 round(aCI[2], digits = 3),
                 "]")
        }) |>
          list_c() |>
          paste(collapse = "\n")
        txt_caption <- paste0(ci_note, txt_caption)
      }

      coef_df$value <- factor(coef_df$value, labels = facet_labs)
    }

    # Plotting the general plot
    if (plot == FALSE) {
      names(coef_df)[1:4] <- c(var1, "coef", "ub", "lb") # just rename the first four cols; the factorial/three-way results have a fifth column "value"

      return(list(df_coef = coef_df, stats_ci = ci_diff))

    } else {
      overlay <- is_3way && !facet

      aPlot <- interplot.plot(
        m = coef_df,
        var1 = var1,
        var2 = var2,
        hist = hist,
        steps = steps,
        var2_dt = var2_dt,
        predPro = predPro,
        point = point,
        ercolor = ercolor,
        esize = esize,
        ralpha = ralpha,
        rfill = rfill,
        stats_cp = stats_cp,
        txt_caption = txt_caption,
        ci_diff = ci_diff,
        ks_diff = ks_diff,
        overlay = overlay,
        ...
      )

      # Facet for factors or three-way small multiples
      if (factor_v1 | factor_v2) {
        aPlot <- aPlot + facet_grid(. ~ value)
      } else if (is_3way && facet) {
        aPlot <- aPlot + facet_wrap(~ value)
      } else if (is_3way && !facet) {
        aPlot <- aPlot + labs(colour = var3, fill = var3)
      }

      return(aPlot)
    }
  }


extract_coef_numM <- function(
    m,
    m.sims,
    var1,
    var2,
    plot,
    steps,
    ci,
    sims,
    adjCI,
    predPro,
    var2_vals,
    point,
    xmin,
    xmax
){
  
  # Detect if it is a quadratic model
  if (var1 == var2) {
    var12 <- paste0("I(", var1, "^2)")
  } else {
    var12 <- paste0(var2, ":", var1)
  }

  coef_names <- unlist(dimnames(m@pp$X)[2])

  # Check if the interaction term is correctly specified ####
  if (!var12 %in% coef_names)
    var12 <- paste0(var1, ":", var2)
  if (!var12 %in% coef_names)
    stop(paste("Model does not include the interaction of", var1, "and", var2, "."))

  # Set the min, max, and steps ####
  if (is.na(xmin)) xmin <- min(m@frame[[var2]], na.rm = TRUE)
  if (is.na(xmax)) xmax <- max(m@frame[[var2]], na.rm = TRUE)

  if (is.null(steps)) {
    steps <- length(unique(na.omit(m@frame[[var2]])))
  }
  if (steps > 100) steps <- 100

  fake <- seq(xmin, xmax, length.out = steps)

  # Calculate the effects ####
  ci_diff <- numeric(0)
  ks_diff <- NULL
  multiplier <- if (var1 == var2) 2 else 1

  if (predPro == TRUE) {
    if (is.null(var2_vals))
      stop("The predicted probabilities cannot be estimated without defining 'var2_vals'.")

    df <- data.frame(m@frame)
    coef <- map(var2_vals, \(bound) {
      fake_data <- mutate(df, !!var2 := bound)
      merTools::predictInterval(m,
        newdata = fake_data,
        level = ci,
        n.sims = sims,
        type = "probability"
      ) |>
        mutate(value = bound, fake = df[[var1]])
    }) |>
      list_rbind() |>
      mutate(value = as.factor(value)) |>
      setNames(c("coef1", "lb", "ub", "value", "fake"))

  } else {
    sim_v1 <- m.sims@fixef[, match(var1, coef_names)]
    sim_v12 <- m.sims@fixef[, match(var12, coef_names)]

    coef <- sim_coef_vec(sim_v1, sim_v12, fake, ci, multiplier)

    diff_stats <- sim_diff_stats(sim_v1, sim_v12, xmin, xmax, ci, multiplier)
    ci_diff <- diff_stats$ci_diff
    ks_diff <- diff_stats$ks_diff

    if (adjCI == TRUE) {
      n <- nrow(model.frame(m))
      p <- length(fixef(m))
      theta <- getME(m, "theta")
      satterth_df <- n - p - 2 * sum(theta != 0)
      warning("For `lme4` objects, the degree of freedom is calculated by the Satterthwaite approximation.")

      coef$sd <- (coef$ub - coef$coef1) / qnorm(1 - (1 - ci) / 2)
      invisible(utils::capture.output(
        tAdj <- interactionTest::fdrInteraction(coef$coef1, coef$sd, df = satterth_df, level = ci)))
      coef$ub <- coef$coef1 + tAdj * coef$sd
      coef$lb <- coef$coef1 - tAdj * coef$sd
    }
  }

  return(list(coef, ci_diff, steps, ks_diff))
}


extract_coef_facM <- function(
    factor_v1,
    factor_v2,
    m,
    m.sims,
    var1,
    var2,
    plot,
    steps,
    ci,
    adjCI,
    predPro,
    var2_vals,
    point,
    xmin,
    xmax
) {
  # Generate the name of the coefficients ####
  if (factor_v1) {
    var1 <- paste0(var1, levels(m@frame[[var1]]))
  } else if (factor_v2) {
    var2 <- paste0(var2, levels(m@frame[[var2]]))
  }

  coef_names <- unlist(dimnames(m@pp$X)[2])
  var12 <- paste0(var2, ":", var1)[-1]

  # Check if the interaction terms are correctly specified ####
  for (i in seq(var12)) {
    if (!var12[i] %in% coef_names)
      var12[i] <- paste0(var1, ":", var2)[-1][i]
    if (!var12[i] %in% coef_names)
      stop(paste("Model does not include the interaction of", var1, "and", var2, "."))
  }

  # Set the min, max, and steps ####
  if (factor_v2) {
    xmin <- 0
    xmax <- 1
    steps <- 2
  } else {
    if (is.na(xmin)) xmin <- min(m@frame[[var2]], na.rm = TRUE)
    if (is.na(xmax)) xmax <- max(m@frame[[var2]], na.rm = TRUE)
    if (is.null(steps)) {
      steps <- length(unique(m@frame[[var2]]))
    }
    if (steps > 100) steps <- 100
  }

  fake_seq <- seq(xmin, xmax, length.out = steps)

  # Calculate the effects ####
  ci_diff <- vector(mode = "list")

  # Precompute adjCI quantities once if needed
  satterth_df <- NULL
  if (adjCI == TRUE) {
    n <- nrow(model.frame(m))
    p <- length(fixef(m))
    theta <- getME(m, "theta")
    satterth_df <- n - p - 2 * sum(theta != 0)
    warning("For `lme4` objects, the degree of freedom is calculated by the Satterthwaite approximation.")
  }

  if (factor_v1) {
    results <- map(seq(var1)[-length(var1)], \(j) {
      sim_v1 <- m.sims@fixef[, var1[j + 1]]
      sim_v12 <- m.sims@fixef[, var12[j]]
      coef <- sim_coef_vec(sim_v1, sim_v12, fake_seq, ci)
      d <- sim_diff_stats(sim_v1, sim_v12, xmin, xmax, ci)

      if (adjCI == TRUE) {
        coef$sd <- (coef$ub - coef$coef1) / qnorm(1 - (1 - ci) / 2)
        invisible(utils::capture.output(
        tAdj <- interactionTest::fdrInteraction(coef$coef1, coef$sd, df = satterth_df, level = ci)))
        coef$ub <- coef$coef1 + tAdj * coef$sd
        coef$lb <- coef$coef1 - tAdj * coef$sd
      }

      coef$value <- var1[j + 1]
      list(coef = coef, ci_diff = d$ci_diff)
    })

    coef_df <- map(results, "coef") |> list_rbind()
    ci_diff <- map(results, "ci_diff") |> setNames(var12)

  } else if (factor_v2) {
    results <- map(seq(var2)[-length(var2)], \(j) {
      sim_v1 <- m.sims@fixef[, match(var1, coef_names)]
      sim_v12 <- m.sims@fixef[, match(var12[j], coef_names)]
      coef <- sim_coef_vec(sim_v1, sim_v12, fake_seq, ci)
      d <- sim_diff_stats(sim_v1, sim_v12, xmin, xmax, ci)

      if (adjCI == TRUE) {
        coef$sd <- (coef$ub - coef$coef1) / qnorm(1 - (1 - ci) / 2)
        invisible(utils::capture.output(
        tAdj <- interactionTest::fdrInteraction(coef$coef1, coef$sd, df = satterth_df, level = ci)))
        coef$ub <- coef$coef1 + tAdj * coef$sd
        coef$lb <- coef$coef1 - tAdj * coef$sd
      }

      coef$value <- var2[j + 1]
      list(coef = coef, ci_diff = d$ci_diff)
    })

    coef_df <- map(results, "coef") |> list_rbind()
    ci_diff <- map(results, "ci_diff") |> setNames(var12)
  }

  return(list(coef_df, ci_diff, steps, NULL))
} 

#' @export
#' 

interplot.glmerMod <- interplot.lmerMod

#' Compute Johnson-Neyman Interval for Interaction Effects
#'
#' Identifies the values of the moderating variable at which the
#' conditional effect of the other variable transitions between
#' statistical significance and non-significance.
#'
#' @param m A model object (\code{lm}, \code{glm}, \code{lmerMod}, or \code{glmerMod}).
#' @param var1 The name (as a string) of the variable whose conditional effect is of interest.
#' @param var2 The name (as a string) of the moderating variable.
#' @param ci A numeric value defining the confidence level. The default is 0.95.
#'
#' @details The Johnson-Neyman (JN) technique finds the values of the moderating
#' variable (\code{var2}) at which the conditional effect of \code{var1} is
#' exactly at the boundary of statistical significance. This is computed
#' analytically from the regression coefficients and their variance-covariance
#' matrix.
#'
#' For linear mixed-effects models (\code{lmerMod}, \code{glmerMod}), a normal
#' approximation (z-distribution) is used instead of the t-distribution due
#' to the controversy over appropriate degrees of freedom.
#'
#' The function does not support factor variables or quadratic terms (\code{var1 == var2}).
#'
#' @return An object of class \code{jn_interval} containing:
#' \describe{
#'   \item{bounds}{All Johnson-Neyman bounds (may be 0, 1, or 2 values).}
#'   \item{bounds_in_range}{Bounds that fall within the observed data range of \code{var2}.}
#'   \item{var2_range}{The range of the moderating variable in the data.}
#'   \item{sig_pattern}{One of \code{"always"}, \code{"never"}, \code{"between"}, \code{"outside"}, \code{"below"}, or \code{"above"}, indicating where the effect is statistically significant.}
#'   \item{ns_regions}{List of \code{(xmin, xmax)} pairs marking non-significant regions.}
#' }
#'
#' @examples
#' m <- lm(mpg ~ wt * cyl, data = mtcars)
#' jn <- jn_interval(m, "cyl", "wt")
#' print(jn)
#'
#' # Add JN bounds to an interplot
#' interplot(m, "cyl", "wt") + jn_layer(jn)
#'
#' @importFrom stats qt coef vcov df.residual
#' @export
jn_interval <- function(m, var1, var2, ci = 0.95) {

  if (inherits(m, c("lmerMod", "glmerMod"))) {
    coefs <- lme4::fixef(m)
    vc <- as.matrix(stats::vcov(m))
    df_resid <- Inf
    model_data <- m@frame
  } else if (inherits(m, c("lm", "glm"))) {
    coefs <- stats::coef(m)
    vc <- stats::vcov(m)
    df_resid <- stats::df.residual(m)
    model_data <- m$model
  } else {
    stop("jn_interval supports lm, glm, lmerMod, and glmerMod objects.")
  }

  coef_names <- names(coefs)

  if (is.factor(model_data[[var1]]) || is.factor(model_data[[var2]]))
    stop("Johnson-Neyman intervals require continuous variables for both var1 and var2.")

  if (var1 == var2)
    stop("Johnson-Neyman intervals are not applicable to quadratic terms (var1 == var2).")

  var12 <- paste0(var2, ":", var1)
  if (!var12 %in% coef_names) var12 <- paste0(var1, ":", var2)
  if (!var12 %in% coef_names)
    stop("Model does not include the interaction of ", var1, " and ", var2, ".")

  b1 <- unname(coefs[var1])
  b3 <- unname(coefs[var12])
  v_b1 <- vc[var1, var1]
  v_b3 <- vc[var12, var12]
  cov_b1b3 <- vc[var1, var12]

  t_crit <- stats::qt(1 - (1 - ci) / 2, df_resid)

  a <- b3^2 - t_crit^2 * v_b3
  b_q <- 2 * (b1 * b3 - t_crit^2 * cov_b1b3)
  c_q <- b1^2 - t_crit^2 * v_b1
  disc <- b_q^2 - 4 * a * c_q

  if (disc < 0) {
    bounds <- numeric(0)
  } else if (abs(a) < .Machine$double.eps) {
    if (abs(b_q) < .Machine$double.eps) {
      bounds <- numeric(0)
    } else {
      bounds <- -c_q / b_q
    }
  } else {
    z1 <- (-b_q - sqrt(disc)) / (2 * a)
    z2 <- (-b_q + sqrt(disc)) / (2 * a)
    bounds <- sort(c(z1, z2))
  }

  var2_range <- range(model_data[[var2]], na.rm = TRUE)
  bounds_in_range <- bounds[bounds >= var2_range[1] & bounds <= var2_range[2]]

  is_sig <- function(z) {
    eff <- b1 + b3 * z
    se <- sqrt(v_b1 + z^2 * v_b3 + 2 * z * cov_b1b3)
    abs(eff / se) > t_crit
  }

  if (length(bounds) == 2) {
    sig_between <- is_sig(mean(bounds))
    sig_pattern <- if (sig_between) "between" else "outside"
    ns_regions <- if (sig_between) {
      list(c(-Inf, bounds[1]), c(bounds[2], Inf))
    } else {
      list(c(bounds[1], bounds[2]))
    }
  } else if (length(bounds) == 1) {
    sig_below <- is_sig(bounds[1] - 1)
    sig_pattern <- if (sig_below) "below" else "above"
    ns_regions <- if (sig_below) {
      list(c(bounds[1], Inf))
    } else {
      list(c(-Inf, bounds[1]))
    }
  } else {
    center_sig <- is_sig(mean(var2_range))
    sig_pattern <- if (center_sig) "always" else "never"
    ns_regions <- if (center_sig) list() else list(c(-Inf, Inf))
  }

  structure(
    list(
      bounds = bounds,
      bounds_in_range = bounds_in_range,
      var2_range = var2_range,
      var1 = var1,
      var2 = var2,
      ci = ci,
      b1 = b1,
      b3 = b3,
      sig_pattern = sig_pattern,
      ns_regions = ns_regions
    ),
    class = "jn_interval"
  )
}


#' @export
print.jn_interval <- function(x, ...) {
  cat("Johnson-Neyman Interval\n")
  cat("=======================\n")
  cat("Variable of interest:", x$var1, "\n")
  cat("Moderating variable: ", x$var2, "\n")
  cat("Confidence level:    ", x$ci * 100, "%\n\n")

  if (length(x$bounds) == 0) {
    if (x$sig_pattern == "always") {
      cat("The effect of", x$var1, "is significant across the entire range of", x$var2, ".\n")
    } else {
      cat("The effect of", x$var1, "is not significant at any value of", x$var2, ".\n")
    }
  } else {
    cat("JN bound(s):", paste(round(x$bounds, 3), collapse = ", "), "\n")

    if (length(x$bounds_in_range) > 0) {
      cat("Within data range [", round(x$var2_range[1], 3), ", ",
          round(x$var2_range[2], 3), "]: ",
          paste(round(x$bounds_in_range, 3), collapse = ", "), "\n", sep = "")
    } else {
      cat("No bounds within the observed data range [",
          round(x$var2_range[1], 3), ", ", round(x$var2_range[2], 3), "].\n", sep = "")
    }

    sig_desc <- switch(x$sig_pattern,
      "between" = paste0("The effect is significant when ", x$var2,
                         " is between ", round(x$bounds[1], 3),
                         " and ", round(x$bounds[2], 3), "."),
      "outside" = paste0("The effect is significant when ", x$var2,
                         " is below ", round(x$bounds[1], 3),
                         " or above ", round(x$bounds[2], 3), "."),
      "below"   = paste0("The effect is significant when ", x$var2,
                         " is below ", round(x$bounds[1], 3), "."),
      "above"   = paste0("The effect is significant when ", x$var2,
                         " is above ", round(x$bounds[1], 3), ".")
    )
    cat("\n", sig_desc, "\n", sep = "")
  }

  invisible(x)
}


#' Add Johnson-Neyman Bounds to an interplot
#'
#' Returns a list of \code{ggplot2} layers that overlay Johnson-Neyman significance
#' boundaries on an \code{interplot} output. Add to any interplot with \code{+}.
#'
#' @param jn A \code{jn_interval} object produced by \code{\link{jn_interval}}.
#' @param line_color Color of the boundary lines. Default \code{"red"}.
#' @param linetype Line type for boundaries. Default \code{"dashed"}.
#' @param shade_color Fill color for non-significant regions. Default \code{"grey80"}.
#' @param shade_alpha Transparency of shading. Default \code{0.15}.
#' @param label Logical; if \code{TRUE} (default), annotate the JN bound values.
#'
#' @return A list of \code{ggplot2} layers.
#'
#' @examples
#' m <- lm(mpg ~ wt * cyl, data = mtcars)
#' jn <- jn_interval(m, "cyl", "wt")
#' interplot(m, "cyl", "wt") + jn_layer(jn)
#'
#' @import ggplot2
#' @export
jn_layer <- function(jn, line_color = "red", linetype = "dashed",
                     shade_color = "grey80", shade_alpha = 0.15,
                     label = TRUE) {
  if (!inherits(jn, "jn_interval"))
    stop("jn must be a jn_interval object from jn_interval().")

  layers <- list()

  # Shade only the part of each non-significant region that falls within the
  # observed range of var2, so the overlay never widens the plot's x-axis
  # beyond the data (JN bounds can lie far outside the data range).
  rng <- jn$var2_range
  for (r in jn$ns_regions) {
    lo <- max(r[1], rng[1])
    hi <- min(r[2], rng[2])
    if (lo < hi) {
      layers <- c(layers, list(
        annotate("rect", xmin = lo, xmax = hi,
                 ymin = -Inf, ymax = Inf,
                 fill = shade_color, alpha = shade_alpha)
      ))
    }
  }

  if (length(jn$bounds_in_range) > 0) {
    layers <- c(layers, list(
      geom_vline(xintercept = jn$bounds_in_range,
                 color = line_color, linetype = linetype, linewidth = 0.7)
    ))

    if (label) {
      for (b in jn$bounds_in_range) {
        layers <- c(layers, list(
          annotate("text", x = b, y = Inf,
                   label = paste0("JN = ", round(b, 2)),
                   vjust = 1.5, hjust = -0.1, size = 3, color = line_color)
        ))
      }
    }
  }

  layers
}

# Johnson-Neyman interval + layer.

test_that("jn_interval returns bounds and a recognized significance pattern", {
  m <- lm(mpg ~ wt * cyl, data = mtcars)
  jn <- jn_interval(m, "cyl", "wt")
  expect_s3_class(jn, "jn_interval")
  expect_true(jn$sig_pattern %in% c("always", "never", "between", "outside", "below", "above"))
  expect_length(jn$var2_range, 2)
})

test_that("jn_layer shading does not widen the x-axis beyond the data range", {
  m <- lm(mpg ~ wt * cyl, data = mtcars)
  jn <- jn_interval(m, "cyl", "wt")
  # This example has a JN bound (~14) far outside the data range (~[1.5, 5.4]);
  # the shaded non-significant region must be clipped to the data range so the
  # overlay does not stretch the axis (regression guard for that bug).
  p <- interplot(m, "cyl", "wt") + jn_layer(jn)
  xr <- ggplot2::ggplot_build(p)$layout$panel_params[[1]]$x.range
  # allow ggplot's default 5% expansion beyond the data range
  span <- diff(jn$var2_range)
  expect_lte(xr[2], jn$var2_range[2] + 0.1 * span)
  expect_gte(xr[1], jn$var2_range[1] - 0.1 * span)
})

if(getRversion() >= "2.15.1") utils::globalVariables(c(".", "X.weights."))

#' Plot Conditional Coefficients in Mixed-Effects Models with Imputed Data and Interaction Terms
#'
#' \code{interplot.mlmmi} is a method to calculate conditional coefficient estimates from the results of multilevel (mixed-effects) regression models with interaction terms and multiply imputed data.
#'
#' @param m A model object including an interaction term, or, alternately, a data frame recording conditional coefficients.
#' @param var1 The name (as a string) of the variable of interest in the interaction term; its conditional coefficient estimates will be plotted.
#' @param var2 The name (as a string) of the other variable in the interaction term.
#' @param plot A logical value indicating whether the output is a plot or a dataframe including the conditional coefficient estimates of var1, their upper and lower bounds, and the corresponding values of var2.
#' @param steps Desired length of the sequence. A non-negative number, which for seq and seq.int will be rounded up if fractional. The default is 100 or the unique categories in the \code{var2} (when it is less than 100. Also see \code{\link{unique}}).
#' @param ci A numeric value defining the confidence intervals. The default value is 95\% (0.95).
#' @param adjCI Not working for `lmer` outputs yet.
#' @param hist A logical value indicating if there is a histogram of `var2` added at the bottom of the conditional effect plot.
#' @param var2_dt A numerical value indicating the frequency distribution of `var2`. It is only used when `hist == TRUE`. When the object is a model, the default is the distribution of `var2` of the model.
#' @param predPro A logical value with default of `FALSE`. When the `m` is an object of class `glmerMod` and the argument is set to `TRUE`, the function will plot predicted probabilities at the values given by `var2_vals`.
#' @param var2_vals A numerical value indicating the values the predicted probabilities are estimated, when `predPro` is `TRUE`.
#' @param point A logical value determining the format of plot. By default, the function produces a line plot when var2 takes on ten or more distinct values and a point (dot-and-whisker) plot otherwise; option TRUE forces a point plot.
#' @param sims Number of independent simulation draws used to calculate upper and lower bounds of coefficient estimates: lower values run faster; higher values produce smoother curves.
#' @param xmin A numerical value indicating the minimum value shown of x shown in the graph. Rarely used.
#' @param xmax A numerical value indicating the maximum value shown of x shown in the graph. Rarely used.
#' @param ercolor A character value indicating the outline color of the whisker or ribbon.
#' @param esize A numerical value indicating the size of the whisker or ribbon.
#' @param ralpha A numerical value indicating the transparency of the ribbon.
#' @param rfill A character value indicating the filling color of the ribbon.
#' @param stats_cp A character value indicating what statistics to present as the plot note. Three options are available: "none", "ci", and "ks". The default is "none". See the Details for more information.
#' @param txt_caption A character string to add a note for the plot, a value will sending to \code{ggplot2::labs(caption = txt_caption))}.
#' @param facet_labs An optional character vector of facet labels to be used when plotting an interaction with a factor variable.
#' @param var3 An optional name (as a string) of a third variable for a three-way interaction \code{var1 * var2 * var3}, pooled across imputations. When supplied, the conditional effect of \code{var1} across \code{var2} is shown at several values/levels of \code{var3}. The default \code{NULL} gives the standard two-way behavior. Requires continuous \code{var1} and \code{var2}.
#' @param var3_vals An optional numeric vector giving the values of a continuous \code{var3} to condition on. The default is the mean and the mean plus or minus one standard deviation. Ignored when \code{var3} is a factor.
#' @param facet A logical value, used only with \code{var3}. \code{TRUE} (default) draws one panel per value/level of \code{var3}; \code{FALSE} overlays the curves colored by \code{var3}.
#' @param ... Other ggplot aesthetics arguments for points in the dot-whisker plot or lines in the line-ribbon plots. Not currently used.
#'
#' @details \code{interplot.mlmmi} and \code{interplot.gmlmmi} are S3 methods from the \code{interplot}. It works on lists of mixed-effects objects with class \code{lmerMod} and \code{glmerMod} generated by \code{mitools} and \code{lme4}.
#'
#' Because the output function is based on \code{\link[ggplot2]{ggplot}}, any additional arguments and layers supported by \code{ggplot2} can be added with the \code{+}.
#'
#' \code{interplot} visualizes the conditional effect based on simulated marginal effects. The simulation provides a probabilistic distribution of moderation effect of the conditioning variable (\code{var2}) at every preset values (including the minimum and maximum values) of the conditioned variable (\code{var1}), denoted as Emin and Emax. This output allows the function to further examine the conditional effect statistically in two ways. One is to examine if the distribution of \eqn{Emax - Emin} covers zero. The other is to directly compare Emin and Emax through statistical tools for distributional comparisons. Users can choose either method by setting the argument \code{stats_cp} to "ci" or "ks".
#' \itemize{
#'   \item "ci" provides the confidence interval of the difference of \eqn{Emax - Emin}. An interval including 0 suggests no statistical difference before and after the conditional effect is applied, and vise versa.
#'   \item "ks" presents the result of a two-sample Kolmogorov-Smirnov test of the simulated distributions of Emin and Emax. The output includes a D statistics and a p-value of the null hypothesis that the two distributions come from the same distribution at the 0.05 level.
#' }
#'
#' See an illustration in the package vignette.
#'
#'
#' @return The function returns a \code{ggplot} object.
#'
#' @importFrom abind abind
#' @importFrom arm sim
#' @importFrom stats quantile median plogis model.matrix ks.test
#' @importFrom purrr map map2 list_c list_rbind
#' @import ggplot2
#' @import dplyr
#'
#'
#' @export

# Coding function for mlm, mi objects
interplot.mlmmi <- function(m, var1, var2, plot = TRUE, steps = NULL,
                            ci = .95, adjCI = FALSE, hist = FALSE,
                            var2_dt = NA, predPro = FALSE, var2_vals = NULL,
                            point = FALSE, sims = 5000, xmin = NA, xmax = NA,
                            ercolor = NA, esize = 0.5, ralpha = 0.5,
                            rfill = "grey70", stats_cp = "none",
                            txt_caption = NULL, facet_labs = NULL,
                            var3 = NULL, var3_vals = NULL, facet = TRUE, ...) {

  if (predPro == TRUE)
    stop("Predicted probability is estimated only for general linear models.")

  # Combine simulations from multiple imputed models
  m.list <- m
  m <- m.list[[1]]
  class(m.list) <- class(m)
  m.sims.list <- map(m.list, \(i) arm::sim(i, sims))
  m.sims <- m.sims.list[[1]]

  for (i in 2:length(m.sims.list)) {
    m.sims@fixef <- rbind(m.sims@fixef, m.sims.list[[i]]@fixef)
    m.sims@ranef[[1]] <- abind::abind(m.sims@ranef[[1]], m.sims.list[[i]]@ranef[[1]],
                                       along = 1)
  }

  coef_names <- unlist(dimnames(m@pp$X)[2])

  ## For factor base terms ####
  orig_var1 <- var1
  orig_var2 <- var2

  factor_v1 <- is.factor(m@frame[[var1]])
  factor_v2 <- is.factor(m@frame[[var2]])

  if (factor_v1 & factor_v2)
    stop("The function does not support interactions between two factors.")

  # Three-way and nonlinear paths (pooled across imputations) ####
  fixef_draws <- m.sims@fixef
  colnames(fixef_draws) <- coef_names
  out <- interplot_mi_extra(
    m = m, m.list = m.list, draws_coef = fixef_draws, frame = m@frame,
    tt_fixed = stats::terms(m, fixed.only = TRUE), coef_fun = lme4::fixef,
    vcov_fun = stats::vcov, df_resid = NULL, var1 = var1, var2 = var2,
    var3 = var3, var3_vals = var3_vals, facet = facet, factor_v1 = factor_v1,
    factor_v2 = factor_v2, predPro = predPro, plot = plot, steps = steps,
    ci = ci, adjCI = adjCI, hist = hist, var2_dt = var2_dt, point = point,
    sims = sims, xmin = xmin, xmax = xmax, ercolor = ercolor, esize = esize,
    ralpha = ralpha, rfill = rfill, stats_cp = stats_cp,
    txt_caption = txt_caption, orig_var1 = orig_var1, orig_var2 = orig_var2, ...
  )
  if (!is.null(out)) return(out$value)

  if (factor_v1) {
    var1 <- paste0(var1, levels(m@frame[[var1]]))
  } else if (factor_v2) {
    var2 <- paste0(var2, levels(m@frame[[var2]]))
  }

  # Build interaction term names
  if (factor_v1 | factor_v2) {
    var12 <- paste0(var2, ":", var1)[-1]
    for (i in seq(var12)) {
      if (!var12[i] %in% coef_names)
        var12[i] <- paste0(var1, ":", var2)[-1][i]
      if (!var12[i] %in% coef_names)
        stop(paste("Model does not include the interaction of", var1, "and", var2, "."))
    }
  } else {
    if (var1 == var2) {
      var12 <- paste0("I(", var1, "^2)")
    } else {
      var12 <- paste0(var2, ":", var1)
    }
    if (!var12 %in% coef_names)
      var12 <- paste0(var1, ":", var2)
    if (!var12 %in% coef_names)
      stop(paste("Model does not include the interaction of", var1, "and", var2, "."))
  }

  # Set the min, max, and steps ####
  if (factor_v2) {
    xmin <- 0
    xmax <- 1
    steps <- 2
  } else {
    if (is.na(xmin)) xmin <- min(m@frame[[var2]], na.rm = TRUE)
    if (is.na(xmax)) xmax <- max(m@frame[[var2]], na.rm = TRUE)
    if (is.null(steps)) {
      steps <- length(unique(na.omit(m@frame[[var2]])))
    }
    if (steps > 100) steps <- 100
  }

  fake_seq <- seq(xmin, xmax, length.out = steps)
  ci_diff <- NULL
  ks_diff <- NULL

  # Calculate the effects ####
  if (factor_v1) {
    results <- map(seq(var1)[-length(var1)], \(j) {
      sim_v1 <- m.sims@fixef[, match(var1[j + 1], coef_names)]
      sim_v12 <- m.sims@fixef[, match(var12[j], coef_names)]
      coef <- sim_coef_vec(sim_v1, sim_v12, fake_seq, ci)
      d <- sim_diff_stats(sim_v1, sim_v12, xmin, xmax, ci)
      coef$value <- var1[j + 1]
      list(coef = coef, ci_diff = d$ci_diff)
    })

    coef_df <- map(results, "coef") |> list_rbind()
    ci_diff <- map(results, "ci_diff") |> setNames(var12)

    if (plot == TRUE) {
      if (hist == TRUE & all(is.na(var2_dt))) var2_dt <- m@frame[[var2]]
      if (is.null(facet_labs)) facet_labs <- unique(coef_df$value)

      if (stats_cp == "ci") {
        facet_labs <- map2(ci_diff, facet_labs, \(aCI, aLevel) {
          paste0(aLevel, "\n CI(Max - Min): [",
                 round(aCI[1], digits = 3), ", ",
                 round(aCI[2], digits = 3), "]")
        }) |> list_c()
      }

      coef_df$value <- factor(coef_df$value, labels = facet_labs)
      interplot.plot(m = coef_df, var1 = orig_var1, var2 = orig_var2,
                     hist = hist, steps = steps, var2_dt = var2_dt,
                     point = point, ercolor = ercolor, esize = esize,
                     ralpha = ralpha, rfill = rfill, ci_diff = ci_diff,
                     ks_diff = ks_diff, stats_cp = stats_cp,
                     txt_caption = txt_caption, ...) +
        facet_grid(. ~ value)
    } else {
      names(coef_df)[1:4] <- c(var2, "coef", "ub", "lb")
      return(list(df_coef = coef_df, stats_ci = ci_diff))
    }

  } else if (factor_v2) {
    results <- map(seq(var2)[-length(var2)], \(j) {
      sim_v1 <- m.sims@fixef[, match(var1, coef_names)]
      sim_v12 <- m.sims@fixef[, match(var12[j], coef_names)]
      coef <- sim_coef_vec(sim_v1, sim_v12, fake_seq, ci)
      d <- sim_diff_stats(sim_v1, sim_v12, xmin, xmax, ci)
      coef$value <- var2[j + 1]
      list(coef = coef, ci_diff = d$ci_diff)
    })

    coef_df <- map(results, "coef") |> list_rbind()
    ci_diff <- map(results, "ci_diff") |> setNames(var12)

    if (plot == TRUE) {
      if (hist == TRUE & all(is.na(var2_dt))) var2_dt <- m@frame[[var2]]
      if (is.null(facet_labs)) facet_labs <- unique(coef_df$value)

      if (stats_cp == "ci") {
        facet_labs <- map2(ci_diff, facet_labs, \(aCI, aLevel) {
          paste0(aLevel, "\n CI(Max - Min): [",
                 round(aCI[1], digits = 3), ", ",
                 round(aCI[2], digits = 3), "]")
        }) |> list_c()
      }

      coef_df$value <- factor(coef_df$value, labels = facet_labs)
      interplot.plot(m = coef_df, var1 = orig_var1, var2 = orig_var2,
                     steps = steps, hist = hist, var2_dt = var2_dt,
                     point = point, ercolor = ercolor, esize = esize,
                     ralpha = ralpha, rfill = rfill, ci_diff = ci_diff,
                     ks_diff = ks_diff, stats_cp = stats_cp,
                     txt_caption = txt_caption, ...) +
        facet_grid(. ~ value)
    } else {
      names(coef_df)[1:4] <- c(var2, "coef", "ub", "lb")
      return(list(df_coef = coef_df, stats_ci = ci_diff))
    }

  } else {
    ## Numeric case
    multiplier <- if (var1 == var2) 2 else 1

    sim_v1 <- m.sims@fixef[, match(var1, coef_names)]
    sim_v12 <- m.sims@fixef[, match(var12, coef_names)]

    coef <- sim_coef_vec(sim_v1, sim_v12, fake_seq, ci, multiplier)

    diff_stats <- sim_diff_stats(sim_v1, sim_v12, xmin, xmax, ci, multiplier)
    ci_diff <- diff_stats$ci_diff
    ks_diff <- diff_stats$ks_diff

    if (plot == TRUE) {
      if (hist == TRUE & all(is.na(var2_dt))) var2_dt <- m@frame[[var2]]
      interplot.plot(m = coef, var1 = orig_var1, var2 = orig_var2,
                     hist = hist, var2_dt = var2_dt, point = point,
                     ercolor = ercolor, esize = esize, ralpha = ralpha,
                     rfill = rfill, ci_diff = ci_diff, ks_diff = ks_diff,
                     stats_cp = stats_cp, txt_caption = txt_caption, ...)
    } else {
      names(coef) <- c(var2, "coef", "ub", "lb")
      return(list(df_coef = coef, stats_ci = ci_diff))
    }
  }
}


#' @export
interplot.gmlmmi <- function(m, var1, var2, plot = TRUE, steps = NULL,
                             ci = .95, adjCI = FALSE, hist = FALSE,
                             var2_dt = NA, predPro = FALSE, var2_vals = NULL,
                             point = FALSE, sims = 5000, xmin = NA, xmax = NA,
                             ercolor = NA, esize = 0.5, ralpha = 0.5,
                             rfill = "grey70", stats_cp = "none",
                             txt_caption = NULL, facet_labs = NULL,
                             var3 = NULL, var3_vals = NULL, facet = TRUE, ...) {

  # Combine simulations from multiple imputed models
  m.list <- m
  m <- m.list[[1]]
  class(m.list) <- class(m)
  m.sims.list <- map(m.list, \(i) arm::sim(i, sims))
  m.sims <- m.sims.list[[1]]

  for (i in 2:length(m.sims.list)) {
    m.sims@fixef <- rbind(m.sims@fixef, m.sims.list[[i]]@fixef)
    m.sims@ranef[[1]] <- abind::abind(m.sims@ranef[[1]], m.sims.list[[i]]@ranef[[1]],
                                       along = 1)
  }

  coef_names <- unlist(dimnames(m@pp$X)[2])

  ## For factor base terms ####
  orig_var1 <- var1
  orig_var2 <- var2

  factor_v1 <- is.factor(m@frame[[var1]])
  factor_v2 <- is.factor(m@frame[[var2]])

  if (factor_v1 & factor_v2)
    stop("The function does not support interactions between two factors.")
  if ((factor_v1 | factor_v2) & predPro == TRUE)
    stop("The current version does not support estimating predicted probabilities for factor base terms.")

  # Three-way and nonlinear paths (pooled across imputations) ####
  fixef_draws <- m.sims@fixef
  colnames(fixef_draws) <- coef_names
  out <- interplot_mi_extra(
    m = m, m.list = m.list, draws_coef = fixef_draws, frame = m@frame,
    tt_fixed = stats::terms(m, fixed.only = TRUE), coef_fun = lme4::fixef,
    vcov_fun = stats::vcov, df_resid = NULL, var1 = var1, var2 = var2,
    var3 = var3, var3_vals = var3_vals, facet = facet, factor_v1 = factor_v1,
    factor_v2 = factor_v2, predPro = predPro, plot = plot, steps = steps,
    ci = ci, adjCI = adjCI, hist = hist, var2_dt = var2_dt, point = point,
    sims = sims, xmin = xmin, xmax = xmax, ercolor = ercolor, esize = esize,
    ralpha = ralpha, rfill = rfill, stats_cp = stats_cp,
    txt_caption = txt_caption, orig_var1 = orig_var1, orig_var2 = orig_var2, ...
  )
  if (!is.null(out)) return(out$value)

  if (factor_v1) {
    var1 <- paste0(var1, levels(m@frame[[var1]]))
  } else if (factor_v2) {
    var2 <- paste0(var2, levels(m@frame[[var2]]))
  }

  # Build interaction term names
  if (factor_v1 | factor_v2) {
    var12 <- paste0(var2, ":", var1)[-1]
    for (i in seq(var12)) {
      if (!var12[i] %in% coef_names)
        var12[i] <- paste0(var1, ":", var2)[-1][i]
      if (!var12[i] %in% coef_names)
        stop(paste("Model does not include the interaction of", var1, "and", var2, "."))
    }
  } else {
    if (var1 == var2) {
      var12 <- paste0("I(", var1, "^2)")
    } else {
      var12 <- paste0(var2, ":", var1)
    }
    if (!var12 %in% coef_names)
      var12 <- paste0(var1, ":", var2)
    if (!var12 %in% coef_names)
      stop(paste("Model does not include the interaction of", var1, "and", var2, "."))
  }

  # Set the min, max, and steps ####
  if (factor_v2) {
    xmin <- 0
    xmax <- 1
    steps <- 2
  } else {
    if (is.na(xmin)) xmin <- min(m@frame[[var2]], na.rm = TRUE)
    if (is.na(xmax)) xmax <- max(m@frame[[var2]], na.rm = TRUE)
    if (is.null(steps)) {
      steps <- length(unique(na.omit(m@frame[[var2]])))
    }
    if (steps > 100) steps <- 100
  }

  fake_seq <- seq(xmin, xmax, length.out = steps)
  ci_diff <- NULL
  ks_diff <- NULL

  # Calculate the effects ####
  if (factor_v1) {
    results <- map(seq(var1)[-length(var1)], \(j) {
      sim_v1 <- m.sims@fixef[, match(var1[j + 1], coef_names)]
      sim_v12 <- m.sims@fixef[, match(var12[j], coef_names)]
      coef <- sim_coef_vec(sim_v1, sim_v12, fake_seq, ci)
      d <- sim_diff_stats(sim_v1, sim_v12, xmin, xmax, ci)
      coef$value <- var1[j + 1]
      list(coef = coef, ci_diff = d$ci_diff)
    })

    coef_df <- map(results, "coef") |> list_rbind()
    ci_diff <- map(results, "ci_diff") |> setNames(var12)

    if (plot == TRUE) {
      if (hist == TRUE & all(is.na(var2_dt))) var2_dt <- m@frame[[var2]]
      if (is.null(facet_labs)) facet_labs <- unique(coef_df$value)

      if (stats_cp == "ci") {
        facet_labs <- map2(ci_diff, facet_labs, \(aCI, aLevel) {
          paste0(aLevel, "\n CI(Max - Min): [",
                 round(aCI[1], digits = 3), ", ",
                 round(aCI[2], digits = 3), "]")
        }) |> list_c()
      }

      coef_df$value <- factor(coef_df$value, labels = facet_labs)
      interplot.plot(m = coef_df, var1 = orig_var1, var2 = orig_var2,
                     steps = steps, hist = hist, var2_dt = var2_dt,
                     point = point, ercolor = ercolor, esize = esize,
                     ralpha = ralpha, rfill = rfill, ci_diff = ci_diff,
                     ks_diff = ks_diff, stats_cp = stats_cp,
                     txt_caption = txt_caption, ...) +
        facet_grid(. ~ value)
    } else {
      names(coef_df)[1:4] <- c(var2, "coef", "ub", "lb")
      return(list(df_coef = coef_df, stats_ci = ci_diff))
    }

  } else if (factor_v2) {
    results <- map(seq(var2)[-length(var2)], \(j) {
      sim_v1 <- m.sims@fixef[, match(var1, coef_names)]
      sim_v12 <- m.sims@fixef[, match(var12[j], coef_names)]
      coef <- sim_coef_vec(sim_v1, sim_v12, fake_seq, ci)
      d <- sim_diff_stats(sim_v1, sim_v12, xmin, xmax, ci)
      coef$value <- var2[j + 1]
      list(coef = coef, ci_diff = d$ci_diff)
    })

    coef_df <- map(results, "coef") |> list_rbind()
    ci_diff <- map(results, "ci_diff") |> setNames(var12)

    if (plot == TRUE) {
      if (hist == TRUE & all(is.na(var2_dt))) var2_dt <- m@frame[[var2]]
      if (is.null(facet_labs)) facet_labs <- unique(coef_df$value)

      if (stats_cp == "ci") {
        facet_labs <- map2(ci_diff, facet_labs, \(aCI, aLevel) {
          paste0(aLevel, "\n CI(Max - Min): [",
                 round(aCI[1], digits = 3), ", ",
                 round(aCI[2], digits = 3), "]")
        }) |> list_c()
      }

      coef_df$value <- factor(coef_df$value, labels = facet_labs)
      interplot.plot(m = coef_df, var1 = orig_var1, var2 = orig_var2,
                     steps = steps, hist = hist, var2_dt = var2_dt,
                     point = point, ercolor = ercolor, esize = esize,
                     ralpha = ralpha, rfill = rfill, ci_diff = ci_diff,
                     ks_diff = ks_diff, stats_cp = stats_cp,
                     txt_caption = txt_caption, ...) +
        facet_grid(. ~ value)
    } else {
      names(coef_df)[1:4] <- c(var2, "coef", "ub", "lb")
      return(list(df_coef = coef_df, stats_ci = ci_diff))
    }

  } else {
    ## Numeric case
    multiplier <- if (var1 == var2) 2 else 1

    if (predPro == TRUE) {
      if (is.null(var2_vals))
        stop("The predicted probabilities cannot be estimated without defining 'var2_vals'.")

      df <- data.frame(m@frame)
      df[[names(m@flist)]] <- NULL
      if (any(grepl("X.weights.", names(df))))
        df <- select(df, -X.weights.)
      df_temp <- select(df, 1)
      df <- df[-1] |>
        map(\(v) {
          if (is.factor(v)) {
            as.data.frame(model.matrix(~ v - 1)[, -1, drop = FALSE])
          } else {
            as.numeric(v)
          }
        })

      for (i in seq(df)) {
        if (!is.data.frame(df[[i]])) {
          namesUpdate <- c(names(df_temp), names(df)[[i]])
          df_temp <- cbind(df_temp, df[[i]])
          names(df_temp) <- namesUpdate
        } else {
          df_temp <- cbind(df_temp, df[[i]])
        }
      }
      df <- df_temp

      names(df)[1] <- "(Intercept)"
      df$`(Intercept)` <- 1

      if (var1 == var2) {
        names(df) <- sub("I\\.(.*)\\.2\\.", "I\\(\\1\\^2\\)", names(df))
      }

      iv_medians <- summarize(df, across(everything(), \(x) median(x, na.rm = TRUE)))
      fake_data <- iv_medians[rep(1, each = steps * length(var2_vals)), ]
      fake_data[[var1]] <- rep(
        seq(min(df[[var1]]), max(df[[var1]]), length.out = steps),
        times = length(var2_vals)
      )
      fake_data[[var2]] <- rep(var2_vals, each = steps)
      fake_data[[var12]] <- fake_data[[var1]] * fake_data[[var2]]

      linpred <- data.matrix(fake_data) %*% t(data.matrix(m.sims@fixef))
      pp_mat <- plogis(linpred)
      probs <- c((1 - ci) / 2, 1 - (1 - ci) / 2)

      coef <- data.frame(
        fake = fake_data[[var1]],
        value = as.factor(fake_data[[var2]]),
        coef1 = rowMeans(pp_mat) * 100,
        lb = apply(pp_mat, 1, quantile, probs = probs[1]) * 100,
        ub = apply(pp_mat, 1, quantile, probs = probs[2]) * 100
      )
    } else {
      sim_v1 <- m.sims@fixef[, match(var1, coef_names)]
      sim_v12 <- m.sims@fixef[, match(var12, coef_names)]

      coef <- sim_coef_vec(sim_v1, sim_v12, fake_seq, ci, multiplier)

      diff_stats <- sim_diff_stats(sim_v1, sim_v12, xmin, xmax, ci, multiplier)
      ci_diff <- diff_stats$ci_diff
      ks_diff <- diff_stats$ks_diff
    }

    if (plot == TRUE) {
      if (hist == TRUE & all(is.na(var2_dt))) var2_dt <- m@frame[[var2]]
      interplot.plot(m = coef, var1 = orig_var1, var2 = orig_var2,
                     steps = steps, hist = hist, predPro = predPro,
                     var2_vals = var2_vals, var2_dt = var2_dt, point = point,
                     ercolor = ercolor, esize = esize, ralpha = ralpha,
                     rfill = rfill, ci_diff = ci_diff, ks_diff = ks_diff,
                     stats_cp = stats_cp, txt_caption = txt_caption, ...)
    } else {
      if (predPro == TRUE) {
        names(coef) <- c(var2, paste0("values_in_", var1), "coef", "ub", "lb")
      } else {
        names(coef) <- c(var2, "coef", "ub", "lb")
      }
      return(list(df_coef = coef, stats_ci = ci_diff))
    }
  }
}

if(getRversion() >= "2.15.1") utils::globalVariables(c("fake", "coef1", "lb", "ub"))

#' Add a Binning Diagnostic to an interplot
#'
#' Overlays within-bin estimates of the conditional effect of \code{var1} on an
#' \code{interplot} output, following the binning estimator of Hainmueller,
#' Mummolo, and Xu (2019). It provides a visual check of the linear-interaction-
#' effect (LIE) assumption: if the moderating relationship is truly linear, the
#' binned point estimates fall on the interplot line; systematic departures
#' signal a nonlinear conditional effect.
#'
#' @param m A model object of class \code{lm} or \code{glm} including the
#'   interaction of \code{var1} and \code{var2}.
#' @param var1 The name (as a string) of the variable whose conditional effect is plotted.
#' @param var2 The name (as a string) of the moderating variable.
#' @param ci A numeric value defining the confidence level. The default is 0.95.
#' @param bins The number of moderator bins (quantile groups). The default is 3
#'   (low / medium / high terciles).
#' @param point_color Color of the binned points and whiskers. Default \code{"#BD472A"}.
#' @param point_shape Plotting shape of the binned points. Default \code{18} (filled diamond).
#'
#' @details For each quantile bin of \code{var2}, the model is refitted on the
#'   observations in that bin with \code{var2} centered at the bin median. The
#'   coefficient on \code{var1} is then its marginal effect evaluated at the bin
#'   median, estimated from only that bin's data; this is algebraically the
#'   Hainmueller-Mummolo-Xu L-estimator. Each estimate is drawn as a
#'   dot-and-whisker at the bin median.
#'
#'   Bins with singular or failed fits are dropped with a warning.
#'
#' @return A list of \code{ggplot2} layers, to be added to an \code{interplot}
#'   plot with \code{+}.
#'
#' @examples
#' m <- lm(mpg ~ wt * cyl, data = mtcars)
#' interplot(m, "cyl", "wt") + bin_layer(m, "cyl", "wt")
#'
#' @source Hainmueller, Jens, Jonathan Mummolo, and Yiqing Xu. 2019. "How Much
#'   Should We Trust Estimates from Multiplicative Interaction Models? Simple
#'   Tools to Improve Empirical Practice." Political Analysis 27(2): 163--192.
#'
#' @importFrom stats quantile median coef vcov update df.residual qnorm qt
#' @importFrom purrr map list_rbind
#' @import ggplot2
#' @export
bin_layer <- function(m, var1, var2, ci = 0.95, bins = 3,
                      point_color = "#BD472A", point_shape = 18) {

  if (!inherits(m, c("lm", "glm")))
    stop("bin_layer currently supports lm and glm objects.")

  mdata <- m$model
  if (is.factor(mdata[[var1]]) || is.factor(mdata[[var2]]))
    stop("bin_layer requires continuous var1 and var2.")
  if (var1 == var2)
    stop("bin_layer is not applicable to quadratic terms (var1 == var2).")

  z <- mdata[[var2]]
  breaks <- unique(stats::quantile(z, probs = seq(0, 1, length.out = bins + 1), na.rm = TRUE))
  if (length(breaks) - 1 < 2)
    stop("var2 does not have enough distinct values to form ", bins, " bins.")
  bin_id <- cut(z, breaks = breaks, include.lowest = TRUE, labels = FALSE)

  use_z <- inherits(m, "glm")

  est <- map(sort(unique(bin_id[!is.na(bin_id)])), \(j) {
    sub <- mdata[!is.na(bin_id) & bin_id == j, , drop = FALSE]
    zmed <- stats::median(sub[[var2]], na.rm = TRUE)
    sub[[var2]] <- sub[[var2]] - zmed # center so coef on var1 is the ME at the bin median

    mb <- tryCatch(stats::update(m, data = sub), error = function(e) NULL)
    if (is.null(mb) || !var1 %in% names(stats::coef(mb))) return(NULL)

    eff <- stats::coef(mb)[[var1]]
    se <- sqrt(stats::vcov(mb)[var1, var1])
    crit <- if (use_z) stats::qnorm(1 - (1 - ci) / 2)
            else stats::qt(1 - (1 - ci) / 2, stats::df.residual(mb))

    data.frame(fake = zmed, coef1 = eff, lb = eff - crit * se, ub = eff + crit * se)
  }) |> list_rbind()

  if (is.null(est) || nrow(est) == 0)
    stop("Could not estimate any bins (singular within-bin fits).")
  if (nrow(est) < length(unique(bin_id[!is.na(bin_id)])))
    warning("Some bins were dropped due to singular or failed fits.")

  list(
    geom_errorbar(data = est, aes(x = fake, ymin = lb, ymax = ub),
                  width = 0, linewidth = 0.7, color = point_color, inherit.aes = FALSE),
    geom_point(data = est, aes(x = fake, y = coef1),
               size = 2.5, shape = point_shape, color = point_color, inherit.aes = FALSE)
  )
}

# Feature B2: bin_layer() binning diagnostic.

test_that("bin_layer returns a list of ggplot layers", {
  m <- lm(mpg ~ wt * cyl, data = mtcars)
  L <- bin_layer(m, "cyl", "wt")
  expect_type(L, "list")
  expect_length(L, 2)
  expect_s3_class(L[[1]], "ggproto")
  expect_s3_class(L[[2]], "ggproto")
})

test_that("bin_layer composes with an interplot", {
  m <- lm(mpg ~ wt * cyl, data = mtcars)
  set.seed(1)
  p <- interplot(m, "cyl", "wt") + bin_layer(m, "cyl", "wt")
  expect_s3_class(p, "ggplot")
})

test_that("bin_layer respects the number of bins", {
  m <- lm(mpg ~ wt * cyl, data = mtcars)
  expect_length(bin_layer(m, "cyl", "wt", bins = 4), 2)
})

test_that("bin_layer rejects factors and quadratic self-terms", {
  mt <- mtcars
  mt$cyl_f <- factor(mt$cyl)
  m_f <- lm(mpg ~ wt * cyl_f, data = mt)
  expect_error(bin_layer(m_f, "wt", "cyl_f"), "continuous")
  m <- lm(mpg ~ wt * cyl, data = mtcars)
  expect_error(bin_layer(m, "wt", "wt"), "quadratic")
})

#' Plot Conditional Coefficients of a Variable in an Interaction Term
#' 
#' \code{interplot} is a generic function to produce a plot of the coefficient estimates of one variable in a two-way interaction conditional on the values of the other variable in the interaction term. The function invokes particular \code{methods} which depend on the \code{\link{class}} of the first argument. 
#' 
#' @param m A model object including an interaction term, or, alternately, a data frame generated by an earlier call to interplot using the argument plot = FALSE.
#' @param var1 The name (as a string) of the variable of interest in the interaction term; its conditional coefficient estimates will be plotted.
#' @param var2 The name (as a string) of the other variable in the interaction term
#' @param plot A logical value indicating whether the output is a plot or a dataframe including the conditional coefficient estimates of var1, their upper and lower bounds, and the corresponding values of var2.
#' @param steps Desired length of the sequence. A non-negative number, which for seq and seq.int will be rounded up if fractional. The default is 100 or the unique categories in the \code{var2} (when it is less than 100. Also see \code{\link{unique}}).
#' @param ci A numeric value defining the confidence intervals. The default value is 95\% (0.95).
#' @param adjCI A logical value indication if applying the adjustment of confidence intervals to control the false discovery rate following the Esarey and Sumner (2017) procedure. (See also Benjamini and Hochberg 1995.) The default is FALSE; the plot presents the confidence intervals suggested by Brambor, Clark, and Golder (2006). The functions dealing with multilevel model outputs in this package do not equip with this argument, because there is the controversy on the accurate degrees of freedom for multilevel models, esp. when random effect is engaged and the degrees of freedom is a necessary information to conduct the CI adjustment. See e.g., https://stat.ethz.ch/pipermail/r-help/2006-May/094765.html and https://stat.ethz.ch/pipermail/r-sig-mixed-models/2008q1/000517.html. 
#' @param hist A logical value indicating if there is a histogram of `var2` added at the bottom of the conditional effect plot.
#' @param var2_dt A numerical value indicating the frequency distribution of `var2`. It is only used when `hist == TRUE`. When the object is a model, the default is the distribution of `var2` of the model.  
#' @param predPro A logical value with default of `FALSE`. When the `m` is an object of class `glm` or `glmerMod` and the argument is set to `TRUE`, the function will plot predicted probabilities at the values given by `var2_vals`. 
#' @param var2_vals A numerical value indicating the values the predicted probabilities are estimated, when `predPro` is `TRUE`. 
#' @param point A logical value determining the format of plot. By default, the function produces a line plot when var2 takes on ten or more distinct values and a point (dot-and-whisker) plot otherwise; option TRUE forces a point plot.
#' @param sims Number of independent simulation draws used to calculate upper and lower bounds of coefficient estimates: lower values run faster; higher values produce smoother curves.
#' @param xmin A numerical value indicating the minimum value shown of x shown in the graph. Rarely used.
#' @param xmax A numerical value indicating the maximum value shown of x shown in the graph. Rarely used.
#' @param ercolor A character value indicating the outline color of the whisker or ribbon.
#' @param esize A numerical value indicating the size of the whisker or ribbon.
#' @param ralpha A numerical value indicating the transparency of the ribbon.
#' @param rfill A character value indicating the filling color of the ribbon.
#' @param stats_cp A character value indicating what statistics to present as the plot note. Three options are available: "none", "ci", and "ks". The default is "none". See the Details for more information.
#' @param txt_caption A character string to add a note for the plot, a value will sending to \code{ggplot2::labs(caption = txt_caption))}.
#' @param ... Additional arguments passed to the specific \code{interplot} method. For \code{lm} and \code{glm} objects (see \code{\link{interplot.default}}) these include \code{var3}, \code{var3_vals}, and \code{facet} for three-way interactions. Other ggplot aesthetics arguments for points or lines may also be supplied.
#' 
#' @details \code{interplot} visualizes the changes in the coefficient of one term in a two-way interaction conditioned by the other term. In the current version, the function works with interactions in the following classes of models:
#' \itemize{
#'   \item Ordinary linear models (object class: \code{lm});
#'   \item Generalized linear models (object class: \code{glm});
#'   \item Linear mixed-effects models (object class: \code{lmerMod});
#'   \item Generalized linear mixed-effects models (object class: \code{glmerMod});
#'   \item Ordinary linear models with imputed data (object class: \code{list});
#'   \item Generalized linear models with imputed data (object class: \code{list})
#'   \item Linear mixed-effects models with imputed data (object class: \code{list});
#'   \item Generalized linear mixed-effects models with imputed data (object class: \code{list}).
#' }
#' 
#' For \code{lm} and \code{glm} objects, \code{interplot} additionally supports three-way interactions (supply a third variable via \code{var3}; see \code{\link{interplot.default}}) and nonlinear conditional effects, in which \code{var1} interacts with a polynomial (e.g. \code{I(var2^2)}) or spline (e.g. \code{splines::ns(var2)}) of the moderator. The companion function \code{\link{bin_layer}} adds the Hainmueller, Mummolo, and Xu (2019) binning diagnostic as a composable overlay.
#'
#' The examples below illustrate how methods invoked by this generic deal with different type of objects.
#' 
#' Because the output function is based on \code{\link[ggplot2]{ggplot}}, any additional arguments and layers supported by \code{ggplot2} can be added with the \code{+}. 
#' 
#' \code{interplot} visualizes the conditional effect based on simulated marginal effects. The simulation provides a probabilistic distribution of moderation effect of the conditioning variable (\code{var2}) at every preset values (including the minimum and maximum values) of the conditioned variable (\code{var1}), denoted as Emin and Emax. This output allows the function to further examine the conditional effect statistically in two ways. One is to examine if the distribution of \eqn{Emax - Emin} covers zero. The other is to directly compare Emin and Emax through statistical tools for distributional comparisons. Users can choose either method by setting the argument \code{stats_cp} to "ci" or "ks".
#' \itemize{
#'   \item "ci" provides the confidence interval of the difference of \eqn{Emax - Emin}. An interval including 0 suggests no statistical difference before and after the conditional effect is applied, and vise versa.
#'   \item "ks" presents the result of a two-sample Kolmogorov-Smirnov test of the simulated distributions of Emin and Emax. The output includes a D statistics and a p-value of the null hypothesis that the two distributions come from the same distribution at the 0.05 level.
#' }
#' 
#' See an illustration in the package vignette.
#' 
#' @return The function returns a \code{ggplot} object.
#'
#' 
#' @examples
#' data(mtcars)
#' m_cyl <- lm(mpg ~ wt * cyl, data = mtcars)
#' library(interplot)
#' 
#' # Plot interactions with a continous conditioning variable
#' interplot(m = m_cyl, var1 = 'cyl', var2 = 'wt') +
#' xlab('Automobile Weight (thousands lbs)') +
#' ylab('Estimated Coefficient for Number of Cylinders') +
#' ggtitle('Estimated Coefficient of Engine Cylinders\non Mileage by Automobile Weight') +
#' theme(plot.title = element_text(face='bold'))
#' 
#' 
#' # Plot interactions with a categorical conditioning variable
#' interplot(m = m_cyl, var1 = 'wt', var2 = 'cyl') +
#' xlab('Number of Cylinders') +
#' ylab('Estimated Coefficient for Automobile Weight (thousands lbs)') +
#' ggtitle('Estimated Coefficient of Automobile Weight \non Mileage by Engine Cylinders') +
#' theme(plot.title = element_text(face='bold'))
#'
#' # Three-way interaction: effect of wt across hp, faceted by cyl
#' m_3 <- lm(mpg ~ wt * hp * cyl, data = mtcars)
#' interplot(m_3, var1 = 'wt', var2 = 'hp', var3 = 'cyl')
#'
#' # Nonlinear conditional effect: wt interacted with a quadratic in cyl
#' m_q <- lm(mpg ~ wt * (cyl + I(cyl^2)), data = mtcars)
#' interplot(m_q, var1 = 'wt', var2 = 'cyl')
#'
#' @source Aiken, Leona S., and Stephen G. West. 1991. "Multiple Regression:
#' Testing and Interpreting Interactions". Newbury Park, CA: Sage.
#'
#' Benjamini, Yoav, and Yosef Hochberg. 1995. "Controlling the False
#' Discovery Rate: A Practical and Powerful Approach to Multiple Testing".
#' Journal of the Royal Statistical Society, Series B 57(1): 289--300.
#'
#' Brambor, Thomas, William Roberts Clark, and Matt Golder.
#' "Understanding interaction models: Improving empirical analyses". Political
#' Analysis 14.1 (2006): 63-82.
#'
#' Esarey, Justin, and Jane Lawrence Sumner. 2015. "Marginal Effects in
#' Interaction Models: Determining and Controlling the False Positive Rate".
#' URL: \url{https://jee3.web.rice.edu/interaction-overconfidence.pdf}.
#'
#' @importFrom stats na.omit
#' @export


interplot <- function(m, var1, var2, plot = TRUE, steps = NULL, ci = .95, adjCI = FALSE, hist = FALSE, var2_dt = NA, predPro = FALSE, var2_vals = NULL, point = FALSE, sims = 1000, xmin = NA, xmax = NA, ercolor = NA, esize = 0.5, ralpha = 0.5, rfill = "grey70", stats_cp = "none", txt_caption = NULL, ...) {
  
  
    if (is.list(m) && !is.data.frame(m) && !inherits(m, "lm") && !inherits(m, "brmsfit")) {
        if (inherits(m[[1]], "lmerMod")) {
            class(m) <- "mlmmi"
        } else if (inherits(m[[1]], "glmerMod")) {
            class(m) <- "gmlmmi"
        } else if (inherits(m[[1]], "glm")) {
            class(m) <- "glmmi"
        } else if (inherits(m[[1]], "lm")) {
            class(m) <- "lmmi"
        }
    }

    if (is.data.frame(m))
        class(m) <- "plot"
    
    
    UseMethod("interplot", m)
}

# Regression guard: the existing two-way linear path must keep working exactly
# as before once var3 / nonlinear branches are added. Uses analytic checks
# (robust to R/RNG version) plus within-run determinism.

test_that("two-way linear interplot is reproducible (fixed seed)", {
  m <- lm(mpg ~ wt * cyl, data = mtcars)
  set.seed(324)
  d1 <- interplot(m, "cyl", "wt", plot = FALSE)$df_coef
  set.seed(324)
  d2 <- interplot(m, "cyl", "wt", plot = FALSE)$df_coef
  expect_equal(d1, d2)
})

test_that("two-way linear point estimates match analytic b1 + b12 * z", {
  m <- lm(mpg ~ wt * cyl, data = mtcars)
  set.seed(324)
  d <- interplot(m, "cyl", "wt", plot = FALSE)$df_coef

  expect_named(d, c("cyl", "coef", "ub", "lb"))
  expect_true(all(d$ub >= d$coef & d$coef >= d$lb))

  # Column 1 ("cyl") holds the var2 (wt) sequence; the conditional coefficient
  # of cyl is b_cyl + b_{wt:cyl} * wt.
  expected <- coef(m)["cyl"] + coef(m)["wt:cyl"] * d$cyl
  expect_equal(d$coef, unname(expected), tolerance = 0.15)
})

test_that("var3 = NULL is byte-for-byte identical to omitting var3", {
  m <- lm(mpg ~ wt * cyl, data = mtcars)
  set.seed(99)
  d_omit <- interplot(m, "cyl", "wt", plot = FALSE)$df_coef
  set.seed(99)
  d_null <- interplot(m, "cyl", "wt", var3 = NULL, plot = FALSE)$df_coef
  expect_equal(d_omit, d_null)
})

# Three-way + nonlinear extended to mixed-effects and multiply-imputed methods.
# (brms is excluded here: it requires Stan compilation, too slow for tests.)

test_that("lmer three-way slice matches the fixef formula", {
  skip_if_not_installed("lme4")
  m <- suppressMessages(lme4::lmer(mpg ~ wt * hp * cyl + (1 | carb), data = mtcars))
  set.seed(1)
  c3 <- 6
  d <- interplot(m, "wt", "hp", var3 = "cyl", var3_vals = c3, plot = FALSE)$df_coef
  fe <- lme4::fixef(m)
  v2 <- d$wt
  expected <- fe["wt"] + fe["wt:hp"] * v2 + fe["wt:cyl"] * c3 + fe["wt:hp:cyl"] * v2 * c3
  expect_lt(max(abs(d$coef - unname(expected))), 0.5)
  expect_equal(length(unique(d$value)), 1L)
})

test_that("lmer nonlinear (polynomial) matches the fixef gradient", {
  skip_if_not_installed("lme4")
  m <- suppressMessages(lme4::lmer(mpg ~ wt * (hp + I(hp^2)) + (1 | carb), data = mtcars))
  set.seed(2)
  d <- interplot(m, "wt", "hp", plot = FALSE)$df_coef
  fe <- lme4::fixef(m)
  z <- d$wt
  expected <- fe["wt"] + fe["wt:hp"] * z + fe["wt:I(hp^2)"] * z^2
  expect_lt(max(abs(d$coef - unname(expected))), 0.5)
})

test_that("lmer nonlinear spline returns a ggplot", {
  skip_if_not_installed("lme4")
  m <- suppressMessages(lme4::lmer(mpg ~ wt * splines::ns(hp, 3) + (1 | carb), data = mtcars))
  set.seed(3)
  expect_s3_class(interplot(m, "wt", "hp"), "ggplot")
})

test_that("lmmi three-way pools across imputations", {
  skip_if_not_installed("lme4")
  set.seed(7)
  imps <- lapply(1:3, function(i) {
    d <- mtcars; d$mpg <- d$mpg + rnorm(nrow(d), 0, 0.5); d
  })
  mi <- lapply(imps, function(d) lm(mpg ~ wt * hp * cyl, data = d))

  d <- interplot(mi, "wt", "hp", var3 = "cyl", var3_vals = 6, plot = FALSE)$df_coef
  me_each <- vapply(mi, function(m) {
    cf <- coef(m); unname(cf["wt"] + cf["wt:hp"] * d$wt + cf["wt:cyl"] * 6 + cf["wt:hp:cyl"] * d$wt * 6)
  }, numeric(nrow(d)))
  expect_lt(max(abs(d$coef - rowMeans(me_each))), 0.5)
})

test_that("lmmi nonlinear pools across imputations", {
  set.seed(8)
  imps <- lapply(1:3, function(i) {
    d <- mtcars; d$mpg <- d$mpg + rnorm(nrow(d), 0, 0.5); d
  })
  mi <- lapply(imps, function(d) lm(mpg ~ wt * (hp + I(hp^2)), data = d))

  d <- interplot(mi, "wt", "hp", plot = FALSE)$df_coef
  me_each <- vapply(mi, function(m) {
    cf <- coef(m); unname(cf["wt"] + cf["wt:hp"] * d$wt + cf["wt:I(hp^2)"] * d$wt^2)
  }, numeric(nrow(d)))
  expect_lt(max(abs(d$coef - rowMeans(me_each))), 0.5)
})

# Feature A: three-way interactions.

test_that("continuous var3 yields three Aiken-West groups", {
  m <- lm(mpg ~ wt * hp * cyl, data = mtcars)
  set.seed(1)
  d <- interplot(m, "wt", "hp", var3 = "cyl", plot = FALSE)$df_coef
  expect_true("value" %in% names(d))
  expect_equal(length(unique(d$value)), 3)
  expect_match(paste(levels(d$value), collapse = " "), "Low|Mean|High")
})

test_that("three-way slice matches the analytic conditional coefficient", {
  m <- lm(mpg ~ wt * hp * cyl, data = mtcars)
  cf <- coef(m)
  c3 <- 6
  set.seed(2)
  d <- interplot(m, "wt", "hp", var3 = "cyl", var3_vals = c3, plot = FALSE)$df_coef
  v2 <- d$wt # column 1 holds the var2 (hp) sequence
  expected <- cf["wt"] + cf["wt:hp"] * v2 + cf["wt:cyl"] * c3 + cf["wt:hp:cyl"] * v2 * c3
  expect_lt(max(abs(d$coef - unname(expected))), 0.3)
})

test_that("factor var3 produces one group per level (incl. reference)", {
  mt <- mtcars
  mt$cyl_f <- factor(mt$cyl)
  m <- lm(mpg ~ wt * hp * cyl_f, data = mt)
  set.seed(3)
  d <- interplot(m, "wt", "hp", var3 = "cyl_f", plot = FALSE)$df_coef
  expect_equal(length(unique(d$value)), 3)
})

test_that("facet and overlay both return ggplot objects", {
  m <- lm(mpg ~ wt * hp * cyl, data = mtcars)
  set.seed(4)
  expect_s3_class(interplot(m, "wt", "hp", var3 = "cyl"), "ggplot")
  expect_s3_class(interplot(m, "wt", "hp", var3 = "cyl", facet = FALSE), "ggplot")
})

test_that("custom var3_vals control the number of groups", {
  m <- lm(mpg ~ wt * hp * cyl, data = mtcars)
  set.seed(5)
  d <- interplot(m, "wt", "hp", var3 = "cyl", var3_vals = c(4, 8), plot = FALSE)$df_coef
  expect_equal(length(unique(d$value)), 2)
})

test_that("three distinct variables are required", {
  m <- lm(mpg ~ wt * hp * cyl, data = mtcars)
  expect_error(interplot(m, "wt", "hp", var3 = "wt"), "distinct")
})

if(getRversion() >= "2.15.1") utils::globalVariables(c(".", "X.weights."))

#' Plot Conditional Coefficients in (Generalized) Linear Models with Imputed Data and Interaction Terms
#'
#' \code{interplot.mi} is a method to calculate conditional coefficient estimates from the results of (generalized) linear regression models with interaction terms and multiply imputed data.
#'
#' @param m A model object including an interaction term, or, alternately, a data frame recording conditional coefficients.
#' @param var1 The name (as a string) of the variable of interest in the interaction term; its conditional coefficient estimates will be plotted.
#' @param var2 The name (as a string) of the other variable in the interaction term.
#' @param plot A logical value indicating whether the output is a plot or a dataframe including the conditional coefficient estimates of var1, their upper and lower bounds, and the corresponding values of var2.
#' @param steps Desired length of the sequence. A non-negative number, which for seq and seq.int will be rounded up if fractional. The default is 100 or the unique categories in the \code{var2} (when it is less than 100. Also see \code{\link{unique}}).
#' @param ci A numeric value defining the confidence intervals. The default value is 95\% (0.95).
#' @param adjCI A logical value indication if applying the adjustment of confidence intervals to control the false discovery rate following the Esarey and Sumner (2017) procedure. (See also Benjamini and Hochberg 1995.) The default is FALSE; the plot presents the confidence intervals suggested by Brambor, Clark, and Golder (2006).
#' @param hist A logical value indicating if there is a histogram of `var2` added at the bottom of the conditional effect plot.
#' @param var2_dt A numerical value indicating the frequency distribution of `var2`. It is only used when `hist == TRUE`. When the object is a model, the default is the distribution of `var2` of the model.
#' @param predPro A logical value with default of `FALSE`. When the `m` is an object of class `glm` and the argument is set to `TRUE`, the function will plot predicted probabilities at the values given by `var2_vals`.
#' @param var2_vals A numerical value indicating the values the predicted probabilities are estimated, when `predPro` is `TRUE`.
#' @param point A logical value determining the format of plot. By default, the function produces a line plot when var2 takes on ten or more distinct values and a point (dot-and-whisker) plot otherwise; option TRUE forces a point plot.
#' @param sims Number of independent simulation draws used to calculate upper and lower bounds of coefficient estimates: lower values run faster; higher values produce smoother curves.
#' @param xmin A numerical value indicating the minimum value shown of x shown in the graph. Rarely used.
#' @param xmax A numerical value indicating the maximum value shown of x shown in the graph. Rarely used.
#' @param ercolor A character value indicating the outline color of the whisker or ribbon.
#' @param esize A numerical value indicating the size of the whisker or ribbon.
#' @param ralpha A numerical value indicating the transparency of the ribbon.
#' @param rfill A character value indicating the filling color of the ribbon.
#' @param stats_cp A character value indicating what statistics to present as the plot note. Three options are available: "none", "ci", and "ks". The default is "none". See the Details for more information.
#' @param txt_caption A character string to add a note for the plot, a value will sending to \code{ggplot2::labs(caption = txt_caption))}.
#' @param facet_labs An optional character vector of facet labels to be used when plotting an interaction with a factor variable.
#' @param var3 An optional name (as a string) of a third variable for a three-way interaction \code{var1 * var2 * var3}, pooled across imputations. When supplied, the conditional effect of \code{var1} across \code{var2} is shown at several values/levels of \code{var3}. The default \code{NULL} gives the standard two-way behavior. Requires continuous \code{var1} and \code{var2}.
#' @param var3_vals An optional numeric vector giving the values of a continuous \code{var3} to condition on. The default is the mean and the mean plus or minus one standard deviation. Ignored when \code{var3} is a factor.
#' @param facet A logical value, used only with \code{var3}. \code{TRUE} (default) draws one panel per value/level of \code{var3}; \code{FALSE} overlays the curves colored by \code{var3}.
#' @param ... Other ggplot aesthetics arguments for points in the dot-whisker plot or lines in the line-ribbon plots. Not currently used.
#'
#' @details \code{interplot.lmmi} and \code{interplot.glmmi} are S3 methods from the \code{interplot}. This function can work on interactions from results in the class of \code{list} generated by \code{mitools}.
#'
#' Because the output function is based on \code{\link[ggplot2]{ggplot}}, any additional arguments and layers supported by \code{ggplot2} can be added with the \code{+}.
#'
#' \code{interplot} visualizes the conditional effect based on simulated marginal effects. The simulation provides a probabilistic distribution of moderation effect of the conditioning variable (\code{var2}) at every preset values (including the minimum and maximum values) of the conditioned variable (\code{var1}), denoted as Emin and Emax. This output allows the function to further examine the conditional effect statistically in two ways. One is to examine if the distribution of \eqn{Emax - Emin} covers zero. The other is to directly compare Emin and Emax through statistical tools for distributional comparisons. Users can choose either method by setting the argument \code{stats_cp} to "ci" or "ks".
#' \itemize{
#'   \item "ci" provides the confidence interval of the difference of \eqn{Emax - Emin}. An interval including 0 suggests no statistical difference before and after the conditional effect is applied, and vise versa.
#'   \item "ks" presents the result of a two-sample Kolmogorov-Smirnov test of the simulated distributions of Emin and Emax. The output includes a D statistics and a p-value of the null hypothesis that the two distributions come from the same distribution at the 0.05 level.
#' }
#'
#'
#' See an illustration in the package vignette.
#'
#' @examples
#'
#' library(interplot)
#' library(mitools)
#'
#' data(smi)
#' model1 <- with(smi, glm(drinkreg ~ wave * sex, family = binomial()))
#'
#' interplot(model1, var1 = "sex", var2 = "wave")
#'
#' @return The function returns a \code{ggplot} object.
#'
#' @importFrom arm sim
#' @importFrom stats quantile qnorm median plogis model.matrix ks.test
#' @importFrom purrr map map2 list_c list_rbind
#' @importFrom interactionTest fdrInteraction
#' @import ggplot2
#' @import dplyr
#'
#' @source Benjamini, Yoav, and Yosef Hochberg. 1995. "Controlling the False
#' Discovery Rate: A Practical and Powerful Approach to Multiple Testing".
#' Journal of the Royal Statistical Society, Series B 57(1): 289--300.
#'
#' Brambor, Thomas, William Roberts Clark, and Matt Golder.
#' "Understanding interaction models: Improving empirical analyses". Political
#' Analysis 14.1 (2006): 63-82.
#'
#' Esarey, Justin, and Jane Lawrence Sumner. 2015. "Marginal Effects in
#' Interaction Models: Determining and Controlling the False Positive Rate".
#' URL: \url{https://jee3.web.rice.edu/interaction-overconfidence.pdf}.
#'
#'
#' @export


# Coding function for non-mlm mi objects
interplot.lmmi <- function(m, var1, var2, plot = TRUE, steps = NULL,
                           ci = .95, adjCI = FALSE, hist = FALSE,
                           var2_dt = NA, predPro = FALSE, var2_vals = NULL,
                           point = FALSE, sims = 5000, xmin = NA, xmax = NA,
                           ercolor = NA, esize = 0.5, ralpha = 0.5,
                           rfill = "grey70", stats_cp = "none",
                           txt_caption = NULL, facet_labs = NULL,
                           var3 = NULL, var3_vals = NULL, facet = TRUE, ...) {

  if (predPro == TRUE & inherits(m, "lmmi"))
    stop("Predicted probability is estimated only for general linear models.")

  # Combine simulations from multiple imputed models
  m.list <- m
  m <- m.list[[1]]
  class(m.list) <- class(m)
  m.sims.list <- map(m.list, \(i) arm::sim(i, sims))
  m.sims <- m.sims.list[[1]]

  for (i in 2:length(m.sims.list)) {
    m.sims@coef <- rbind(m.sims@coef, m.sims.list[[i]]@coef)
  }

  ## For factor base terms ####
  orig_var1 <- var1
  orig_var2 <- var2

  factor_v1 <- is.factor(m$model[[var1]])
  factor_v2 <- is.factor(m$model[[var2]])

  if (factor_v1 & factor_v2)
    stop("The function does not support interactions between two factors.")

  # Three-way and nonlinear paths (pooled across imputations) ####
  out <- interplot_mi_extra(
    m = m, m.list = m.list, draws_coef = m.sims@coef, frame = m$model,
    tt_fixed = stats::terms(m), coef_fun = stats::coef, vcov_fun = stats::vcov,
    df_resid = m$df, var1 = var1, var2 = var2, var3 = var3,
    var3_vals = var3_vals, facet = facet, factor_v1 = factor_v1,
    factor_v2 = factor_v2, predPro = predPro, plot = plot, steps = steps,
    ci = ci, adjCI = adjCI, hist = hist, var2_dt = var2_dt, point = point,
    sims = sims, xmin = xmin, xmax = xmax, ercolor = ercolor, esize = esize,
    ralpha = ralpha, rfill = rfill, stats_cp = stats_cp,
    txt_caption = txt_caption, orig_var1 = orig_var1, orig_var2 = orig_var2, ...
  )
  if (!is.null(out)) return(out$value)

  if (factor_v1) {
    var1 <- paste0(var1, m$xlevel[[var1]])
  } else if (factor_v2) {
    var2 <- paste0(var2, m$xlevel[[var2]])
  }

  # Build interaction term names
  if (factor_v1 | factor_v2) {
    var12 <- paste0(var2, ":", var1)[-1]
    for (i in seq(var12)) {
      if (!var12[i] %in% names(m$coef))
        var12[i] <- paste0(var1, ":", var2)[-1][i]
      if (!var12[i] %in% names(m$coef))
        stop(paste("Model does not include the interaction of", var1, "and", var2, "."))
    }
  } else {
    if (var1 == var2) {
      var12 <- paste0("I(", var1, "^2)")
    } else {
      var12 <- paste0(var2, ":", var1)
    }
    if (!var12 %in% names(m$coef))
      var12 <- paste0(var1, ":", var2)
    if (!var12 %in% names(m$coef))
      stop(paste("Model does not include the interaction of", var1, "and", var2, "."))
  }

  # Set the min, max, and steps ####
  if (factor_v2) {
    xmin <- 0
    xmax <- 1
    steps <- 2
  } else {
    if (is.na(xmin)) xmin <- min(m$model[[var2]], na.rm = TRUE)
    if (is.na(xmax)) xmax <- max(m$model[[var2]], na.rm = TRUE)
    if (is.null(steps)) {
      steps <- length(unique(na.omit(m$model[[var2]])))
    }
    if (steps > 100) steps <- 100
  }

  fake_seq <- seq(xmin, xmax, length.out = steps)
  ci_diff <- NULL
  ks_diff <- NULL

  # Calculate the effects ####
  if (factor_v1) {
    results <- map(seq(var1)[-length(var1)], \(j) {
      sim_v1 <- m.sims@coef[, match(var1[j + 1], names(m$coef))]
      sim_v12 <- m.sims@coef[, match(var12[j], names(m$coef))]
      coef <- sim_coef_vec(sim_v1, sim_v12, fake_seq, ci)
      d <- sim_diff_stats(sim_v1, sim_v12, xmin, xmax, ci)

      if (adjCI == TRUE) {
        coef$sd <- (coef$ub - coef$coef1) / qnorm(1 - (1 - ci) / 2)
        tAdj <- fdrInteraction(coef$coef1, coef$sd, df = m$df, level = .95)
        coef$ub <- coef$coef1 + tAdj * coef$sd
        coef$lb <- coef$coef1 - tAdj * coef$sd
      }

      coef$value <- var1[j + 1]
      list(coef = coef, ci_diff = d$ci_diff)
    })

    coef_df <- map(results, "coef") |> list_rbind()
    ci_diff <- map(results, "ci_diff") |> setNames(var12)

    if (plot == TRUE) {
      if (hist == TRUE & all(is.na(var2_dt))) var2_dt <- m$model[[var2]]
      if (is.null(facet_labs)) facet_labs <- unique(coef_df$value)

      if (stats_cp == "ci") {
        facet_labs <- map2(ci_diff, facet_labs, \(aCI, aLevel) {
          paste0(aLevel, "\n CI(Max - Min): [",
                 round(aCI[1], digits = 3), ", ",
                 round(aCI[2], digits = 3), "]")
        }) |> list_c()
      }

      coef_df$value <- factor(coef_df$value, labels = facet_labs)
      interplot.plot(m = coef_df, var1 = orig_var1, var2 = orig_var2,
                     hist = hist, var2_dt = var2_dt, steps = steps,
                     point = point, ercolor = ercolor, esize = esize,
                     ralpha = ralpha, rfill = rfill, ci_diff = ci_diff,
                     ks_diff = ks_diff, stats_cp = stats_cp,
                     txt_caption = txt_caption, ...) +
        facet_grid(. ~ value)
    } else {
      names(coef_df)[1:4] <- c(var2, "coef", "ub", "lb")
      return(list(df_coef = coef_df, stats_ci = ci_diff))
    }

  } else if (factor_v2) {
    results <- map(seq(var2)[-length(var2)], \(j) {
      sim_v1 <- m.sims@coef[, match(var1, names(m$coef))]
      sim_v12 <- m.sims@coef[, match(var12[j], names(m$coef))]
      coef <- sim_coef_vec(sim_v1, sim_v12, fake_seq, ci)
      d <- sim_diff_stats(sim_v1, sim_v12, xmin, xmax, ci)

      if (adjCI == TRUE) {
        coef$sd <- (coef$ub - coef$coef1) / qnorm(1 - (1 - ci) / 2)
        tAdj <- fdrInteraction(coef$coef1, coef$sd, df = m$df, level = .95)
        coef$ub <- coef$coef1 + tAdj * coef$sd
        coef$lb <- coef$coef1 - tAdj * coef$sd
      }

      coef$value <- var2[j + 1]
      list(coef = coef, ci_diff = d$ci_diff)
    })

    coef_df <- map(results, "coef") |> list_rbind()
    ci_diff <- map(results, "ci_diff") |> setNames(var12)

    if (plot == TRUE) {
      if (hist == TRUE & all(is.na(var2_dt))) var2_dt <- m$model[[var2]]
      if (is.null(facet_labs)) facet_labs <- unique(coef_df$value)

      if (stats_cp == "ci") {
        facet_labs <- map2(ci_diff, facet_labs, \(aCI, aLevel) {
          paste0(aLevel, "\n CI(Max - Min): [",
                 round(aCI[1], digits = 3), ", ",
                 round(aCI[2], digits = 3), "]")
        }) |> list_c()
      }

      coef_df$value <- factor(coef_df$value, labels = facet_labs)
      interplot.plot(m = coef_df, var1 = orig_var1, var2 = orig_var2,
                     hist = hist, steps = steps, var2_dt = var2_dt,
                     point = point, ercolor = ercolor, esize = esize,
                     ralpha = ralpha, rfill = rfill, ci_diff = ci_diff,
                     ks_diff = ks_diff, stats_cp = stats_cp,
                     txt_caption = txt_caption, ...) +
        facet_grid(. ~ value)
    } else {
      names(coef_df)[1:4] <- c(var2, "coef", "ub", "lb")
      return(list(df_coef = coef_df, stats_ci = ci_diff))
    }

  } else {
    ## Numeric case
    multiplier <- if (var1 == var2) 2 else 1

    sim_v1 <- m.sims@coef[, match(var1, names(m$coef))]
    sim_v12 <- m.sims@coef[, match(var12, names(m$coef))]

    coef <- sim_coef_vec(sim_v1, sim_v12, fake_seq, ci, multiplier)

    diff_stats <- sim_diff_stats(sim_v1, sim_v12, xmin, xmax, ci, multiplier)
    ci_diff <- diff_stats$ci_diff
    ks_diff <- diff_stats$ks_diff

    if (adjCI == TRUE) {
      coef$sd <- (coef$ub - coef$coef1) / qnorm(1 - (1 - ci) / 2)
      tAdj <- fdrInteraction(coef$coef1, coef$sd, df = m$df, level = .95)
      coef$ub <- coef$coef1 + tAdj * coef$sd
      coef$lb <- coef$coef1 - tAdj * coef$sd
    }

    if (plot == TRUE) {
      if (hist == TRUE & all(is.na(var2_dt))) var2_dt <- m$model[[var2]]
      interplot.plot(m = coef, var1 = orig_var1, var2 = orig_var2,
                     point = point, ercolor = ercolor,
                     esize = esize, ralpha = ralpha, rfill = rfill,
                     ci_diff = ci_diff, ks_diff = ks_diff, stats_cp = stats_cp,
                     txt_caption = txt_caption, ...)
    } else {
      names(coef) <- c(var2, "coef", "ub", "lb")
      return(list(df_coef = coef, stats_ci = ci_diff))
    }
  }
}

#' @export

interplot.glmmi <- function(m, var1, var2, plot = TRUE, steps = NULL,
                            ci = .95, adjCI = FALSE, hist = FALSE,
                            var2_dt = NA, predPro = FALSE, var2_vals = NULL,
                            point = FALSE, sims = 5000, xmin = NA, xmax = NA,
                            ercolor = NA, esize = 0.5, ralpha = 0.5,
                            rfill = "grey70", stats_cp = "none",
                            txt_caption = NULL, facet_labs = NULL,
                            var3 = NULL, var3_vals = NULL, facet = TRUE, ...) {

  # Combine simulations from multiple imputed models
  m.list <- m
  m <- m.list[[1]]
  class(m.list) <- class(m)
  m.sims.list <- map(m.list, \(i) arm::sim(i, sims))
  m.sims <- m.sims.list[[1]]

  for (i in 2:length(m.sims.list)) {
    m.sims@coef <- rbind(m.sims@coef, m.sims.list[[i]]@coef)
  }

  ## For factor base terms ####
  orig_var1 <- var1
  orig_var2 <- var2

  factor_v1 <- is.factor(m$model[[var1]])
  factor_v2 <- is.factor(m$model[[var2]])

  if (factor_v1 & factor_v2)
    stop("The function does not support interactions between two factors.")
  if ((factor_v1 | factor_v2) & predPro == TRUE)
    stop("The current version does not support estimating predicted probabilities for factor base terms.")

  # Three-way and nonlinear paths (pooled across imputations) ####
  out <- interplot_mi_extra(
    m = m, m.list = m.list, draws_coef = m.sims@coef, frame = m$model,
    tt_fixed = stats::terms(m), coef_fun = stats::coef, vcov_fun = stats::vcov,
    df_resid = m$df, var1 = var1, var2 = var2, var3 = var3,
    var3_vals = var3_vals, facet = facet, factor_v1 = factor_v1,
    factor_v2 = factor_v2, predPro = predPro, plot = plot, steps = steps,
    ci = ci, adjCI = adjCI, hist = hist, var2_dt = var2_dt, point = point,
    sims = sims, xmin = xmin, xmax = xmax, ercolor = ercolor, esize = esize,
    ralpha = ralpha, rfill = rfill, stats_cp = stats_cp,
    txt_caption = txt_caption, orig_var1 = orig_var1, orig_var2 = orig_var2, ...
  )
  if (!is.null(out)) return(out$value)

  if (factor_v1) {
    var1 <- paste0(var1, m$xlevel[[var1]])
  } else if (factor_v2) {
    var2 <- paste0(var2, m$xlevel[[var2]])
  }

  # Build interaction term names
  if (factor_v1 | factor_v2) {
    var12 <- paste0(var2, ":", var1)[-1]
    for (i in seq(var12)) {
      if (!var12[i] %in% names(m$coef))
        var12[i] <- paste0(var1, ":", var2)[-1][i]
      if (!var12[i] %in% names(m$coef))
        stop(paste("Model does not include the interaction of", var1, "and", var2, "."))
    }
  } else {
    if (var1 == var2) {
      var12 <- paste0("I(", var1, "^2)")
    } else {
      var12 <- paste0(var2, ":", var1)
    }
    if (!var12 %in% names(m$coef))
      var12 <- paste0(var1, ":", var2)
    if (!var12 %in% names(m$coef))
      stop(paste("Model does not include the interaction of", var1, "and", var2, "."))
  }

  # Set the min, max, and steps ####
  if (factor_v2) {
    xmin <- 0
    xmax <- 1
    steps <- 2
  } else {
    if (is.na(xmin)) xmin <- min(m$model[[var2]], na.rm = TRUE)
    if (is.na(xmax)) xmax <- max(m$model[[var2]], na.rm = TRUE)
    if (is.null(steps)) {
      steps <- length(unique(na.omit(m$model[[var2]])))
    }
    if (steps > 100) steps <- 100
  }

  fake_seq <- seq(xmin, xmax, length.out = steps)
  ci_diff <- NULL
  ks_diff <- NULL

  # Calculate the effects ####
  if (factor_v1) {
    results <- map(seq(var1)[-length(var1)], \(j) {
      sim_v1 <- m.sims@coef[, match(var1[j + 1], names(m$coef))]
      sim_v12 <- m.sims@coef[, match(var12[j], names(m$coef))]
      coef <- sim_coef_vec(sim_v1, sim_v12, fake_seq, ci)
      d <- sim_diff_stats(sim_v1, sim_v12, xmin, xmax, ci)

      if (adjCI == TRUE) {
        coef$sd <- (coef$ub - coef$coef1) / qnorm(1 - (1 - ci) / 2)
        tAdj <- fdrInteraction(coef$coef1, coef$sd, df = m$df, level = .95)
        coef$ub <- coef$coef1 + tAdj * coef$sd
        coef$lb <- coef$coef1 - tAdj * coef$sd
      }

      coef$value <- var1[j + 1]
      list(coef = coef, ci_diff = d$ci_diff)
    })

    coef_df <- map(results, "coef") |> list_rbind()
    ci_diff <- map(results, "ci_diff") |> setNames(var12)

    if (plot == TRUE) {
      if (hist == TRUE & all(is.na(var2_dt))) var2_dt <- m$model[[var2]]
      if (is.null(facet_labs)) facet_labs <- unique(coef_df$value)

      if (stats_cp == "ci") {
        facet_labs <- map2(ci_diff, facet_labs, \(aCI, aLevel) {
          paste0(aLevel, "\n CI(Max - Min): [",
                 round(aCI[1], digits = 3), ", ",
                 round(aCI[2], digits = 3), "]")
        }) |> list_c()
      }

      coef_df$value <- factor(coef_df$value, labels = facet_labs)
      interplot.plot(m = coef_df, var1 = orig_var1, var2 = orig_var2,
                     hist = hist, var2_dt = var2_dt, steps = steps,
                     point = point, ercolor = ercolor, esize = esize,
                     ralpha = ralpha, rfill = rfill, ci_diff = ci_diff,
                     ks_diff = ks_diff, stats_cp = stats_cp,
                     txt_caption = txt_caption, ...) +
        facet_grid(. ~ value)
    } else {
      names(coef_df)[1:4] <- c(var2, "coef", "ub", "lb")
      return(list(df_coef = coef_df, stats_ci = ci_diff))
    }

  } else if (factor_v2) {
    results <- map(seq(var2)[-length(var2)], \(j) {
      sim_v1 <- m.sims@coef[, match(var1, names(m$coef))]
      sim_v12 <- m.sims@coef[, match(var12[j], names(m$coef))]
      coef <- sim_coef_vec(sim_v1, sim_v12, fake_seq, ci)
      d <- sim_diff_stats(sim_v1, sim_v12, xmin, xmax, ci)

      if (adjCI == TRUE) {
        coef$sd <- (coef$ub - coef$coef1) / qnorm(1 - (1 - ci) / 2)
        tAdj <- fdrInteraction(coef$coef1, coef$sd, df = m$df, level = .95)
        coef$ub <- coef$coef1 + tAdj * coef$sd
        coef$lb <- coef$coef1 - tAdj * coef$sd
      }

      coef$value <- var2[j + 1]
      list(coef = coef, ci_diff = d$ci_diff)
    })

    coef_df <- map(results, "coef") |> list_rbind()
    ci_diff <- map(results, "ci_diff") |> setNames(var12)

    if (plot == TRUE) {
      if (hist == TRUE & all(is.na(var2_dt))) var2_dt <- m$model[[var2]]
      if (is.null(facet_labs)) facet_labs <- unique(coef_df$value)

      if (stats_cp == "ci") {
        facet_labs <- map2(ci_diff, facet_labs, \(aCI, aLevel) {
          paste0(aLevel, "\n CI(Max - Min): [",
                 round(aCI[1], digits = 3), ", ",
                 round(aCI[2], digits = 3), "]")
        }) |> list_c()
      }

      coef_df$value <- factor(coef_df$value, labels = facet_labs)
      interplot.plot(m = coef_df, var1 = orig_var1, var2 = orig_var2,
                     hist = hist, steps = steps, var2_dt = var2_dt,
                     point = point, ercolor = ercolor, esize = esize,
                     ralpha = ralpha, rfill = rfill, ci_diff = ci_diff,
                     ks_diff = ks_diff, stats_cp = stats_cp,
                     txt_caption = txt_caption, ...) +
        facet_grid(. ~ value)
    } else {
      names(coef_df)[1:4] <- c(var2, "coef", "ub", "lb")
      return(list(df_coef = coef_df, stats_ci = ci_diff))
    }

  } else {
    ## Numeric case
    multiplier <- if (var1 == var2) 2 else 1

    if (predPro == TRUE) {
      if (is.null(var2_vals))
        stop("The predicted probabilities cannot be estimated without defining 'var2_vals'.")

      df <- data.frame(m$model)
      if (any(grepl("X.weights.", names(df))))
        df <- select(df, -X.weights.)
      df_temp <- select(df, 1)
      df <- df[-1] |>
        map(\(v) {
          if (is.factor(v)) {
            as.data.frame(model.matrix(~ v - 1)[, -1, drop = FALSE])
          } else {
            as.numeric(v)
          }
        })

      for (i in seq(df)) {
        if (!is.data.frame(df[[i]])) {
          namesUpdate <- c(names(df_temp), names(df)[[i]])
          df_temp <- cbind(df_temp, df[[i]])
          names(df_temp) <- namesUpdate
        } else {
          df_temp <- cbind(df_temp, df[[i]])
        }
      }
      df <- df_temp

      if (inherits(m, "polr")) {
        df <- df[, -1]
      } else {
        names(df)[1] <- "(Intercept)"
        df$`(Intercept)` <- 1
      }

      if (var1 == var2) {
        names(df) <- sub("I\\.(.*)\\.2\\.", "I\\(\\1\\^2\\)", names(df))
      }

      iv_medians <- summarize(df, across(everything(), \(x) median(x, na.rm = TRUE)))
      fake_data <- iv_medians[rep(1, each = steps * length(var2_vals)), ]
      fake_data[[var1]] <- rep(
        seq(min(df[[var1]]), max(df[[var1]]), length.out = steps),
        times = length(var2_vals)
      )
      fake_data[[var2]] <- rep(var2_vals, each = steps)
      fake_data[[var12]] <- fake_data[[var1]] * fake_data[[var2]]

      linpred <- data.matrix(fake_data) %*% t(data.matrix(m.sims@coef))
      pp_mat <- plogis(linpred)
      probs <- c((1 - ci) / 2, 1 - (1 - ci) / 2)

      coef <- data.frame(
        fake = fake_data[[var1]],
        value = as.factor(fake_data[[var2]]),
        coef1 = rowMeans(pp_mat) * 100,
        lb = apply(pp_mat, 1, quantile, probs = probs[1]) * 100,
        ub = apply(pp_mat, 1, quantile, probs = probs[2]) * 100
      )
    } else {
      sim_v1 <- m.sims@coef[, match(var1, names(m$coef))]
      sim_v12 <- m.sims@coef[, match(var12, names(m$coef))]

      coef <- sim_coef_vec(sim_v1, sim_v12, fake_seq, ci, multiplier)

      diff_stats <- sim_diff_stats(sim_v1, sim_v12, xmin, xmax, ci, multiplier)
      ci_diff <- diff_stats$ci_diff
      ks_diff <- diff_stats$ks_diff
    }

    if (adjCI == TRUE & predPro == FALSE) {
      coef$sd <- (coef$ub - coef$coef1) / qnorm(1 - (1 - ci) / 2)
      tAdj <- fdrInteraction(coef$coef1, coef$sd, df = m$df, level = .95)
      coef$ub <- coef$coef1 + tAdj * coef$sd
      coef$lb <- coef$coef1 - tAdj * coef$sd
    }

    if (plot == TRUE) {
      if (hist == TRUE & all(is.na(var2_dt))) var2_dt <- m$model[[var2]]
      interplot.plot(m = coef, var1 = orig_var1, var2 = orig_var2,
                     steps = steps, hist = hist, var2_dt = var2_dt,
                     predPro = predPro, var2_vals = var2_vals, point = point,
                     ercolor = ercolor, esize = esize, ralpha = ralpha,
                     rfill = rfill, ci_diff = ci_diff, ks_diff = ks_diff,
                     stats_cp = stats_cp, txt_caption = txt_caption, ...)
    } else {
      if (predPro == TRUE) {
        names(coef) <- c(var2, paste0("values_in_", var1), "coef", "ub", "lb")
      } else {
        names(coef) <- c(var2, "coef", "ub", "lb")
      }
      return(list(df_coef = coef, stats_ci = ci_diff))
    }
  }
}

if(getRversion() >= "2.15.1") utils::globalVariables(c(".", "value", ":="))

#' Plot Conditional Coefficients in Bayesian Models with Interaction Terms
#'
#' \code{interplot.brmsfit} is a method to calculate conditional coefficient
#' estimates from the posterior draws of a Bayesian regression model fitted with
#' \code{\link[brms]{brm}} that includes a two-way interaction term.
#'
#' @param m A model object of class \code{brmsfit} including an interaction term.
#' @param var1 The name (as a string) of the variable of interest in the interaction term; its conditional coefficient estimates will be plotted.
#' @param var2 The name (as a string) of the other variable in the interaction term.
#' @param plot A logical value indicating whether the output is a plot or a dataframe including the conditional coefficient estimates of var1, their upper and lower bounds, and the corresponding values of var2.
#' @param steps Desired length of the sequence. A non-negative number, which for seq and seq.int will be rounded up if fractional. The default is 100 or the unique categories in the \code{var2} (when it is less than 100. Also see \code{\link{unique}}).
#' @param ci A numeric value defining the credible interval. The default value is 95\% (0.95). For \code{brmsfit} objects the interval summarizes the posterior draws, so it is an (equal-tailed) Bayesian credible interval rather than a frequentist confidence interval.
#' @param adjCI The false-discovery-rate adjustment of Esarey and Sumner (2017) is a frequentist correction and does not apply to Bayesian posteriors. The argument is ignored (with a warning) for \code{brmsfit} objects.
#' @param hist A logical value indicating if there is a histogram of `var2` added at the bottom of the conditional effect plot.
#' @param var2_dt A numerical value indicating the frequency distribution of `var2`. It is only used when `hist == TRUE`. When the object is a model, the default is the distribution of `var2` of the model.
#' @param predPro A logical value with default of `FALSE`. When the `m` is fitted with a Bernoulli or binomial family and the argument is set to `TRUE`, the function plots the posterior expected predicted probabilities at the values given by `var2_vals`, computed with \code{\link[brms]{posterior_epred}}.
#' @param var2_vals A numerical value indicating the values the predicted probabilities are estimated, when `predPro` is `TRUE`.
#' @param point A logical value determining the format of plot. By default, the function produces a line plot when var2 takes on ten or more distinct values and a point (dot-and-whisker) plot otherwise; option TRUE forces a point plot.
#' @param sims Ignored for \code{brmsfit} objects: the posterior draws produced by the sampler are used directly instead of re-simulating the coefficients.
#' @param xmin A numerical value indicating the minimum value shown of x shown in the graph. Rarely used.
#' @param xmax A numerical value indicating the maximum value shown of x shown in the graph. Rarely used.
#' @param ercolor A character value indicating the outline color of the whisker or ribbon.
#' @param esize A numerical value indicating the size of the whisker or ribbon.
#' @param ralpha A numerical value indicating the transparency of the ribbon.
#' @param rfill A character value indicating the filling color of the ribbon.
#' @param stats_cp A character value indicating what statistics to present as the plot note. Three options are available: "none", "ci", and "ks". The default is "none". See the Details for more information.
#' @param txt_caption A character string to add a note for the plot, a value will sending to \code{ggplot2::labs(caption = txt_caption))}.
#' @param facet_labs An optional character vector of facet labels to be used when plotting an interaction with a factor variable.
#' @param var3 An optional name (as a string) of a third variable for a three-way interaction \code{var1 * var2 * var3}. When supplied, the conditional effect of \code{var1} across \code{var2} is shown at several values/levels of \code{var3}. The default \code{NULL} gives the standard two-way behavior. Requires continuous \code{var1} and \code{var2}.
#' @param var3_vals An optional numeric vector giving the values of a continuous \code{var3} to condition on. The default is the mean and the mean plus or minus one standard deviation. Ignored when \code{var3} is a factor.
#' @param facet A logical value, used only with \code{var3}. \code{TRUE} (default) draws one panel per value/level of \code{var3}; \code{FALSE} overlays the curves colored by \code{var3}.
#' @param ... Other ggplot aesthetics arguments for points in the dot-whisker plot or lines in the line-ribbon plots. Not currently used.
#'
#' @details \code{interplot.brmsfit} is an S3 method of \code{interplot} for models fitted with \code{\link[brms]{brm}}. Unlike the frequentist methods, it does not call \code{arm::sim}: the posterior draws of the population-level (fixed) effects are extracted directly with \code{as.matrix} and the conditional coefficient \eqn{b_{var1} + b_{var1:var2} \cdot var2} is computed for every draw. Point estimates are posterior means and the bounds are posterior quantiles.
#'
#' Because the output function is based on \code{\link[ggplot2]{ggplot}}, any additional arguments and layers supported by \code{ggplot2} can be added with the \code{+}.
#'
#' The 'brms' package is only suggested by 'interplot'; it must be installed for this method to run.
#'
#' @return The function returns a \code{ggplot} object, or a list with the data frame of conditional coefficients when \code{plot = FALSE}.
#'
#' @examples
#' \dontrun{
#' library(brms)
#' data(mtcars)
#'
#' # A Bayesian linear model with a two-way interaction
#' m_brms <- brm(mpg ~ wt * cyl, data = mtcars, chains = 2, refresh = 0)
#'
#' # Identical interface; the band is a 95% posterior credible interval
#' interplot(m_brms, var1 = "cyl", var2 = "wt")
#'
#' # Posterior predicted probabilities for a Bernoulli model
#' m_brms_bin <- brm(am ~ wt * cyl, data = mtcars,
#'                   family = bernoulli(), chains = 2, refresh = 0)
#' interplot(m_brms_bin, var1 = "wt", var2 = "cyl",
#'           predPro = TRUE, var2_vals = c(4, 6, 8))
#' }
#'
#' @importFrom stats quantile median setNames
#' @importFrom purrr map map2 list_c list_rbind
#' @import ggplot2
#' @import dplyr
#'
#' @export


interplot.brmsfit <- function(m,
                              var1,
                              var2,
                              plot = TRUE,
                              steps = NULL,
                              ci = .95,
                              adjCI = FALSE,
                              hist = FALSE,
                              var2_dt = NA,
                              predPro = FALSE,
                              var2_vals = NULL,
                              point = FALSE,
                              sims = 1000,
                              xmin = NA,
                              xmax = NA,
                              ercolor = NA,
                              esize = 0.5,
                              ralpha = 0.5,
                              rfill = "grey70",
                              stats_cp = "none",
                              txt_caption = NULL,
                              facet_labs = NULL,
                              var3 = NULL,
                              var3_vals = NULL,
                              facet = TRUE,
                              ...) {

  if (!requireNamespace("brms", quietly = TRUE))
    stop("Plotting `brmsfit` objects requires the 'brms' package. Install it with install.packages('brms').")

  if (isTRUE(adjCI))
    warning("`adjCI` is a frequentist false-discovery-rate correction and is ignored for Bayesian (brmsfit) models.")

  # Posterior draws of the population-level coefficients (already sampled; no re-simulation).
  draws <- as.matrix(m)
  mdata <- m$data

  # Coefficient data frame ####
  ## Factorial base terms
  factor_v1 <- is.factor(mdata[[var1]])
  factor_v2 <- is.factor(mdata[[var2]])

  if (factor_v1 & factor_v2)
    stop("The function does not support interactions between two factors.")

  if ((factor_v1 | factor_v2) & predPro == TRUE)
    stop("The current version does not support estimating predicted probabilities for factor base terms.")

  is_3way <- !is.null(var3)

  # Nonlinear-in-moderator (e.g. var1 * I(var2^2), poly/spline of var2). Detected
  # from the brms fixed-effects terms; only for numeric base terms, no predPro.
  brms_fe_terms <- stats::delete.response(stats::terms(brms::brmsterms(m$formula)$dpars$mu$fe))
  is_nonlinear <- !is_3way && !factor_v1 && !factor_v2 && predPro == FALSE &&
    var1 != var2 && detect_nonlinear(brms_fe_terms, var1, var2)

  if (is_3way) {
    if (predPro == TRUE)
      stop("Predicted probabilities are not supported for three-way interactions.")
    if (var1 == var2 || var1 == var3 || var2 == var3)
      stop("var1, var2, and var3 must be three distinct variables.")

    # Family-agnostic three-way core expects plain coefficient names (no brms
    # `b_` prefix): keep the population-level columns and strip the prefix.
    bcols <- startsWith(colnames(draws), "b_")
    draws3 <- draws[, bcols, drop = FALSE]
    colnames(draws3) <- sub("^b_", "", colnames(draws3))

    ls_results <- extract_coef_3way(
      draws = draws3,
      frame = mdata,
      var1 = var1,
      var2 = var2,
      var3 = var3,
      var3_vals = var3_vals,
      ci = ci,
      adjCI = adjCI,
      df_resid = NULL,
      steps = steps,
      xmin = xmin,
      xmax = xmax
    )
  } else if (is_nonlinear) {
    ls_results <- extract_coef_nonlinearB(
      m = m,
      draws = draws,
      tt = brms_fe_terms,
      var1 = var1,
      var2 = var2,
      ci = ci,
      steps = steps,
      xmin = xmin,
      xmax = xmax
    )
  } else if (factor_v1 | factor_v2) {
    ls_results <- extract_coef_facB(
      factor_v1 = factor_v1,
      factor_v2 = factor_v2,
      m = m,
      draws = draws,
      var1 = var1,
      var2 = var2,
      ci = ci,
      steps = steps,
      xmin = xmin,
      xmax = xmax
    )
  } else {
    ls_results <- extract_coef_numB(
      m = m,
      draws = draws,
      var1 = var1,
      var2 = var2,
      ci = ci,
      steps = steps,
      predPro = predPro,
      var2_vals = var2_vals,
      xmin = xmin,
      xmax = xmax
    )
  }

  coef_df <- ls_results[[1]]
  ci_diff <- ls_results[[2]]
  steps   <- ls_results[[3]]
  ks_diff <- ls_results[[4]]

  # Plotting ####
  if (hist == TRUE & all(is.na(var2_dt)))
    var2_dt <- mdata[[var2]]

  # Replace the labels (factor base term). The CI(Max - Min) note goes to the
  # caption (always visible) rather than the facet strip.
  if (is_3way) {
    # The "value" column already carries ordered var3 labels. For stats_cp = "ci"
    # the per-level CI(Max - Min) goes to the caption (survives faceting/overlay).
    if (stats_cp == "ci") {
      ci_note <- map2(ci_diff, levels(coef_df$value), \(aCI, aLevel) {
        paste0(aLevel, " CI(Max - Min): [",
               round(aCI[1], digits = 3), ", ",
               round(aCI[2], digits = 3), "]")
      }) |>
        list_c() |>
        paste(collapse = "\n")
      txt_caption <- paste0(ci_note, txt_caption)
    }
  } else if (factor_v1 | factor_v2) {
    if (is.null(facet_labs)) facet_labs <- unique(coef_df$value)

    if (stats_cp == "ci") {
      ci_note <- map2(ci_diff, facet_labs, \(aCI, aLevel) {
        paste0(if (length(facet_labs) > 1) paste0(aLevel, " ") else "",
               "CI(Max - Min): [",
               round(aCI[1], digits = 3), ", ",
               round(aCI[2], digits = 3), "]")
      }) |>
        list_c() |>
        paste(collapse = "\n")
      txt_caption <- paste0(ci_note, txt_caption)
    }

    coef_df$value <- factor(coef_df$value, labels = facet_labs)
  }

  # Plotting the general plot
  if (plot == FALSE) {
    names(coef_df)[1:4] <- c(var1, "coef", "ub", "lb") # rename the first four cols; factorial results keep a fifth column "value"

    return(list(df_coef = coef_df, stats_ci = ci_diff))
  } else {
    # For a three-way overlay (facet = FALSE) color the curves by var3.
    overlay <- is_3way && !facet

    aPlot <- interplot.plot(
      m = coef_df,
      var1 = var1,
      var2 = var2,
      hist = hist,
      steps = steps,
      var2_dt = var2_dt,
      predPro = predPro,
      point = point,
      ercolor = ercolor,
      esize = esize,
      ralpha = ralpha,
      rfill = rfill,
      stats_cp = stats_cp,
      txt_caption = txt_caption,
      ci_diff = ci_diff,
      ks_diff = ks_diff,
      overlay = overlay,
      ...
    )

    # Facet for factors or three-way small multiples
    if (factor_v1 | factor_v2) {
      aPlot <- aPlot + facet_grid(. ~ value)
    } else if (is_3way && facet) {
      aPlot <- aPlot + facet_wrap(~ value)
    } else if (is_3way && !facet) {
      aPlot <- aPlot + labs(colour = var3, fill = var3)
    }

    return(aPlot)
  }
}


# Resolve a draw-matrix column name for a two-way interaction, trying both
# orderings of the terms (brms preserves the formula order, but be defensive).
resolve_brms_int <- function(draw_names, a, b) {
  cand <- paste0("b_", c(paste0(a, ":", b), paste0(b, ":", a)))
  hit <- cand[cand %in% draw_names]
  if (length(hit)) hit[1] else NA_character_
}


extract_coef_numB <- function(
    m,
    draws,
    var1,
    var2,
    ci,
    steps,
    predPro,
    var2_vals,
    xmin,
    xmax
  ){

  mdata <- m$data
  draw_names <- colnames(draws)

  # Predicted-probability path (Bayesian posterior_epred) ####
  if (predPro == TRUE) {
    if (is.null(var2_vals))
      stop("The predicted probabilities cannot be estimated without defining 'var2_vals'.")

    n_steps <- if (is.null(steps)) 100 else steps

    v1_rng <- range(mdata[[var1]], na.rm = TRUE)
    v1_seq <- seq(v1_rng[1], v1_rng[2], length.out = n_steps)
    grid <- expand.grid(.v1 = v1_seq, .v2 = var2_vals)

    # Representative profile: median for numeric, modal level for factors.
    rep_row <- map(mdata, \(x) {
      if (is.numeric(x)) median(x, na.rm = TRUE)
      else factor(names(which.max(table(x))), levels = levels(factor(x)))
    })
    fake_data <- as.data.frame(rep_row, stringsAsFactors = FALSE)[rep(1, nrow(grid)), , drop = FALSE]
    fake_data[[var1]] <- grid$.v1
    fake_data[[var2]] <- grid$.v2

    pp <- brms::posterior_epred(m, newdata = fake_data, re_formula = NA)
    probs <- c((1 - ci) / 2, 1 - (1 - ci) / 2)

    # Keep "value" as the last column so the positional plot = FALSE rename
    # (first four columns -> var1, coef, ub, lb) leaves it intact.
    coef <- data.frame(
      fake  = grid$.v1,
      coef1 = colMeans(pp) * 100,
      ub    = apply(pp, 2, quantile, probs = probs[2]) * 100,
      lb    = apply(pp, 2, quantile, probs = probs[1]) * 100,
      value = as.factor(grid$.v2)
    )

    return(list(coef, numeric(0), n_steps, NULL))
  }

  # Conditional-coefficient path ####
  bvar1 <- paste0("b_", var1)

  # Detect if it is a quadratic model
  if (var1 == var2) {
    bvar12 <- draw_names[grepl(paste0("^b_I.*", var1, ".*2"), draw_names)][1]
    multiplier <- 2
  } else {
    bvar12 <- resolve_brms_int(draw_names, var1, var2)
    multiplier <- 1
  }

  if (!bvar1 %in% draw_names)
    stop(paste0("Model does not include a population-level coefficient for '", var1, "'."))
  if (is.na(bvar12))
    stop(paste0("Model does not include the interaction of ", var1, " and ", var2,
                ". Available population-level terms: ",
                paste(draw_names[startsWith(draw_names, "b_")], collapse = ", "), "."))

  # Set the min, max, and steps ####
  if (is.na(xmin)) xmin <- min(mdata[[var2]], na.rm = TRUE)
  if (is.na(xmax)) xmax <- max(mdata[[var2]], na.rm = TRUE)
  if (is.null(steps)) steps <- length(unique(na.omit(mdata[[var2]])))
  if (steps > 100) steps <- 100
  fake <- seq(xmin, xmax, length.out = steps)

  # Calculate the effects ####
  sim_v1  <- draws[, bvar1]
  sim_v12 <- draws[, bvar12]

  coef <- sim_coef_vec(sim_v1, sim_v12, fake, ci, multiplier)
  diff_stats <- sim_diff_stats(sim_v1, sim_v12, xmin, xmax, ci, multiplier)

  list(coef, diff_stats$ci_diff, steps, diff_stats$ks_diff)
}


extract_coef_facB <- function(
    factor_v1,
    factor_v2,
    m,
    draws,
    var1,
    var2,
    ci,
    steps,
    xmin,
    xmax
  ){

  mdata <- m$data
  draw_names <- colnames(draws)

  if (factor_v1) {
    # var1 is the factor being plotted; var2 (numeric) is the x-axis.
    lvls <- levels(mdata[[var1]])[-1] # non-reference levels

    if (is.na(xmin)) xmin <- min(mdata[[var2]], na.rm = TRUE)
    if (is.na(xmax)) xmax <- max(mdata[[var2]], na.rm = TRUE)
    if (is.null(steps)) steps <- length(unique(na.omit(mdata[[var2]])))
    if (steps > 100) steps <- 100
    fake_seq <- seq(xmin, xmax, length.out = steps)

    results <- map(lvls, \(L) {
      base  <- paste0("b_", var1, L)
      inter <- resolve_brms_int(draw_names, var2, paste0(var1, L))
      if (!base %in% draw_names || is.na(inter))
        stop(paste("Model does not include the interaction of", var1, "and", var2, "."))

      coef <- sim_coef_vec(draws[, base], draws[, inter], fake_seq, ci)
      d <- sim_diff_stats(draws[, base], draws[, inter], xmin, xmax, ci)
      coef$value <- paste0(var1, L)
      list(coef = coef, ci_diff = d$ci_diff)
    })

  } else {
    # var2 is the factor (x-axis levels 0/1); var1 (numeric) is plotted.
    lvls <- levels(mdata[[var2]])[-1]
    xmin <- 0
    xmax <- 1
    steps <- 2
    fake_seq <- seq(xmin, xmax, length.out = steps)

    base <- paste0("b_", var1)
    if (!base %in% draw_names)
      stop(paste0("Model does not include a population-level coefficient for '", var1, "'."))

    results <- map(lvls, \(L) {
      inter <- resolve_brms_int(draw_names, paste0(var2, L), var1)
      if (is.na(inter))
        stop(paste("Model does not include the interaction of", var1, "and", var2, "."))

      coef <- sim_coef_vec(draws[, base], draws[, inter], fake_seq, ci)
      d <- sim_diff_stats(draws[, base], draws[, inter], xmin, xmax, ci)
      coef$value <- paste0(var2, L)
      list(coef = coef, ci_diff = d$ci_diff)
    })
  }

  coef_df <- map(results, "coef") |> list_rbind()
  ci_diff <- map(results, "ci_diff")

  list(coef_df, ci_diff, steps, NULL)
}


# Nonlinear-in-moderator effect for brmsfit (e.g. var1 * I(var2^2), poly/spline
# of var2). brms mangles transformed terms in its `b_` coefficient names (e.g.
# `b_wt:IhpE2` for `wt:I(hp^2)`), which is awkward to parse. Instead we rebuild
# the marginal-effect gradient from the *clean* fixed-effects terms via
# `model.matrix` (which regenerates spline/polynomial bases through predvars),
# then map the clean design columns onto the brms-mangled draw columns by
# position: `model.matrix(tt, data)` and the population-level `b_*` draws share
# the same term order, so the i-th design column corresponds to the i-th `b_`
# coefficient. The remaining work reuses the family-agnostic gradient helpers.
extract_coef_nonlinearB <- function(
    m,
    draws,
    tt,
    var1,
    var2,
    ci,
    steps,
    xmin,
    xmax
  ){

  mdata <- m$data

  # Population-level draws with the `b_` prefix stripped.
  bcols <- startsWith(colnames(draws), "b_")
  b <- draws[, bcols, drop = FALSE]
  colnames(b) <- sub("^b_", "", colnames(b))

  v2_vals <- mdata[[var2]]
  if (is.na(xmin)) xmin <- min(v2_vals, na.rm = TRUE)
  if (is.na(xmax)) xmax <- max(v2_vals, na.rm = TRUE)
  if (is.null(steps)) steps <- length(unique(na.omit(v2_vals)))
  if (steps > 100) steps <- 100
  fake <- seq(xmin, xmax, length.out = steps)

  # Clean -> brms positional name map. The full design and the `b_` draws list
  # the same terms in the same order (intercept first), so align by position.
  X <- stats::model.matrix(tt, mdata)
  clean <- colnames(X)
  clean[clean == "(Intercept)"] <- "Intercept"
  if (length(clean) != ncol(b))
    stop("Cannot align the design matrix with the brms coefficients for the nonlinear effect of '",
         var1, "'. Use brms-native marginal effects (e.g. marginaleffects) instead.")
  name_map <- setNames(colnames(b), clean)

  # Gradient basis for the var1 marginal effect from the clean terms.
  des <- build_me_design(tt, mdata, var1, var2, fake)
  brms_terms <- unname(name_map[des$term_names])
  if (anyNA(brms_terms) || !all(brms_terms %in% colnames(b)))
    stop("Could not match the nonlinear marginal-effect terms of '", var1,
         "' to the brms coefficients. Use brms-native marginal effects instead.")

  sim_mat <- b[, brms_terms, drop = FALSE]

  coef <- sim_coef_grad(sim_mat, des$G, fake, ci)
  diff_stats <- sim_diff_stats_grad(sim_mat, des$G, ci)

  list(coef, diff_stats$ci_diff, steps, diff_stats$ks_diff)
}

---
title: "interplot: Plot the Effects of Variables in Interaction Terms"
author: "Frederick Solt and Yue Hu"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
bibliography: vignette.bib
vignette: >
  %\VignetteIndexEntry{interplot: Plot the Effects of Variables in Interaction Terms}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  message = FALSE,
  warning = FALSE,
  fig.width = 6,
  fig.height = 4,
  out.width = "100%",
  dpi = 120
)
```

Interaction is a powerful tool to test conditional effects of one variable on the contribution of another variable to the dependent variable and has been extensively applied in the empirical research of social science since the 1970s [@WrightJr1976].
Unfortunately, the nonlinear nature determines that the statistical estimate of an interactive effect cannot be interpreted as straightforward as the coefficient of a regular regression parameter.
Let's use a simple example to illustrate this point:
The following model use an interaction term to test the conditional effect of Z on X's contribution (or the conditional effect of X on Z's contribution) to the variance of Y. 

$$Y = \beta_0 + \beta_1X + \beta_2Z + \beta_3X\times Z + \varepsilon.$$
The contribution of X on Y is typically represented by the marginal effect:

$$\frac{\partial Y}{\partial X} = \beta_1 + \beta_3Z.$$

The standard error of this estimate is thus: 

$$\hat{\sigma}_{\frac{\partial Y}{\partial X}} = \sqrt{var(\hat{\beta_1}) + Z^2var(\hat{\beta_3}) + 2Zcov(\hat{\beta_1}, \hat{\beta_3})}.$$

As @Brambor2006 [p.70] indicates, the above equation suggests that its perfectly possible for the contribution of X on Y to be statistically significant for certain values of Z "even if all of the model parameters are insignificant."
In other words, one cannot infer whether X has a meaningful conditional effect on Y simply from the magnitude and significance of either $\beta_1$ or $\beta_3$ (Ibid., 74). 
Instead, the conditional effect should be examined based on the marginal effect at every observed value of Z [@Berry2012; @Brambor2006; @Braumoeller2004].

Furthermore, scholars have noticed that the point estimations of some interaction models, especially those depending on link functions (e.g., logit and probit) may not be as reliable as the estimates of the confidence intervals [@Berry2016].
Therefore, it is even more important to accurately present and examine the boundaries established by the confidence intervals than the point estimates in the test of conditional effect.
@Berry2012 also pointed out that the substantive significance of the conditional effect highly relates to the distribution of the conditioning variable (viz., Z in the above example). 
They thus recommend researchers to present the frequency distribution of the conditioning variable together with the marginal effects (especially when the effect trend goes across the zero point).

More recently, @Esarey2017 uncovered that the estimation of the marginal effects suggested by @Brambor2006 might cause a "multiple comparison problem" and result over- or underconfidence of the confidential intervals.
They thus recommended to adjust the CIs with a critical t-statistics following the Benjamini1995 procedure.

The `interplot` package provides a convenient way to operate and visualize above points with one or a series of plots produced by a single function. 
The function visualizes the changes in the coefficient of one variable (the posterior probabilities) in a two-way interaction term conditional on the value of the other included variable. 
The plot also includes simulated 95% confidential intervals of these coefficients.In the current version, the function works with ordinary linear regression models, generalized linear models (e.g., logit, probit, ordered logit, etc.), and multilevel (mixed-effects) regressions, all with complete or multiply imputed data (in a list format).

Comparing to established alternatives such as `effects::plot` and `sjplot::sjp.int`, `interplot` provides a more user-friendly way to quickly produce plots that are easy to interpret. 
Moreover, it works on multilevel models and outcomes from multiple imputations.
`interplot` plots the changes in the *conditional coefficient* of one variable in the interaction, rather than changes in the dependent variable itself as in the aforementioned functions. 
This approach avoids displaying interaction effects across multiple panels or multiple lines in favor of a single plot containing all the relevant information. Moreover, by outputting `ggplot` objects, `interplot` allows users to easily further customize their graphs.

This vignette purposes to illustrate how users can apply these functions to improve the presentation of the interactions in their models. 

# Installation

To install:

* the latest released version: `install.packages("interplot")`.
* the latest developing version: `devtools::install_github("sammo3182/interplot")`.

# Basic Use
## Run a Model

This example is based on the `mtcars` dataset, which is drawn from the 1974 volume of the US magazine *Motor Trend*. The dependent variable is mileage in miles per (US) gallon (`mpg`), and the independent variables are the number of engine cylinders (`cyl`) and automobile weight in thousands of pounds (`wt`).

```{r}
data(mtcars)  #load the data
```

Suppose we are interested in how automobile weight affects the relationship between of the number of engine cylinders on mileage and how the number of cylinders affects the relationship between the car's weight and its mileage. Such conditional effects are modeled using a two-way multiplicative interaction term:

```{r}
m_cyl <- lm(mpg ~ wt * cyl, data = mtcars)
summary(m_cyl)
```

The coefficient of the interactive term (`wt:cyl`) is positive and statistically significant at .05 level.  This tells us that the coefficient of `wt` depends on the value of `cyl` and vice versa; these estimated coefficients are conditional.  It does not indicate anything, however, about the magnitude or statistical significance of these conditional coefficients.  A plot produced by `interplot` easily and clearly answers these latter questions.


## Create a Basic Graph of the Conditional Coefficients

To plot conditional coefficients, a user needs to provide only three basic pieces of information: the object of a regression result (`m`), the variable whose coefficient is to be plotted (`var1`), and the variable on which the coefficient is conditional (`var2`). Taking the previous example, if we intend to know how the weight of a car can affect the coefficient for number of cylinders on the mileage, `var1` is `cyl`, and `var2` is `wt`. 

```{r out.width="80%"}
library(interplot)

interplot(m = m_cyl, var1 = "cyl", var2 = "wt")
```

The plot clearly shows that with increasing automobile weight (along the x axis), the magnitude of the coefficient of the number of cylinders on the mileage also increases (along the y axis).

Similarly, to show how the number of cylinders affects the coefficient of automobile weight on mileage, one only needs to switch `var1` and `var2`:

```{r out.width="80%"}
interplot(m = m_cyl, var1 = "wt", var2 = "cyl")
```

Users can adjust the CI level by setting the `ci` option.
The default value is 95% CIs.
The following example resets the the CIs to 90%.

```{r out.width="80%"}
interplot(m = m_cyl, var1 = "wt", var2 = "cyl", ci = .9, point = T)
```

The format of the plot also changes when `wt` is `var1` and `cyl` is `var2`. This is because `cyl` is not a continuous variable but a categorical one with just three values: 4, 6, and 8. `interplot` automatically detects the number of values taken on by `var2` and chooses the appropriate plot format.  If there are fewer than 10 values, the function will produce a "dot-and-whisker" plot; otherwise, by default, it will generate a "line-and-ribbon" plot.  Users may override this default by setting the argument `point` to `TRUE`.

```{r out.width="80%"}
interplot(m = m_cyl, var1 = "cyl", var2 = "wt", point = T) +
  # changing the angle of x labels for a clearer vision
  theme(axis.text.x  = element_text(angle=90))
```

## Change the Appearance of the Plot

Plots generated by `interplot` are, by design, very basic. They are, however, `ggplot` objects and so may be easily modified further.

```{r out.width="80%"}
interplot(m = m_cyl, var1 = "cyl", var2 = "wt") + 
  # Add labels for X and Y axes
    xlab("Automobile Weight (thousands lbs)") +
    ylab("Estimated Coefficient for\nNumber of Cylinders") +
  # Change the background
    theme_bw() +
  # Add the title
    ggtitle("Estimated Coefficient of Engine Cylinders \non Mileage by Automobile Weight") +
    theme(plot.title = element_text(face="bold")) +
  # Add a horizontal line at y = 0
    geom_hline(yintercept = 0, linetype = "dashed")
```

For the default settings of the whisker or ribbon, the users can also use arguments, such as `ercolor` and `esize` to modify. More arguments can be found in the `?interplot` file.

```{r out.width="80%"}
interplot(m = m_cyl, var1 = "wt", var2 = "cyl", ercolor = "blue", esize = 1.5) +
  geom_point(size = 2, color = "red")
```

## Plot the Effect of a Variable with a Quadratic Term

The simplest type of interaction is quadratic term, which can be regarded as a variable interact with itself. `interplot` can visualize this case when the variable names of `var1` and `var2` are the same.

```{r out.width="80%"}
m_wt <- lm(mpg ~ wt + I(wt^2), data = mtcars)

interplot(m = m_wt, var1 = "wt", var2 = "wt")
```


## Plot Interactions with a Factor Term

When either the conditioned or the conditioning base term of an interaction is a factor, `interplot` creates a facet in which the conditional effect under each category of the factor is visualized in a separate panel.

```{r out.width="80%"}
mtcars$gear <- factor(mtcars$gear)
m_gear <- lm(mpg ~ gear * wt, data = mtcars)

interplot(m = m_gear, var1 = "wt", var2 = "gear")
```

To edit the facet labels, use the `facet_labs` argument to specify a character vector of the desired labels:

```{r out.width="80%"}
interplot(m = m_gear, var1 = "wt", var2 = "gear", facet_labs = c("4-speed", "5-speed"))
```


## Include the Distribution of the Conditioning Variable

@Berry2012 points out that, when a variable's conditional effect reaches statistical significance over only part of the range of the conditioning variable, it can be helpful to the evaluation of the substantive significance of the conditional effect to know the distribution of the conditioning variable. For this purpose, `interplot` has the `hist` argument for users to choose to superimpose a histogram at the bottom of the conditional effect plot.

```{r out.width="80%"}
interplot(m = m_cyl, var1 = "cyl", var2 = "wt", hist = TRUE) +
    geom_hline(yintercept = 0, linetype = "dashed")
```

Our implementation of this option was inspired by the excellent work of @Hainmueller2016.
A tip is that when presenting the histogram, some default setting would not be directly modified by the build-in arguments or the `geom` functions. 
Instead, one can change these settings by the `aes` function---as illustrated by the following example.

```{r out.width="80%"}
interplot(m = m_cyl, var1 = "cyl", var2 = "wt", hist = TRUE) +
  aes(color = "pink") + theme(legend.position="none") +  # geom_line(color = "pink") + 
  geom_hline(yintercept = 0, linetype = "dashed")
```



# Advanced Use and Customization
`interplot` visualizes the conditional effect based on (posterior) simulated marginal effects.
The simulation provides a probabilistic distribution of moderation effect of the conditioning variable (`var2`) at every preset values (including the minimum and maximum values) of the conditioned variable (`var1`), denoted as E~min~ and E~max~.
This output allows the function to further examine the conditional effect
statistically. 
`interplot` can examine if the distribution of (E~max~ - E~min~) covers zero.^[
Be aware that this method assumes that either the linear model or the one with an interaction is true. 
It is uncertain how the test would perform if neither is the true model.
An alternative way to test the significance of the conditional effect is to compare the models with and without interactions as the same through model comparison tests, such as Wald, likelihood ratio, or Hausman tests.
They may give a significant result when neither models is a true model.
These tests are also developed in a frequentist sense rather than based on the posterior simulated results.
]
Users can choose either method by setting the argument `stats_cp` to "ci".
The default is "none."


```{R}
set.seed(313)

interplot(m = m_cyl, var1 = "cyl", var2 = "wt", stats_cp = "ci")
```

Note that since the statistical results are presented as a `ggplot` caption, one cannot using `+ lab(caption = ...)` to specify it as for a regular `ggplot` object. 
Instead, users can modify the plot's caption by specify the argument `txt_caption` of the `dwplot` function.

```{R}
set.seed(313)

interplot(m = m_cyl, var1 = "cyl", var2 = "wt", stats_cp = "ci", 
          txt_caption = "\n Source: Motor Trend Car Road Tests 1973.")
```


## Adjust the Overconfidence in Confidence Interval Estimations

@Berry2016 emphasized the importance to estimate the uncertainty in studying conditional effects.
The most common way to do that is to follow the straightforward method of @Brambor2006.
Nevertheless, @Esarey2017 pointed out that this method might cause a "multiple comparison problem" and result over- or underconfidence of the confidential intervals.
For the overconfidence cases, they recommended to adjust the CIs with a critical t-statistics following the Benjamini1995 procedure.

`interplot` incorporates this recommendation with an argument `adjCI`.
If it is set `TRUE`, the function will calculate the critical t-statistics to limit the false discovery rate and adjust the estimation of confidence intervals. 
In the following example, the left panel presents the conditional effect plot with CIs estimated based on the @Brambor2006 method, and the right panel presents results with adjusted CIs.
Although the adjustment does not change the substantial conclusion, we can see the adjusted CIs cover more area both in the middle and at the extreme values of x. 

```{r fig.width=8, fig.height=3.5}
stdCI_plot <- interplot(m = m_wt, var1 = "wt", var2 = "wt", adjCI = FALSE) +
ggtitle("Marginal Effects with Standard CIs")
adjCI_plot <- interplot(m = m_wt, var1 = "wt", var2 = "wt", adjCI = TRUE) +
ggtitle("Marginal Effects with Adjusted CIs")

library(gridExtra)
grid.arrange(stdCI_plot, adjCI_plot, ncol = 2)
```

## Plot Conditional Predicted Probability

@Hanmer2013 point out (also in the associated [post](https://ajps.org/2014/11/13/translating-statistical-results-into-meaningful-quantities-methods/) in the AJPS website) that it is important to translate statistical results into meaningful quantities. 
The task not only pushes researchers to interpret the results in a real-life manner---which may lead to substantively different conclusions---but also provides convenience for a far broader scope of readers (e.g., policy makers, governmental officials, stakeholders, etc.) to understand the implications of the study and their importance.

To accomplish this task requires researchers to go beyond estimating the effect for the "average case," but focuses more on the values or intervals that are illustratively important and meaningful.
This could be a time-consuming job, though, especially when researchers work on models with limited dependent variables.
`interplot` provides a convenient way to achieve the task.
When researchers are dealing with general linear flat or multilevel models, they have the choice to set the argument `predPro` to `TRUE` and give the critical values they are interested in the argument `var2_vals`.
Then, `interplot` will automatically estimated the conditional effects of predicted probabilities at these given values of the conditioned variable.
The following example illustrates how it works.

In this example, we are interested how the economic inequality affect the impact of income on the U.S. citizens' belief in meritocracy, a critical ideology of the "American Dream."
We estimated this conditional effect based on an interaction model with three years of Pew surveys (2006, 2007, and 2009), in which the income is the conditioned variable and economic inequality (county-level Gini coefficients).^[For the purpose of illustration, we omitted the county level variance in this example. One can see more comprehensive analyses in @Solt2016.]
We first estimate the average conditional effect for the entire sample and then estimate the conditional predicted probabilities for citizens with the lowest and highest levels of income separately.
As shown in the following plot, the average case in the left panel only shows a decreasing conditional effect, while the predicted probability plot in the right uncovers conditional effects in opposite directions for the high and low income individuals.

```{r out.width="80%"}
pew1.w <- read.csv("pew1_w.csv")

m <- glm(formula=meritocracy~ginicnty+income_i+ginicnty:income_i+income_cnty+black_cnty+
                perc_bush04+pop_cnty+educ_i+age_i+gender_i+unemp_i+union_i+partyid_i+
                ideo_i+attend_i+survid2006+survid2007+survid2009,
              data=pew1.w,family=binomial(link="logit"))
              
plot_avg <- interplot(m, var1 = "ginicnty",var2 = "income_i", predPro = FALSE) + 
ggtitle("Average Conditional Effects")

plot_3val <- interplot(m, var1 = "ginicnty",var2 = "income_i", predPro = TRUE, var2_vals = c(min(pew1.w$income_i), max(pew1.w$income_i))) +
ggtitle("Conditional Predicted Prob for \nLow and High Income Citizens") +
scale_colour_discrete(guide = guide_legend(title = "Income"), labels = c("Low", "High")) + 
scale_fill_discrete(guide = guide_legend(title = "Income"), labels = c("Low", "High")) +
theme(legend.position = c(0, .8), legend.justification = c(0, .5))

grid.arrange(plot_avg, plot_3val, ncol = 2)
```

## Plot Three-Way Interactions

When a model contains a three-way interaction `var1 * var2 * var3`, the conditional effect of `var1` depends on *both* `var2` and `var3`. Supplying `var3` to `interplot` plots the effect of `var1` across `var2` at several values (or levels) of `var3`. Fixing `var3` at a value $c$ turns the three-way effect into the familiar two-way problem with a shifted intercept and slope, $(b_{1} + b_{13} c) + (b_{12} + b_{123} c)\, var2$, so the same simulation engine and statistics are reused.

For a continuous `var3`, the default conditioning values are the mean and the mean plus or minus one standard deviation (Aiken and West 1991); supply `var3_vals` to override them. A factor `var3` uses all of its levels. Three-way interactions are supported for ordinary and generalized linear models, mixed-effects models (`lmer`/`glmer`), multiply-imputed model lists, and Bayesian (`brms`) models.

```{r fig.width=7.5, fig.height=3}
m_3way <- lm(mpg ~ wt * hp * cyl, data = mtcars)

# One small-multiple panel per value of cyl (the default)
interplot(m_3way, var1 = "wt", var2 = "hp", var3 = "cyl")
```

Set `facet = FALSE` to overlay the curves in a single panel, colored by `var3`:

```{r message=FALSE}
interplot(m_3way, var1 = "wt", var2 = "hp", var3 = "cyl", facet = FALSE)
```

## Plot Nonlinear Conditional Effects

The standard multiplicative interaction model assumes that the marginal effect of `var1` is a *linear* function of the moderator `var2`. When that assumption is too restrictive, one can let `var1` interact with a transformation of `var2`---a polynomial term such as `I(var2^2)`, or a spline such as `splines::ns(var2)`. `interplot` detects these terms automatically and plots the conditional effect as the implied nonlinear function of `var2`, computed from the model's own design-matrix gradient so that the confidence band, `stats_cp`, and `adjCI` all continue to work. As with the two-way case, this is available for linear, generalized linear, mixed-effects, multiply-imputed, and Bayesian (`brms`) models.

```{r message=FALSE}
# Effect of wt allowed to vary quadratically with cyl
m_nl <- lm(mpg ~ wt * (cyl + I(cyl^2)), data = mtcars)
interplot(m_nl, var1 = "wt", var2 = "cyl")
```

A spline interaction is handled the same way; `interplot` rebuilds the spline basis at its original knots, so the conditional-effect curve can take any smooth shape the moderator implies:

```{r message=FALSE}
library(splines)
m_spline <- lm(mpg ~ wt * ns(hp, 3), data = mtcars)
interplot(m_spline, var1 = "wt", var2 = "hp")
```

To check whether the simpler linear-interaction assumption is adequate, `bin_layer()` overlays the binning estimator of Hainmueller, Mummolo, and Xu (2019): the conditional effect of `var1` estimated separately within terciles of `var2`, drawn as dot-and-whiskers at the bin medians. If those binned estimates depart from the `interplot` line, a nonlinear specification is warranted.

```{r message=FALSE}
m_lin <- lm(mpg ~ wt * cyl, data = mtcars)
interplot(m_lin, var1 = "wt", var2 = "cyl") +
  bin_layer(m_lin, var1 = "wt", var2 = "cyl")
```

## Identify Significance Regions with Johnson-Neyman Intervals

A conditional effect is often statistically significant over only part of the moderator's range. The Johnson-Neyman (JN) technique locates the exact values of `var2` at which the effect of `var1` crosses the boundary of significance. `jn_interval()` computes those bounds analytically from the model's coefficients and variance-covariance matrix (it works with `lm`, `glm`, `lmer`, and `glmer` objects):

```{r}
m_cyl <- lm(mpg ~ wt * cyl, data = mtcars)
jn <- jn_interval(m_cyl, var1 = "cyl", var2 = "wt")
print(jn)
```

`jn_layer()` turns the result into composable `ggplot2` layers---vertical lines at the JN bounds and shading over the non-significant region---so it can be added to any `interplot` output with `+`:

```{r message=FALSE}
interplot(m_cyl, var1 = "cyl", var2 = "wt") +
  jn_layer(jn)
```

## Plot Conditional Effects Without a Model

In some cases, one may analyze some complicated or self-made regression functions which are not supported by the current version of `interplot`. For such models, as long as the user has a dataset loading the simulated results of the interaction effects, she can still use `interplot` to visualize it. The dataset needs four columns the scale of the conditioning variable (`fake`), the simulated interactive effect at each break of the conditioning variable (`coef1`), and the simulated lower bound and upper bound of the confidence interval (`lb`, and `ub`). The column names should be exactly the ones shown in the above parentheses. Here is an example with some arbitrary artificial data:

```{r out.width="80%"}
# Create a fake dataset of conditional effects
fake <- rnorm(100, 0, 1)
coef1 <- fake * sample(.5:2.5, 100, replace = T)
lb <- coef1 - .5
ub <- coef1 + .5

df_fake <- data.frame(cbind(fake, coef1, lb, ub))

# Use interplot directly with the dataset
interplot(df_fake)
```

If one also has the data of the `var2`, she can also draw a histogram under it by the argument `var_dt`. 

```{r out.width="80%"}
var2_fake <- fake
# Set `hist` to TRUE is required to superimpose a histogram.
interplot(df_fake, hist = TRUE, var2_dt = var2_fake)
## The ribbon and histogram do not fit. This is just an illustration
```


## Conclusion

The `interplot` package provides a flexible and convenient way to visualize conditional coefficients of variables in multiplicative interaction terms. This vignette offers an overview of its use and features. We encourage users to consult the help files for more details. 

The development of the package is ongoing, and future research promises a compatible tool for more types of regressions and more functions. Please contact us with any questions, bug reports, and comments.

## Affiliation

Frederick Solt

Department of Political Science,

University of Iowa, 

324 Schaeffer Hall,

20 E Washington St, Iowa City, IA, 52242

Email: <frederick-solt@uiowa.edu>

Website: <https://fsolt.org>

<br></br>

Yue Hu

Department of Political Science,

Tsinghua University, 

Mingzhai 114,

Qinghua Yuan, Haidian District, Beijing 100084, China

Email: <yuehu@tsinghua.edu.cn>

Website: <https://www.drhuyue.site/>

## Reference

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Interplot_plot.R
\name{interplot.plot}
\alias{interplot.plot}
\title{Plot Conditional Coefficients in Models with Interaction Terms}
\usage{
\method{interplot}{plot}(
  m,
  var1 = NULL,
  var2 = NULL,
  plot = TRUE,
  steps = NULL,
  ci = 0.95,
  adjCI = FALSE,
  hist = FALSE,
  var2_dt = NULL,
  predPro = FALSE,
  var2_vals = NULL,
  point = FALSE,
  sims = 5000,
  xmin = NA,
  xmax = NA,
  ercolor = NA,
  esize = 0.5,
  ralpha = 0.5,
  rfill = "grey70",
  stats_cp = "none",
  txt_caption = NULL,
  ci_diff = NULL,
  ks_diff = NULL,
  overlay = FALSE,
  ...
)
}
\arguments{
\item{m}{A model object including an interaction term, or, alternately, a data frame recording conditional coefficients. This data frame should includes four columns:
\itemize{
   \item fake: The sequence of \code{var1} (the item whose effect will be conditioned on in the interaction);
   \item coef1: The point estimates of the coefficient of \code{var1} at each break point.
   \item ub: The upper bound of the simulated 95\% CI.
   \item lb: The lower bound of the simulated 95\% CI.
}}

\item{var1}{The name (as a string) of the variable of interest in the interaction term; its conditional coefficient estimates will be plotted.}

\item{var2}{The name (as a string) of the other variable in the interaction term.}

\item{plot}{A logical value indicating whether the output is a plot or a dataframe including the conditional coefficient estimates of var1, their upper and lower bounds, and the corresponding values of var2.}

\item{steps}{Desired length of the sequence. A non-negative number, which for seq and seq.int will be rounded up if fractional. The default is 100 or the unique categories in the \code{var2} (when it is less than 100. Also see \code{\link{unique}}).}

\item{ci}{is a numeric value inherited from the data wrangling functions in this package. Adding it here is just for the method consistency.}

\item{adjCI}{Succeeded from the data management functions in `interplot` package.}

\item{hist}{A logical value indicating if there is a histogram of `var2` added at the bottom of the conditional effect plot.}

\item{var2_dt}{A numerical value indicating the frequency distribution of `var2`. It is only used when `hist == TRUE`. When the object is a model, the default is the distribution of `var2` of the model.}

\item{predPro}{A logical value with default of `FALSE`. When the `m` is an output of a general linear model (class `glm` or `glmerMod`) and the argument is set to `TRUE`, the function will plot predicted probabilities at the values given by `var2_vals`.}

\item{var2_vals}{A numerical value indicating the values the predicted probabilities are estimated, when `predPro` is `TRUE`.}

\item{point}{A logical value determining the format of plot. By default, the function produces a line plot when var2 takes on ten or more distinct values and a point (dot-and-whisker) plot otherwise; option TRUE forces a point plot.}

\item{sims}{Number of independent simulation draws used to calculate upper and lower bounds of coefficient estimates: lower values run faster; higher values produce smoother curves.}

\item{xmin}{A numerical value indicating the minimum value shown of x shown in the graph. Rarely used.}

\item{xmax}{A numerical value indicating the maximum value shown of x shown in the graph. Rarely used.}

\item{ercolor}{A character value indicating the outline color of the whisker or ribbon.}

\item{esize}{A numerical value indicating the size of the whisker or ribbon.}

\item{ralpha}{A numerical value indicating the transparency of the ribbon.}

\item{rfill}{A character value indicating the filling color of the ribbon.}

\item{stats_cp}{A character value indicating what statistics to present as the plot note. Three options are available: "none", "ci", and "ks". The default is "none". See the Details for more information.}

\item{txt_caption}{A character string to add a note for the plot, a value will sending to \code{ggplot2::labs(caption = txt_caption))}.}

\item{ci_diff}{A numerical vector with a pair of values indicating the confidence intervals of the difference between \code{var1} and \code{var2}.}

\item{ks_diff}{A \code{ks.test} object of the effect of \code{var1} conditioned on \code{var2}.}

\item{overlay}{A logical value. When \code{TRUE}, the conditional-effect curves are colored by the grouping column \code{value} in a single panel (used by three-way interaction overlays). The default is \code{FALSE}.}

\item{...}{Other ggplot aesthetics arguments for points in the dot-whisker plot or lines in the line-ribbon plots. Not currently used.}
}
\value{
The function returns a \code{ggplot} object.
}
\description{
Graph based on the data frame of statistics about the conditional effect of an interaction.
}
\details{
\code{interplot.plot} is a S3 method from the \code{interplot}. It generates plots of conditional coefficients.

Because the output function is based on \code{\link[ggplot2]{ggplot}}, any additional arguments and layers supported by \code{ggplot2} can be added with the \code{+}. 

\code{interplot} visualizes the conditional effect based on simulated marginal effects. The simulation provides a probabilistic distribution of moderation effect of the conditioning variable (\code{var2}) at every preset values (including the minimum and maximum values) of the conditioned variable (\code{var1}), denoted as Emin and Emax. This output allows the function to further examine the conditional effect statistically in two ways. One is to examine if the distribution of \eqn{Emax - Emin} covers zero. The other is to directly compare Emin and Emax through statistical tools for distributional comparisons. Users can choose either method by setting the argument \code{stats_cp} to "ci" or "ks".
\itemize{
  \item "ci" provides the confidence interval of the difference of \eqn{Emax - Emin}. An interval including 0 suggests no statistical difference before and after the conditional effect is applied, and vise versa.
  \item "ks" presents the result of a two-sample Kolmogorov-Smirnov test of the simulated distributions of Emin and Emax. The output includes a D statistics and a p-value of the null hypothesis that the two distributions come from the same distribution at the 0.05 level.
}

See an illustration in the package vignette.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Interplot_brms.R
\name{interplot.brmsfit}
\alias{interplot.brmsfit}
\title{Plot Conditional Coefficients in Bayesian Models with Interaction Terms}
\usage{
\method{interplot}{brmsfit}(
  m,
  var1,
  var2,
  plot = TRUE,
  steps = NULL,
  ci = 0.95,
  adjCI = FALSE,
  hist = FALSE,
  var2_dt = NA,
  predPro = FALSE,
  var2_vals = NULL,
  point = FALSE,
  sims = 1000,
  xmin = NA,
  xmax = NA,
  ercolor = NA,
  esize = 0.5,
  ralpha = 0.5,
  rfill = "grey70",
  stats_cp = "none",
  txt_caption = NULL,
  facet_labs = NULL,
  var3 = NULL,
  var3_vals = NULL,
  facet = TRUE,
  ...
)
}
\arguments{
\item{m}{A model object of class \code{brmsfit} including an interaction term.}

\item{var1}{The name (as a string) of the variable of interest in the interaction term; its conditional coefficient estimates will be plotted.}

\item{var2}{The name (as a string) of the other variable in the interaction term.}

\item{plot}{A logical value indicating whether the output is a plot or a dataframe including the conditional coefficient estimates of var1, their upper and lower bounds, and the corresponding values of var2.}

\item{steps}{Desired length of the sequence. A non-negative number, which for seq and seq.int will be rounded up if fractional. The default is 100 or the unique categories in the \code{var2} (when it is less than 100. Also see \code{\link{unique}}).}

\item{ci}{A numeric value defining the credible interval. The default value is 95\% (0.95). For \code{brmsfit} objects the interval summarizes the posterior draws, so it is an (equal-tailed) Bayesian credible interval rather than a frequentist confidence interval.}

\item{adjCI}{The false-discovery-rate adjustment of Esarey and Sumner (2017) is a frequentist correction and does not apply to Bayesian posteriors. The argument is ignored (with a warning) for \code{brmsfit} objects.}

\item{hist}{A logical value indicating if there is a histogram of `var2` added at the bottom of the conditional effect plot.}

\item{var2_dt}{A numerical value indicating the frequency distribution of `var2`. It is only used when `hist == TRUE`. When the object is a model, the default is the distribution of `var2` of the model.}

\item{predPro}{A logical value with default of `FALSE`. When the `m` is fitted with a Bernoulli or binomial family and the argument is set to `TRUE`, the function plots the posterior expected predicted probabilities at the values given by `var2_vals`, computed with \code{\link[brms]{posterior_epred}}.}

\item{var2_vals}{A numerical value indicating the values the predicted probabilities are estimated, when `predPro` is `TRUE`.}

\item{point}{A logical value determining the format of plot. By default, the function produces a line plot when var2 takes on ten or more distinct values and a point (dot-and-whisker) plot otherwise; option TRUE forces a point plot.}

\item{sims}{Ignored for \code{brmsfit} objects: the posterior draws produced by the sampler are used directly instead of re-simulating the coefficients.}

\item{xmin}{A numerical value indicating the minimum value shown of x shown in the graph. Rarely used.}

\item{xmax}{A numerical value indicating the maximum value shown of x shown in the graph. Rarely used.}

\item{ercolor}{A character value indicating the outline color of the whisker or ribbon.}

\item{esize}{A numerical value indicating the size of the whisker or ribbon.}

\item{ralpha}{A numerical value indicating the transparency of the ribbon.}

\item{rfill}{A character value indicating the filling color of the ribbon.}

\item{stats_cp}{A character value indicating what statistics to present as the plot note. Three options are available: "none", "ci", and "ks". The default is "none". See the Details for more information.}

\item{txt_caption}{A character string to add a note for the plot, a value will sending to \code{ggplot2::labs(caption = txt_caption))}.}

\item{facet_labs}{An optional character vector of facet labels to be used when plotting an interaction with a factor variable.}

\item{var3}{An optional name (as a string) of a third variable for a three-way interaction \code{var1 * var2 * var3}. When supplied, the conditional effect of \code{var1} across \code{var2} is shown at several values/levels of \code{var3}. The default \code{NULL} gives the standard two-way behavior. Requires continuous \code{var1} and \code{var2}.}

\item{var3_vals}{An optional numeric vector giving the values of a continuous \code{var3} to condition on. The default is the mean and the mean plus or minus one standard deviation. Ignored when \code{var3} is a factor.}

\item{facet}{A logical value, used only with \code{var3}. \code{TRUE} (default) draws one panel per value/level of \code{var3}; \code{FALSE} overlays the curves colored by \code{var3}.}

\item{...}{Other ggplot aesthetics arguments for points in the dot-whisker plot or lines in the line-ribbon plots. Not currently used.}
}
\value{
The function returns a \code{ggplot} object, or a list with the data frame of conditional coefficients when \code{plot = FALSE}.
}
\description{
\code{interplot.brmsfit} is a method to calculate conditional coefficient
estimates from the posterior draws of a Bayesian regression model fitted with
\code{\link[brms]{brm}} that includes a two-way interaction term.
}
\details{
\code{interplot.brmsfit} is an S3 method of \code{interplot} for models fitted with \code{\link[brms]{brm}}. Unlike the frequentist methods, it does not call \code{arm::sim}: the posterior draws of the population-level (fixed) effects are extracted directly with \code{as.matrix} and the conditional coefficient \eqn{b_{var1} + b_{var1:var2} \cdot var2} is computed for every draw. Point estimates are posterior means and the bounds are posterior quantiles.

Because the output function is based on \code{\link[ggplot2]{ggplot}}, any additional arguments and layers supported by \code{ggplot2} can be added with the \code{+}.

The 'brms' package is only suggested by 'interplot'; it must be installed for this method to run.
}
\examples{
\dontrun{
library(brms)
data(mtcars)

# A Bayesian linear model with a two-way interaction
m_brms <- brm(mpg ~ wt * cyl, data = mtcars, chains = 2, refresh = 0)

# Identical interface; the band is a 95\% posterior credible interval
interplot(m_brms, var1 = "cyl", var2 = "wt")

# Posterior predicted probabilities for a Bernoulli model
m_brms_bin <- brm(am ~ wt * cyl, data = mtcars,
                  family = bernoulli(), chains = 2, refresh = 0)
interplot(m_brms_bin, var1 = "wt", var2 = "cyl",
          predPro = TRUE, var2_vals = c(4, 6, 8))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Interplot_default.R
\name{interplot.default}
\alias{interplot.default}
\title{Plot Conditional Coefficients in (Generalized) Linear Models with Interaction Terms}
\source{
Benjamini, Yoav, and Yosef Hochberg. 1995. "Controlling the False
Discovery Rate: A Practical and Powerful Approach to Multiple Testing".
Journal of the Royal Statistical Society, Series B 57(1): 289--300.

Brambor, Thomas, William Roberts Clark, and Matt Golder.
"Understanding interaction models: Improving empirical analyses". Political
Analysis 14.1 (2006): 63-82.

Esarey, Justin, and Jane Lawrence Sumner. 2015. "Marginal Effects in
Interaction Models: Determining and Controlling the False Positive Rate".
URL: \url{https://jee3.web.rice.edu/interaction-overconfidence.pdf}.
}
\usage{
\method{interplot}{default}(
  m,
  var1,
  var2,
  plot = TRUE,
  steps = NULL,
  ci = 0.95,
  adjCI = FALSE,
  hist = FALSE,
  var2_dt = NA,
  predPro = FALSE,
  var2_vals = NULL,
  point = FALSE,
  sims = 1000,
  xmin = NA,
  xmax = NA,
  ercolor = NA,
  esize = 0.5,
  ralpha = 0.5,
  rfill = "grey70",
  stats_cp = "none",
  txt_caption = NULL,
  facet_labs = NULL,
  var3 = NULL,
  var3_vals = NULL,
  facet = TRUE,
  ...
)
}
\arguments{
\item{m}{A model object including an interaction term, or, alternately, a data frame recording conditional coefficients.}

\item{var1}{The name (as a string) of the variable of interest in the interaction term; its conditional coefficient estimates will be plotted.}

\item{var2}{The name (as a string) of the other variable in the interaction term.}

\item{plot}{A logical value indicating whether the output is a plot or a dataframe including the conditional coefficient estimates of var1, their upper and lower bounds, and the corresponding values of var2.}

\item{steps}{Desired length of the sequence. A non-negative number, which for seq and seq.int will be rounded up if fractional. The default is 100 or the unique categories in the \code{var2} (when it is less than 100. Also see \code{\link{unique}}).}

\item{ci}{A numeric value defining the confidence intervals. The default value is 95\% (0.95).}

\item{adjCI}{A logical value indication if applying the adjustment of confidence intervals to control the false discovery rate following the Esarey and Sumner (2017) procedure. (See also Benjamini and Hochberg 1995.) The default is FALSE; the plot presents the confidence intervals suggested by Brambor, Clark, and Golder (2006).}

\item{hist}{A logical value indicating if there is a histogram of `var2` added at the bottom of the conditional effect plot.}

\item{var2_dt}{A numerical value indicating the frequency distribution of `var2`. It is only used when `hist == TRUE`. When the object is a model, the default is the distribution of `var2` of the model.}

\item{predPro}{A logical value with default of `FALSE`. When the `m` is an object of class `glm` and the argument is set to `TRUE`, the function will plot predicted probabilities at the values given by `var2_vals`.}

\item{var2_vals}{A numerical value indicating the values the predicted probabilities are estimated, when `predPro` is `TRUE`.}

\item{point}{A logical value determining the format of plot. By default, the function produces a line plot when var2 takes on ten or more distinct values and a point (dot-and-whisker) plot otherwise; option TRUE forces a point plot.}

\item{sims}{Number of independent simulation draws used to calculate upper and lower bounds of coefficient estimates: lower values run faster; higher values produce smoother curves.}

\item{xmin}{A numerical value indicating the minimum value shown of x shown in the graph. Rarely used.}

\item{xmax}{A numerical value indicating the maximum value shown of x shown in the graph. Rarely used.}

\item{ercolor}{A character value indicating the outline color of the whisker or ribbon.}

\item{esize}{A numerical value indicating the size of the whisker or ribbon.}

\item{ralpha}{A numerical value indicating the transparency of the ribbon.}

\item{rfill}{A character value indicating the filling color of the ribbon.}

\item{stats_cp}{A character value indicating what statistics to present as the plot note. Three options are available: "none", "ci", and "ks". The default is "none". See the Details for more information.}

\item{txt_caption}{A character string to add a note for the plot, a value will sending to \code{ggplot2::labs(caption = txt_caption))}.}

\item{facet_labs}{An optional character vector of facet labels to be used when plotting an interaction with a factor variable.}

\item{var3}{An optional name (as a string) of a third variable for a three-way interaction \code{var1 * var2 * var3}. When supplied, the conditional effect of \code{var1} across \code{var2} is shown at several values/levels of \code{var3}. The default \code{NULL} gives the standard two-way behavior. Requires continuous \code{var1} and \code{var2}.}

\item{var3_vals}{An optional numeric vector giving the values of a continuous \code{var3} to condition on. The default is the mean and the mean plus or minus one standard deviation. Ignored when \code{var3} is a factor.}

\item{facet}{A logical value, used only with \code{var3}. \code{TRUE} (default) draws one panel per value/level of \code{var3}; \code{FALSE} overlays the curves colored by \code{var3}.}

\item{...}{Other ggplot aesthetics arguments for points in the dot-whisker plot or lines in the line-ribbon plots. Not currently used.}
}
\value{
The function returns a \code{ggplot} object.
}
\description{
\code{interplot.default} is a method to calculate conditional coefficient estimates from the results of (generalized) linear regression models with interaction terms.
}
\details{
\code{interplot.default} is a S3 method from the \code{interplot}. It works on two classes of objects:
\itemize{
  \item Ordinary linear models (object class: \code{lm});
  \item Generalized linear models (object class: \code{glm}).
  }

Because the output function is based on \code{\link[ggplot2]{ggplot}}, any additional arguments and layers supported by \code{ggplot2} can be added with the \code{+}. 

\code{interplot} visualizes the conditional effect based on simulated marginal effects. The simulation provides a probabilistic distribution of moderation effect of the conditioning variable (\code{var2}) at every preset values (including the minimum and maximum values) of the conditioned variable (\code{var1}), denoted as Emin and Emax. This output allows the function to further examine the conditional effect statistically in two ways. One is to examine if the distribution of \eqn{Emax - Emin} covers zero. The other is to directly compare Emin and Emax through statistical tools for distributional comparisons. Users can choose either method by setting the argument \code{stats_cp} to "ci" or "ks".
\itemize{
  \item "ci" provides the confidence interval of the difference of \eqn{Emax - Emin}. An interval including 0 suggests no statistical difference before and after the conditional effect is applied, and vise versa.
  \item "ks" presents the result of a two-sample Kolmogorov-Smirnov test of the simulated distributions of Emin and Emax. The output includes a D statistics and a p-value of the null hypothesis that the two distributions come from the same distribution at the 0.05 level.
}

See an illustration in the package vignette.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/johnson_neyman.R
\name{jn_interval}
\alias{jn_interval}
\title{Compute Johnson-Neyman Interval for Interaction Effects}
\usage{
jn_interval(m, var1, var2, ci = 0.95)
}
\arguments{
\item{m}{A model object (\code{lm}, \code{glm}, \code{lmerMod}, or \code{glmerMod}).}

\item{var1}{The name (as a string) of the variable whose conditional effect is of interest.}

\item{var2}{The name (as a string) of the moderating variable.}

\item{ci}{A numeric value defining the confidence level. The default is 0.95.}
}
\value{
An object of class \code{jn_interval} containing:
\describe{
  \item{bounds}{All Johnson-Neyman bounds (may be 0, 1, or 2 values).}
  \item{bounds_in_range}{Bounds that fall within the observed data range of \code{var2}.}
  \item{var2_range}{The range of the moderating variable in the data.}
  \item{sig_pattern}{One of \code{"always"}, \code{"never"}, \code{"between"}, \code{"outside"}, \code{"below"}, or \code{"above"}, indicating where the effect is statistically significant.}
  \item{ns_regions}{List of \code{(xmin, xmax)} pairs marking non-significant regions.}
}
}
\description{
Identifies the values of the moderating variable at which the
conditional effect of the other variable transitions between
statistical significance and non-significance.
}
\details{
The Johnson-Neyman (JN) technique finds the values of the moderating
variable (\code{var2}) at which the conditional effect of \code{var1} is
exactly at the boundary of statistical significance. This is computed
analytically from the regression coefficients and their variance-covariance
matrix.

For linear mixed-effects models (\code{lmerMod}, \code{glmerMod}), a normal
approximation (z-distribution) is used instead of the t-distribution due
to the controversy over appropriate degrees of freedom.

The function does not support factor variables or quadratic terms (\code{var1 == var2}).
}
\examples{
m <- lm(mpg ~ wt * cyl, data = mtcars)
jn <- jn_interval(m, "cyl", "wt")
print(jn)

# Add JN bounds to an interplot
interplot(m, "cyl", "wt") + jn_layer(jn)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bin_layer.R
\name{bin_layer}
\alias{bin_layer}
\title{Add a Binning Diagnostic to an interplot}
\source{
Hainmueller, Jens, Jonathan Mummolo, and Yiqing Xu. 2019. "How Much
  Should We Trust Estimates from Multiplicative Interaction Models? Simple
  Tools to Improve Empirical Practice." Political Analysis 27(2): 163--192.
}
\usage{
bin_layer(
  m,
  var1,
  var2,
  ci = 0.95,
  bins = 3,
  point_color = "#BD472A",
  point_shape = 18
)
}
\arguments{
\item{m}{A model object of class \code{lm} or \code{glm} including the
interaction of \code{var1} and \code{var2}.}

\item{var1}{The name (as a string) of the variable whose conditional effect is plotted.}

\item{var2}{The name (as a string) of the moderating variable.}

\item{ci}{A numeric value defining the confidence level. The default is 0.95.}

\item{bins}{The number of moderator bins (quantile groups). The default is 3
(low / medium / high terciles).}

\item{point_color}{Color of the binned points and whiskers. Default \code{"#BD472A"}.}

\item{point_shape}{Plotting shape of the binned points. Default \code{18} (filled diamond).}
}
\value{
A list of \code{ggplot2} layers, to be added to an \code{interplot}
  plot with \code{+}.
}
\description{
Overlays within-bin estimates of the conditional effect of \code{var1} on an
\code{interplot} output, following the binning estimator of Hainmueller,
Mummolo, and Xu (2019). It provides a visual check of the linear-interaction-
effect (LIE) assumption: if the moderating relationship is truly linear, the
binned point estimates fall on the interplot line; systematic departures
signal a nonlinear conditional effect.
}
\details{
For each quantile bin of \code{var2}, the model is refitted on the
  observations in that bin with \code{var2} centered at the bin median. The
  coefficient on \code{var1} is then its marginal effect evaluated at the bin
  median, estimated from only that bin's data; this is algebraically the
  Hainmueller-Mummolo-Xu L-estimator. Each estimate is drawn as a
  dot-and-whisker at the bin median.

  Bins with singular or failed fits are dropped with a warning.
}
\examples{
m <- lm(mpg ~ wt * cyl, data = mtcars)
interplot(m, "cyl", "wt") + bin_layer(m, "cyl", "wt")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Interplot.R
\name{interplot}
\alias{interplot}
\title{Plot Conditional Coefficients of a Variable in an Interaction Term}
\source{
Aiken, Leona S., and Stephen G. West. 1991. "Multiple Regression:
Testing and Interpreting Interactions". Newbury Park, CA: Sage.

Benjamini, Yoav, and Yosef Hochberg. 1995. "Controlling the False
Discovery Rate: A Practical and Powerful Approach to Multiple Testing".
Journal of the Royal Statistical Society, Series B 57(1): 289--300.

Brambor, Thomas, William Roberts Clark, and Matt Golder.
"Understanding interaction models: Improving empirical analyses". Political
Analysis 14.1 (2006): 63-82.

Esarey, Justin, and Jane Lawrence Sumner. 2015. "Marginal Effects in
Interaction Models: Determining and Controlling the False Positive Rate".
URL: \url{https://jee3.web.rice.edu/interaction-overconfidence.pdf}.
}
\usage{
interplot(
  m,
  var1,
  var2,
  plot = TRUE,
  steps = NULL,
  ci = 0.95,
  adjCI = FALSE,
  hist = FALSE,
  var2_dt = NA,
  predPro = FALSE,
  var2_vals = NULL,
  point = FALSE,
  sims = 1000,
  xmin = NA,
  xmax = NA,
  ercolor = NA,
  esize = 0.5,
  ralpha = 0.5,
  rfill = "grey70",
  stats_cp = "none",
  txt_caption = NULL,
  ...
)
}
\arguments{
\item{m}{A model object including an interaction term, or, alternately, a data frame generated by an earlier call to interplot using the argument plot = FALSE.}

\item{var1}{The name (as a string) of the variable of interest in the interaction term; its conditional coefficient estimates will be plotted.}

\item{var2}{The name (as a string) of the other variable in the interaction term}

\item{plot}{A logical value indicating whether the output is a plot or a dataframe including the conditional coefficient estimates of var1, their upper and lower bounds, and the corresponding values of var2.}

\item{steps}{Desired length of the sequence. A non-negative number, which for seq and seq.int will be rounded up if fractional. The default is 100 or the unique categories in the \code{var2} (when it is less than 100. Also see \code{\link{unique}}).}

\item{ci}{A numeric value defining the confidence intervals. The default value is 95\% (0.95).}

\item{adjCI}{A logical value indication if applying the adjustment of confidence intervals to control the false discovery rate following the Esarey and Sumner (2017) procedure. (See also Benjamini and Hochberg 1995.) The default is FALSE; the plot presents the confidence intervals suggested by Brambor, Clark, and Golder (2006). The functions dealing with multilevel model outputs in this package do not equip with this argument, because there is the controversy on the accurate degrees of freedom for multilevel models, esp. when random effect is engaged and the degrees of freedom is a necessary information to conduct the CI adjustment. See e.g., https://stat.ethz.ch/pipermail/r-help/2006-May/094765.html and https://stat.ethz.ch/pipermail/r-sig-mixed-models/2008q1/000517.html.}

\item{hist}{A logical value indicating if there is a histogram of `var2` added at the bottom of the conditional effect plot.}

\item{var2_dt}{A numerical value indicating the frequency distribution of `var2`. It is only used when `hist == TRUE`. When the object is a model, the default is the distribution of `var2` of the model.}

\item{predPro}{A logical value with default of `FALSE`. When the `m` is an object of class `glm` or `glmerMod` and the argument is set to `TRUE`, the function will plot predicted probabilities at the values given by `var2_vals`.}

\item{var2_vals}{A numerical value indicating the values the predicted probabilities are estimated, when `predPro` is `TRUE`.}

\item{point}{A logical value determining the format of plot. By default, the function produces a line plot when var2 takes on ten or more distinct values and a point (dot-and-whisker) plot otherwise; option TRUE forces a point plot.}

\item{sims}{Number of independent simulation draws used to calculate upper and lower bounds of coefficient estimates: lower values run faster; higher values produce smoother curves.}

\item{xmin}{A numerical value indicating the minimum value shown of x shown in the graph. Rarely used.}

\item{xmax}{A numerical value indicating the maximum value shown of x shown in the graph. Rarely used.}

\item{ercolor}{A character value indicating the outline color of the whisker or ribbon.}

\item{esize}{A numerical value indicating the size of the whisker or ribbon.}

\item{ralpha}{A numerical value indicating the transparency of the ribbon.}

\item{rfill}{A character value indicating the filling color of the ribbon.}

\item{stats_cp}{A character value indicating what statistics to present as the plot note. Three options are available: "none", "ci", and "ks". The default is "none". See the Details for more information.}

\item{txt_caption}{A character string to add a note for the plot, a value will sending to \code{ggplot2::labs(caption = txt_caption))}.}

\item{...}{Additional arguments passed to the specific \code{interplot} method. For \code{lm} and \code{glm} objects (see \code{\link{interplot.default}}) these include \code{var3}, \code{var3_vals}, and \code{facet} for three-way interactions. Other ggplot aesthetics arguments for points or lines may also be supplied.}
}
\value{
The function returns a \code{ggplot} object.
}
\description{
\code{interplot} is a generic function to produce a plot of the coefficient estimates of one variable in a two-way interaction conditional on the values of the other variable in the interaction term. The function invokes particular \code{methods} which depend on the \code{\link{class}} of the first argument.
}
\details{
\code{interplot} visualizes the changes in the coefficient of one term in a two-way interaction conditioned by the other term. In the current version, the function works with interactions in the following classes of models:
\itemize{
  \item Ordinary linear models (object class: \code{lm});
  \item Generalized linear models (object class: \code{glm});
  \item Linear mixed-effects models (object class: \code{lmerMod});
  \item Generalized linear mixed-effects models (object class: \code{glmerMod});
  \item Ordinary linear models with imputed data (object class: \code{list});
  \item Generalized linear models with imputed data (object class: \code{list})
  \item Linear mixed-effects models with imputed data (object class: \code{list});
  \item Generalized linear mixed-effects models with imputed data (object class: \code{list}).
}

For \code{lm} and \code{glm} objects, \code{interplot} additionally supports three-way interactions (supply a third variable via \code{var3}; see \code{\link{interplot.default}}) and nonlinear conditional effects, in which \code{var1} interacts with a polynomial (e.g. \code{I(var2^2)}) or spline (e.g. \code{splines::ns(var2)}) of the moderator. The companion function \code{\link{bin_layer}} adds the Hainmueller, Mummolo, and Xu (2019) binning diagnostic as a composable overlay.

The examples below illustrate how methods invoked by this generic deal with different type of objects.

Because the output function is based on \code{\link[ggplot2]{ggplot}}, any additional arguments and layers supported by \code{ggplot2} can be added with the \code{+}. 

\code{interplot} visualizes the conditional effect based on simulated marginal effects. The simulation provides a probabilistic distribution of moderation effect of the conditioning variable (\code{var2}) at every preset values (including the minimum and maximum values) of the conditioned variable (\code{var1}), denoted as Emin and Emax. This output allows the function to further examine the conditional effect statistically in two ways. One is to examine if the distribution of \eqn{Emax - Emin} covers zero. The other is to directly compare Emin and Emax through statistical tools for distributional comparisons. Users can choose either method by setting the argument \code{stats_cp} to "ci" or "ks".
\itemize{
  \item "ci" provides the confidence interval of the difference of \eqn{Emax - Emin}. An interval including 0 suggests no statistical difference before and after the conditional effect is applied, and vise versa.
  \item "ks" presents the result of a two-sample Kolmogorov-Smirnov test of the simulated distributions of Emin and Emax. The output includes a D statistics and a p-value of the null hypothesis that the two distributions come from the same distribution at the 0.05 level.
}

See an illustration in the package vignette.
}
\examples{
data(mtcars)
m_cyl <- lm(mpg ~ wt * cyl, data = mtcars)
library(interplot)

# Plot interactions with a continous conditioning variable
interplot(m = m_cyl, var1 = 'cyl', var2 = 'wt') +
xlab('Automobile Weight (thousands lbs)') +
ylab('Estimated Coefficient for Number of Cylinders') +
ggtitle('Estimated Coefficient of Engine Cylinders\non Mileage by Automobile Weight') +
theme(plot.title = element_text(face='bold'))


# Plot interactions with a categorical conditioning variable
interplot(m = m_cyl, var1 = 'wt', var2 = 'cyl') +
xlab('Number of Cylinders') +
ylab('Estimated Coefficient for Automobile Weight (thousands lbs)') +
ggtitle('Estimated Coefficient of Automobile Weight \non Mileage by Engine Cylinders') +
theme(plot.title = element_text(face='bold'))

# Three-way interaction: effect of wt across hp, faceted by cyl
m_3 <- lm(mpg ~ wt * hp * cyl, data = mtcars)
interplot(m_3, var1 = 'wt', var2 = 'hp', var3 = 'cyl')

# Nonlinear conditional effect: wt interacted with a quadratic in cyl
m_q <- lm(mpg ~ wt * (cyl + I(cyl^2)), data = mtcars)
interplot(m_q, var1 = 'wt', var2 = 'cyl')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/johnson_neyman.R
\name{jn_layer}
\alias{jn_layer}
\title{Add Johnson-Neyman Bounds to an interplot}
\usage{
jn_layer(
  jn,
  line_color = "red",
  linetype = "dashed",
  shade_color = "grey80",
  shade_alpha = 0.15,
  label = TRUE
)
}
\arguments{
\item{jn}{A \code{jn_interval} object produced by \code{\link{jn_interval}}.}

\item{line_color}{Color of the boundary lines. Default \code{"red"}.}

\item{linetype}{Line type for boundaries. Default \code{"dashed"}.}

\item{shade_color}{Fill color for non-significant regions. Default \code{"grey80"}.}

\item{shade_alpha}{Transparency of shading. Default \code{0.15}.}

\item{label}{Logical; if \code{TRUE} (default), annotate the JN bound values.}
}
\value{
A list of \code{ggplot2} layers.
}
\description{
Returns a list of \code{ggplot2} layers that overlay Johnson-Neyman significance
boundaries on an \code{interplot} output. Add to any interplot with \code{+}.
}
\examples{
m <- lm(mpg ~ wt * cyl, data = mtcars)
jn <- jn_interval(m, "cyl", "wt")
interplot(m, "cyl", "wt") + jn_layer(jn)

}
